/**
 *
 *   Equations.h
 *
 *   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _EQUATIONS_H_
#define _EQUATIONS_H_

#include "Global.h"

/// abstract class

class Equations{
    
public:
    
    // destructor
    
    virtual ~Equations(){}
    
    // functions
    
    virtual unsigned short int GetNumberOfEquations(void) = 0;
    virtual string GetName(unsigned short int aNumberOfEquation) = 0;
    virtual void LoadRHS(const double aTau,
                         Vector3D *** const aVArr,
                         const usi aArg,
                         const usi aTo = 2
                         ) = 0;
    virtual void SetEquationOfState(EquationOfState *aEquationOfState) = 0;
    virtual void PrintInfo(void) = 0;
    virtual double FormatOutput(double aX, int aN) = 0;
    virtual double FormatInput(double aX, int aN) = 0;
    
    virtual double Geteeff(int i, int j, int k) = 0;
    virtual double GetTeff(double x, double y, double e) = 0;
    virtual double GetTeffpt(int i, int j, int k) = 0;
    
    virtual void EnforceLandauMatching(Vector3D *** const aVArr) = 0;
    
};
#endif // _EQUATIONS_H_
