/** 
*
*   GlauberInspiredInitialSource.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _GLAUBER_INSPIRED_INITIAL_SOURCE_H_
#define _GLAUBER_INSPIRED_INITIAL_SOURCE_H_

#include "Global.h"

class GlauberInspiredInitialSource : public InitialSource {

    public:

        // constructor

        GlauberInspiredInitialSource(Configurator *aConfigurator, EquationOfState *aEquationOfState, Grid *aGrid);

        // destructor

        ~GlauberInspiredInitialSource();

        // functions

        virtual double GetValue(int i, int j, int k);
        virtual void    PrintInfo(void);

    private:

        double GetSource2(OpticalGlauber* mGlau, double x, double y, double z);
        double TiltedSource2(OpticalGlauber* mGlau, double x, double y, double z);
        double ShiftedSource2(OpticalGlauber* mGlau, double x, double y, double z);
        double GetSource(int i, int j, int k);
        double TiltedSource(int i, int j, int k);
        double ShiftedSource(int i, int j, int k);

        // tilted source

        inline double FunctionF(double e);
        inline double SetParticipantRapidity(double aCollEn);

        // shifted source

        inline double GetRapidityShift(OpticalGlauber *aGlau, double x, double y);
        inline double SetParticipantVelocity(double aCollEn);

        // both

        inline double PseudorapidtyProfile(double z);

        Configurator *mConfigurator;
        EquationOfState *mEOS;
        Grid *mGrid;

        OpticalGlauber *mGlau;
        OpticalGlauber *mGlauC;

        int mSourcesScalingChoice;
        int mSourcesNormalizationChoice;
        int mEquationsChoice;
        int mInitialSourceChoice;

        string mInitialSourceDesc;
        string mSourcesNormalizationDesc;
        string mSourcesScalingDesc;

        double (GlauberInspiredInitialSource::*aSourceFunction)(int i, int j, int k);
        double (GlauberInspiredInitialSource::*aSourceFunction2)(OpticalGlauber* mGlau, double x, double y, double z);

        double mParticipantV;
        double mParticipantR;
        double mScale;
        double mNormalizationFactor;    // initial central temperature
        double mPlateauWidth;           // plateau half width
        double mGaussianWidth;          // gaussian tails half width
        double mMix;                     // mixing factor
        double mCollEn;                  // collision energy [GeV]
        double mRapCorr;                 // range of rapidity correlations
        double mShift;                  // shift in rapidity


        // arrays are used in order to speed-up the calculation

        double** AllocateArray2D(int aChoice);
        void FreeArray2D(double **aArray2D);

        double **WNAArr;
        double **WNBArr;
        double **BCArr;
        double **RSArr;

};

#endif // _GLAUBER_INSPIRED_INITIAL_SOURCE_H_
