/** 
*
*   Grid.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _GRID_H_
#define _GRID_H_

#include "Global.h"

class Grid {

    public:

        Grid(Configurator *aConfigurator);
        ~Grid();

        // in order to share this class with other classes the inline functions have to
        // be DEFINED inside the *.h file because then the inline functions are known and can be "incorporated inside the code"


        void    PrintInfo(void);

        inline double GetXMax(void);
        inline double GetYMax(void);
        inline double GetZMax(void);
        inline double GetTMax(void);

        inline double GetXMin(void);
        inline double GetYMin(void);
        inline double GetZMin(void);
        inline double GetTMin(void);

        inline double GetX(unsigned short int i);
        inline double GetY(unsigned short int j);
        inline double GetZ(unsigned short int k);
        inline double GetT(unsigned short int t);

        inline double GetdX(void);
        inline double GetdY(void);
        inline double GetdZ(void);
        inline double GetdT(void);

        inline unsigned short int GetXP(void);
        inline unsigned short int GetYP(void);
        inline unsigned short int GetZP(void);
        inline unsigned short int GetTP(void);

    private :

		double mdX;
		double mdY;
		double mdZ;
        double mdT;

		double mXMin;
		double mYMin;
		double mZMin;
        double mTMin;

		double mXMax;
		double mYMax;
		double mZMax;
        double mTMax;

		unsigned short int mXPts;
		unsigned short int mYPts;
		unsigned short int mZPts;
		unsigned short int mTPts;

		// optimization

        double mFacX;
        double mFacY;
        double mFacZ;

};

inline double Grid::GetXMax(void){

    return mXMax;

}

inline double Grid::GetYMax(void){

    return mYMax;

}

inline double Grid::GetZMax(void){

    return mZMax;

}

inline double Grid::GetTMax(void){

    return mTMax;

}

inline double Grid::GetXMin(void){

    return mXMin;

}

inline double Grid::GetYMin(void){

    return mYMin;

}

inline double Grid::GetZMin(void){

    return mZMin;

}

inline double Grid::GetTMin(void){

    return mTMin;

}

inline double Grid::GetX(unsigned short int i){

    return (i - mFacX) * mdX;

}

inline double Grid::GetY(unsigned short int j){

    return (j - mFacY) * mdY;

}

inline double Grid::GetZ(unsigned short int k){

    return (k - mFacZ) * mdZ;

}

inline double Grid::GetT(unsigned short int t){

    return mTMin + mdT * t;

}

inline double Grid::GetdX(void){

    return mdX;

}

inline double Grid::GetdY(void){

    return mdY;

}

inline double Grid::GetdZ(void){

    return mdZ;

}

inline double Grid::GetdT(void){

    return mdT;

}

inline unsigned short int Grid::GetXP(void){

    return mXPts;

}

inline unsigned short int Grid::GetYP(void){

    return mYPts;

}

inline unsigned short int Grid::GetZP(void){

    return mZPts;

}

inline unsigned short int Grid::GetTP(void){

    return mTPts;

}

#endif // _GRID_H_

