/**
 *
 *   Hfuncs.h
 *
 *   2017 Mubarak Alqahtani, Mohammad Nopoush, Radoslaw Ryblewski, and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _HFUNCS_H_
#define _HFUNCS_H_

#define USE_INTERPOLATIONS 1

#include "Global.h"
#include "Pintegrals.h"

#define Nt 0.20725784867432664 // N tilde

class Hfuncs {
    
public:
    
    // constructor
    
    Hfuncs();
    
    // destructor
    
    ~Hfuncs();
    
    // functions

    double H3(double ax, double ay, double az, double m);
    double d1H3(double ax, double ay, double az, double m);
    double d2H3(double ax, double ay, double az, double m);
    double d3H3(double ax, double ay, double az, double m);
    double d4H3(double ax, double ay, double az, double m);
    

    double H3X(double ax, double ay, double az, double m);
    double d1H3X(double ax, double ay, double az, double m);
    double d2H3X(double ax, double ay, double az, double m);
    double d3H3X(double ax, double ay, double az, double m);
    double d4H3X(double ax, double ay, double az, double m);
    
    double H3Y(double ax, double ay, double az, double m);
    double d1H3Y(double ax, double ay, double az, double m);
    double d2H3Y(double ax, double ay, double az, double m);
    double d3H3Y(double ax, double ay, double az, double m);
    double d4H3Y(double ax, double ay, double az, double m);
    
    double H3L(double ax, double ay, double az, double m);
    double d1H3L(double ax, double ay, double az, double m);
    double d2H3L(double ax, double ay, double az, double m);
    double d3H3L(double ax, double ay, double az, double m);
    double d4H3L(double ax, double ay, double az, double m);
    
    double H3B(double ax, double ay, double az, double m);
    
    void loadInterpolations();
    void freeInterpolations();
    
private:
    
    Pintegrals pints;
    
    double genI3(double ax, double ay, double az, double m, double delta0);
    double I3_1(double ax, double ay, double az, double m);
    double I3_2(double ax, double ay, double az, double m);
    double H3exp(double ax, double ay, double az, double m);
    
    double genI31(double ax, double ay, double az, double m, double delta0);
    double I31_1(double ax, double ay, double az, double m);
    double I31_2(double ax, double ay, double az, double m);
    double genI3m(double ax, double ay, double az, double m, double delta0);
    
    double I3m_1(double ax, double ay, double az, double m);
    double I3m_2(double ax, double ay, double az, double m);
    double d1H3exp(double ax, double ay, double az, double m);
    double d4H3exp(double ax, double ay, double az, double m);
    
    double genI3X(double ax, double ay, double az, double m, double delta0);
    double I3X_1(double ax, double ay, double az, double m);
    double I3X_2(double ax, double ay, double az, double m);
    double H3Xexp(double ax, double ay, double az, double m);
    
    double genI3X1(double ax, double ay, double az, double m, double delta0);
    double I3X1_1(double ax, double ay, double az, double m);
    double I3X1_2(double ax, double ay, double az, double m);
    
    double genI3X2(double ax, double ay, double az, double m, double delta0);
    double I3X2_1(double ax, double ay, double az, double m);
    double I3X2_2(double ax, double ay, double az, double m);
    
    double genI3Xm(double ax, double ay, double az, double m, double delta0);
    double I3Xm_1(double ax, double ay, double az, double m);
    double I3Xm_2(double ax, double ay, double az, double m);
    
    double d1H3Xexp(double ax, double ay, double az, double m);
    double d2H3Xexp(double ax, double ay, double az, double m);
    double d4H3Xexp(double ax, double ay, double az, double m);
    
};

#endif // _HFUNCS_H_
