/** 
*
*   IdealEquationOfState.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _IDEAL_EQUATION_OF_STATE_H_
#define _IDEAL_EQUATION_OF_STATE_H_

#include "Global.h"

class IdealEquationOfState : public EquationOfState{

    public:

        // constructor

        IdealEquationOfState(int aStatistics = 1, double aDegeneracy = 0, int aQuarkFlavours = 2);

        // destructor

        virtual  ~IdealEquationOfState();

        // functions

        virtual double CsT(double T);
        virtual double TP(double P);
        virtual double TE(double E);
        virtual double TS(double S);

        virtual double ET(double T);
        virtual double ST(double T);
        virtual double PT(double T);

        virtual double EpT(double T);
        virtual double SpT(double T);

        virtual void PrintInfo(void);

    private:

        // function returning multiplier depending on statistics type

        inline double* GetMultiplier(void);

        // degeneracy factor

        double mDegeneracy;

        // else predefined values are used

        const int mGluonColors;
        const int mColors;
        const int mSpin;
        const int mAntiParticle_Particle;
        int mQuarkFlavours;

        // type of the statistics

        int mStatistics;

        // multipliers present in the expressions for thermodynamic variables

        double mMultiplierB;
        double mMultiplierBE;
        double mMultiplierFD;
        double mM;
        string mStatisticsDesc;

        // user defined

        bool BuiltIn;

};
#endif // _IDEAL_EQUATION_OF_STATE_H_
