/**
*
*   ParametrizationFreezer.h
*
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef _PARAMETRIZATION_FREEZER_H_
#define _PARAMETRIZATION_FREEZER_H_

#include "Global.h"

class ParametrizationFreezer : public Freezer {

    public:

        // constructor

        ParametrizationFreezer(Configurator *aConfigurator, string aBINDir);

        // destructor

        ~ParametrizationFreezer();

        // functions

        void    ExtractFOHS(void);
        virtual void PrintInfo(void);

    private:

        void ExtractFOHSPerfectFluid(void);
        void ExtractFOHSAnisotropicFluid(void);
        void ExtractFOHSAnisotropicFluidNew(void);
        double R(double x);

        // ExportVector3D may be used to output data for testing
        void ExportAllVectors3D(void);
        void ExportVector3D(string aVectorName, Vector3D *aVec, double aMultip);

        Configurator    *mConfigurator;
        Grid            *mGrid;
        Vector4D        *mField;
        Vector4D        *mFieldX;
        //Vector4D        *mFieldY;
        //Vector4D        *mFieldZ;
        Vector3D        **mVArr;

    	EquationOfState 	*mEOS;
        EquationOfState* GetEquationOfState();
        double meFO;

        Hfuncs hfuncs;

        int mEquationsChoice;
        int mNumberOfEquations;

        string *mNames;
        string *mNamesFO;
        string *mUnitsFO;

        string mTSVDir;
        string mXMLDir;
        string mBINDir;
        string mHydroID;

        // field domain and grid spacing

        double tmin, tmax, xmax, ymax, emax;
        double deltat, deltax, deltay, deltae;

        // freeze-out options

        double  TFO;
        double  lambda;

        // parametrization points

        int NZ, NP, NT;

        // parametrization angles ranges

        double zetamin, zetamax, phimin, phimax, thetamin, thetamax;

        // parametrization spacing

        double dzeta, dphi, dtheta;

        // error

        double epsilon;

        double f1(double d, double p, double z, double t);
        double f2(double d, double p, double z, double t);
        double f3(double d, double p, double z, double t);
        inline double FindRoot(double (ParametrizationFreezer::*aFunction)(double, double, double, double),
                                double p, double z, double t,
                                int MaxIterations, double xmin, double xmax, double eps);

        // angles at which d is maximal

        inline double phi0(void);
        inline double zeta0(double phi);
        inline double theta0(double phi, double zeta);

        // maximal values of d

        inline double dmaxp(double phi);
        inline double dmaxz(double phi, double zeta);
        inline double dmaxt(double phi, double zeta, double theta);

        // tau, x, y, eta parametization by (d, zeta, phi, theta)

        double  tau(double d, double zeta, double phi, double theta);
        double  x  (double d, double zeta, double phi, double theta);
        double  y  (double d, double zeta, double phi, double theta);
        double  eta(double d, double zeta, double phi, double theta);

        // description of the .xml file

        // export data to THERMINATOR .xml file

        void SetDescriptions(void);

        string mEquations;
        string mEquationOfState;
        string mDeviceType;
        string mCollisionalSystem;
        string mCMin;
        string mCMax;
        string mInitialSource;
        string mDensityScaling;
        string mDensityNormalization;
        string mUxInitialFlow;
        string mUyInitialFlow;
        string mRInitialFlow;
        string mIdealStatistics;

        void    ExportFOHSXML(string aFileName);
        void    ExportVector3DXml(ofstream &aFile, Vector3D* const aVec);
        void    GetDATA(string Tab,ofstream &aFile, Vector3D* const aVec, long long int aPts);
        void    ExportVector(string Tab, ofstream &aFile, int n);
        string  GetAXIS(string Tab, string aAxisName, double aMin, double aMax, int aPts, string aUnit);
        string  GetPARAMETERS(string Tab);
        string  GetDESCRIPTION(string Tab);
        string  GetTAG(string aTagName, string aDetailName, string aUnit, double aValue);
        string  GetTAG(string aTagName, string aDetailName, string aUnit, string aValue);
        string  GetTAG(string aTagName, string aDetailName, string aUnit, int aValue);


};
#endif //_PARAMETRIZATION_FREEZER_H_
