/** 
*
*   PerfectFluidEquations.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _PERFECT_FLUID_EQUATIONS_H_
#define _PERFECT_FLUID_EQUATIONS_H_

#include "Global.h"

class PerfectFluidEquations : public Equations{

    public:

        // constructor

        PerfectFluidEquations(Configurator *aConfigurator, Grid *aGrid);

        // destructor

        virtual          ~PerfectFluidEquations();

        // functions

        virtual usi          GetNumberOfEquations(void);
        virtual string      GetName(usi aNumberOfEquation);
        virtual void        LoadRHS(const double aTau, Vector3D *** const aVArr, const usi aArg, const usi aTo = 2);
        virtual void        SetEquationOfState(EquationOfState *aEquationOfState);
        virtual void        PrintInfo(void);
        virtual double      FormatOutput(double aX, int aN);
        virtual double      FormatInput(double aX, int aN);
        virtual double      Geteeff(int i, int j, int k) { return 1e6; }
    	virtual double      GetTeff(double x, double y, double e) { return 1e6;}
        virtual double      GetTeffpt(int i, int j, int k) { return 1e6;}
    
        virtual void EnforceLandauMatching(Vector3D *** const aVArr) {}
    
    private:

        // member functions

        void             SetVariables(const double aTau, const usi i, const usi j, const usi k, Vector3D * const aVec0, Vector3D * const aVec1,Vector3D * const aVec2,Vector3D * const aVec3);
        double           GetRHSValue(const usi aEquation);
        inline double   Dx(Vector3D * const aVec, const usi i, const usi j, const usi k);
        inline double   Dy(Vector3D * const aVec, const usi i, const usi j, const usi k);
        inline double   Dz(Vector3D * const aVec, const usi i, const usi j, const usi k);

        inline void     CheckValues( Vector3D * const tVec0,
                                                Vector3D * const tVec1,
                                                Vector3D * const tVec2,
                                                Vector3D * const tVec3, int i, int j, int k);
        // member variables

        Vector3D ***mVArr;
        static const usi mNumberOfEquations = 4;
        string *mNames;
        EquationOfState *mEquationOfState;

        // for optimization

        Grid * const mGrid;

        const usi mXP;
        const usi mYP;
        const usi mZP;

        const double mdX;
        const double mdY;
        const double mdZ;

        // optimization for derivatives

        const double mdXi;
        const double mdYi;
        const double mdZi;

        const double m2dX;
        const double m2dY;
        const double m2dZ;

        const double m2dXi;
        const double m2dYi;
        const double m2dZi;

        const usi mXPm1;
        const usi mYPm1;
        const usi mZPm1;

        // definitions of some functions appearing in equations

        inline double Cs(const double aX);

        // optimization of equations

        double DFDz, DFDx, DFDy;
        double DRDz, DRDx, DRDy;
        double DUxDz, DUxDx, DUxDy;
        double DUyDz, DUyDx, DUyDy;

        double Eta;
        double Tau;

        double F, R, Ux, Uy;

        double U0, U02, Y;
        double UP, UP2;
        double Ux2, Ux3, Ux4;
        double Uy2, Uy3, Uy4;

        double SinhEmR;
        double CoshEmR;
        double SechEmR;
        double TanhEmR;

        double Sinh2EmR;
        double Cosh2EmR;

        double CoshEmR2;
        double SinhEmR2;

        double Cs2m1;
        double Cs2;
        double TU0, TU02, T2, Ux2p1;

};
#endif // _PERFECT_FLUID_EQUATIONS_H_
