/**
 *
 *   pintegrals.h
 *
 *   Copyright (C) 2015 Michael Strickland and Mubarak Alqahtani
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _PINTEGRALS_H_
#define _PINTEGRALS_H_

#include "Global.h"

#define Power(a,b) pow((double)a,(double)b)
#define Sqrt(a) sqrt((double)a)



#include <iostream>
#include <iomanip>
#include <cmath>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>
#include <vector>

#define intTOLERANCE 1.0e-6
#define intNPTS 200

#define MMIN 1e-3
#define MMAX 20+MMIN
#define INTPTS 400

#define EXPORDER 13 // should be nominal exporder from mathematica notebook + 1

struct functionWrapperPint {
    double (* function) (double x, void * params);
};

struct integralParamsPint {
    double n, m, delta0;
};

struct interpolationData {
    gsl_interp_accel *a;
    gsl_spline *s;
};


class Pintegrals {
    
public:
    
    // constructor
    
    Pintegrals();
    
    // destructor
    
    ~Pintegrals();
    
    // functions
    

    double pInt(double n, double m, double d0);
    double pmInt(double n, double m, double d0);

    void freeInterpolations();
    void loadInterpolations();

private:
    
    vector<interpolationData> pintDataVector; // for pInt
    vector<interpolationData> pmintDataVector; // for pmInt
   
    void loadpmInterpolation(int region, double n);
    void loadpInterpolation(int region, double n);
    
    static double integrate(functionWrapperPint * fw, double n, double m, double delta0);
    static double pInt_integrand(double p, void *params);
    static double pmInt_integrand(double p, void *params);
    static double pIntegral(double n, double m, double delta0);
    static double pmIntegral(double n, double m, double delta0);
    
};

#endif // _PINTEGRALS_H_
