/** 
*
*   RealisticEquationOfState.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _REALISTIC_EQUATION_OF_STATE_H_
#define _REALISTIC_EQUATION_OF_STATE_H_

#include "Global.h"

class RealisticEquationOfState : public EquationOfState{

    public:

        // constructor

        RealisticEquationOfState(string aDirectory);

        // destructor

        virtual  ~RealisticEquationOfState();

        // functions
        // return values in fm
        // take arguments in fm

        virtual double ET(double T);
        virtual double PT(double T);
        virtual double ST(double T);

        virtual double TE(double E);
        virtual double TP(double P);
        virtual double TS(double S);

        virtual double EpT(double T);
        virtual double SpT(double T);

        // speed of sound constructed by C++ code

        virtual double CsT(double T);

        virtual void PrintInfo(void);

	private:

        Vector1D **mVecs;
        const int mN;  // number of vectors constructed Parser
        string *mNames;

        // optimization, minimal and maximal values in fm

        double Tmin;
        double Tmax;
        double Emin;
        double Emax;
        double Pmin;
        double Pmax;
        double Smin;
        double Smax;
        double Csmin;
        double Csmax;
        double Epmin;
        double Epmax;
        double Spmin;
        double Spmax;

};
#endif // _REALISTIC_EQUATION_OF_STATE_H_
