/**
 *
 *   RelaxationTime.h
 *
 *   Copyright (C) 2015 Michael Strickland and Mubarak Alqahtani
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _RELAXATIONTIME_H_
#define _RELAXATIONTIME_H_

#include "Global.h"
#include "RealisticEquationOfState.h"
#include "ThermalMass.h"
#include "Hfuncs.h"
#include "LinearTransportCoefficient.h"

#define INT_TOLERANCE 1.0e-4
#define NPTS 200

#include <iostream>
#include <iomanip>
#include <cmath>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_sf_bessel.h>
#include <gsl/gsl_roots.h>


struct integralParamsteq {
    double x;
};

struct functionWrapperteq {
    double (* function) (double y, void * params);
};

// natural log grid
#define LTMIN -6
#define LTMAX 2.2
#define LTPTS 300

class RelaxationTime {
    
public:
    
    // constructor
    
    RelaxationTime();
    
    // destructor
    
    ~RelaxationTime();
    
    // functions

	double teq(double T);
    void loadInterpolation();
    void freeInterpolation();
    
    void setup(Hfuncs aHfuncs, ThermalMass aMass, LinearTransportCoefficient aTransportCoeff);
    
private:
    
    ThermalMass mass;
    Hfuncs hfuncs;
    //double eb;
    LinearTransportCoefficient etabar;
    
    double teqEval(double T);

    
    double L_0(double x);
    double L_1(double x);
    double L_m1(double x);
    double kappa(double x);
    
	static double L_0_integrand(double t, void *params);
	static double L_1_integrand(double t, void *params);
	static double integrateteq(functionWrapperteq * fw, double x);

    gsl_interp_accel *mAcc;
    gsl_spline *mSpline;
    
    double LT[LTPTS];
    double LTEQ[LTPTS];
    
};

#endif // _RELAXATIONTIME_H_
