/** 
*
*   RungeKuttaStepper.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _RUNGE_KUTTA_STEPPER_H_
#define _RUNGE_KUTTA_STEPPER_H_

#include "Global.h"

class RungeKuttaStepper : public Stepper {

    public:

        // constructor

        RungeKuttaStepper(Configurator *aConfigurator, Grid *aGrid);

        // destructor

        virtual ~RungeKuttaStepper();

        // functions

        virtual void    MakeTimeStep(const double aTau);
        virtual void    AllocateMemory(Equations *aEquations);
        virtual void    Initialize(Initializer **aIArr);
        virtual void    FreeMemory(void);
        virtual Vector3D *** GetArrays(void);
        virtual usi     GetNumberOfVectors(void);
        virtual void    PrintInfo(void);

    private:

        inline void     CopyArrays(const usi aFrom, const usi aTo);
        inline void     SwapArrays(const usi aArrA, const usi aArrB);
        void             MakeStep(  const double aTau,
                                    const usi aArg,
                                    const usi aC1,
                                    const usi aC2,
                                    const bool aLoad);

        Vector3D ***mVArr;
        static const usi mNumberOfVectors = 4;
        Equations *mEquations;
        Grid *mGrid;

        // optimization variables

        const usi mXP;
        const usi mYP;
        const usi mZP;

        const double mdT;
};
#endif //_RUNGE_KUTTA_STEPPER_H_
