/**
 *
 *   Solver.h
 *
 *   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _SOLVER_H_
#define _SOLVER_H_

#include "Global.h"

// define structure used for memory usage container

struct MemoryUsed{
    
    double tMemoryUsedBIN;
    double tMemoryUsedTSV;
    
};

class Solver {
    
public:
    
    // constructor
    
    Solver(Configurator *aConfigurator);
    
    // destructor
    
    ~Solver();
    
    // functions
    
    string              SolveEquations(void);
    
private:
    
    // export data
    
    void                PrintMonitorHeader(void);
    MemoryUsed          ExportMeasurables(MemoryUsed tMem, int i, double time);
    void                ExportSnapshot(MemoryUsed &aMem, const double aTau);
    double              ExportSnapshotTSV(Vector3D ***aArr, double aTau);
    double              ExportSnapshotBIN(Vector3D ***aArr, double aTau);
    double              ExportArrayTSV(Vector3D *aArr, double aTau, int aN);
    double              ExportArrayBIN(Vector3D *aArr, double aTau, int aN);
    
    double*             GetTimeDifference(time_t aTimeStart, time_t aTimeStop, double aMult = 1);
    
    // checks values of the parameters
    
    void                CheckParameters(void);
    void                SetParameters(void);
    
    EquationOfState*    GetEquationOfState(void);
    Equations*          GetEquations(void);
    Stepper*            GetStepper(void);
    Smoother*           GetSmoother(void);
    Initializer **      GetInitializerArr(void);
    Initializer*        GetInitializer(string aName);
    
    InitialRapidity*    GetInitialRapidity();
    InitialFlowY*       GetInitialFlowY();
    InitialFlowX*       GetInitialFlowX();
    InitialSource*      GetInitialSource();
    InitialAnisotropy*  GetInitialAnisotropy(string aKey);
    
    // member variables
    
    Initializer         **mIArr;
    EquationOfState     *mEquationOfState;
    Equations           *mEquations;
    Stepper             *mStepper;
    Smoother            *mSmoother;
    Grid                *mGrid;
    Configurator        *mConfigurator;
    
    bool mFileTSV;
    bool mFileBIN;
    
    // recording frequency
    
    int mRecordingFrequency;
    
    // test option
    
    bool istest;
    
    // hydro identifier
    
    string mHydroID;
    
    // directories of TSV and BIN files
    
    string mTSVDir;
    string mBINDir;
    
    // calculation time evaluation
    
    time_t tTimeBegin, tTimeEnd;
    double *tDate;
    
    // memory usage
    
    MemoryUsed tMem;
    
    double tTau;
    
    // random seed
    
    unsigned int mSeed;
    
    // termination handling
    
    bool mTerminateQ;
    
};
#endif // _SOLVER_H_
