/**
 *
 *   thermalmass.h
 *
 *   Copyright (C) 2015 Michael Strickland and Mubarak Alqahtani
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _THERMALMASS_H_
#define _THERMALMASS_H_

#include "Global.h"

#include <iostream>
#include <iomanip>
#include <cmath>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>
#include <vector>

class ThermalMass {
    
public:
    
    // constructor
    
    ThermalMass();
    
    // destructor
    
    ~ThermalMass();
    
    // functions

    double mass(double T);
    double dTmass(double T);
    double Beq(double T);
    
    void loadInterpolations(string aEOSDir);
    void freeInterpolations();


private:
    
    string mEOSDir;
    
    double mTmin,mTmax;
    double mMmin,mMmax;
    double mDMmin,mDMmax;
    double mBmin,mBmax;
    
    int npts;
    double *T;
    double *M;
    double *DM;
    double *BEQ;
    
    gsl_interp_accel *accm;
    gsl_spline *splinem;
    
    gsl_interp_accel *accdm;
    gsl_spline *splinedm;
    
    gsl_interp_accel *accbeq;
    gsl_spline *splinebeq;
    
    int nTfnc;
    double *TfncX;
    double *TfncY;
    
    gsl_interp_accel *accTfnc;
    gsl_spline *splineTfnc;
    
    void readTfnc();
    void readMassData();
};

#endif // _THERMALMASS_H_
