/**
 *
 *   TriangulationFreezer.h
 *
 *   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _TRIANGULATION_FREEZER_H_
#define _TRIANGULATION_FREEZER_H_

#include "Global.h"

//class HyperCube{
//
//    public:
//
//        HyperCube()
//
//    private:
//}

struct quadruple {
    double t,x,y,z;
};

class TriangulationFreezer : public Freezer {
    
public:
    
    // constructor
    
    TriangulationFreezer(Configurator *aConfigurator, string aBINDir);
    
    // destructor
    
    ~TriangulationFreezer();
    
    // functions
    
    void    ExtractFOHS(void);
    virtual void PrintInfo(void);
    
private:
    
    void FindCells(void);
    
    string mTSVDir;
    string mXMLDir;
    string mBINDir;
    string mHydroID;
    
    Configurator    *mConfigurator;
    Grid            *mGrid;
    Vector4D        *mField;
    
    int mEquationsChoice;
    int mNumberOfEquations;
    
    string *mNames;
    string *mNamesFO;
    string *mUnitsFO;
    
    EquationOfState 	*mEOS;
    EquationOfState* GetEquationOfState();
    double meFO;
    
    // field domain and grid spacing
    
    double tmin, tmax, xmax, ymax, emax;
    double deltat, deltax, deltay, deltae;
    
    // freeze-out options
    
    double  TFO;
    
    // vectors to store centroids and normal vectors
    vector<quadruple>   mCentroids;
    vector<quadruple>   mNormalVectors;
    
    // description of the .xml file
    
    // export data to THERMINATOR .xml file
    
    void SetDescriptions(void);
    
    string mEquations;
    string mEquationOfState;
    string mDeviceType;
    string mCollisionalSystem;
    string mCMin;
    string mCMax;
    string mInitialSource;
    string mDensityScaling;
    string mDensityNormalization;
    string mUxInitialFlow;
    string mUyInitialFlow;
    string mRInitialFlow;
    string mIdealStatistics;
    
    void    ExportFOHSXML(string aFileName);
    void    ExportVariable(string Tab, ofstream &aFile, int n);
    //string  GetAXIS(string Tab, string aAxisName, double aMin, double aMax, int aPts, string aUnit);
    string  GetPARAMETERS(string Tab);
    string  GetDESCRIPTION(string Tab);
    string  GetTAG(string aTagName, string aDetailName, string aUnit, double aValue);
    string  GetTAG(string aTagName, string aDetailName, string aUnit, string aValue);
    string  GetTAG(string aTagName, string aDetailName, string aUnit, int aValue);
    
};
#endif //_TRIANGULATION_FREEZER_H_
