/** 
*
*   Vector1D.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _VECTOR1D_H_
#define _VECTOR1D_H_

#include "Global.h"

class Vector1D{

    public:

        // constructors

        Vector1D();
        Vector1D(string aName, double aXmin, double aXmax, int aXpts);
        Vector1D(const Vector1D& aVector);

        // destructor

        ~Vector1D();

        // functions

        inline string GetName(void) const;
        inline double GetXMin(void) const;
        inline double GetXMax(void) const;
        inline int GetXPts(void) const;

        double  Interpolate(double aX);
        Vector1D* DerivativeX(const char* aName);
        // oprtators

        inline double& operator()(unsigned int i);

    private:

        inline int InitDerivative(int aIdx, double aAMin, double aAMax, int aAPts);
        inline double Derivative(double aFin, double aFi0, double aFip);
        string	mVecName;
        double* mVec;

        double	mXmin, mXmax;
        int		mXpts;
        double	mDi;

        // used by InitDerivative() and Derivative()

        static double mXin, mXi0, mXip, mX0, mXd;

        // optimization

        int    mi;
        double mti;
};

/*!
*
*  \brief Operator()
*
*/

inline double& Vector1D::operator()(const unsigned int i){

  return mVec[i];

}

/*!
*
*  \brief Function returning name of the vector
*
*/

inline string Vector1D::GetName() const	{ return mVecName.c_str(); }

/*!
*
*  \brief Function returning Xmin
*
*/

inline double Vector1D::GetXMin() const	{ return mXmin; }

/*!
*
*  \brief Function returning Xmax
*
*/

inline double Vector1D::GetXMax() const	{ return mXmax; }

/*!
*
*  \brief Function returning Xpts
*
*/

inline int Vector1D::GetXPts() const	{ return mXpts; }

#endif // _VECTOR1D_H_
