/** 
*
*   Vector3D.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _VECTOR3D_H_
#define _VECTOR3D_H_

#include "Global.h"

class Vector3D{

    public:

        // constructors

        Vector3D();
        Vector3D(string aName,
	     double aXmin, double aXmax, int aXpts,
	     double aYmin, double aYmax, int aYpts,
	     double aZmin, double aZmax, int aZpts);
        Vector3D(const Vector3D& aVector);

        // destructor

        ~Vector3D();

        // functions

        inline double& operator()(unsigned int i, unsigned int j, unsigned int k);


        string GetName() const;
        double      GetXMin() const;
        double      GetXMax() const;
        int         GetXPts() const;
        double      GetYMin() const;
        double      GetYMax() const;
        int         GetYPts() const;
        double      GetZMin() const;
        double      GetZMax() const;
        int         GetZPts() const;

        double	Interpolate(double aX, double aY, double aZ);
        Vector3D*	DerivativeX(const char* aName);
        Vector3D*	DerivativeY(const char* aName);
        Vector3D*	DerivativeZ(const char* aName);

    private:

        double	Interpolate1D(double aX);
        double	Interpolate2D(double aX, double aY);
        double	Interpolate3D(double aX, double aY, double aZ);

        inline int	  InitDerivative(int aIdx, double aAMin, double aAMax, int aAPts);
        inline double Derivative(double aFin, double aFi, double aFip);

        string	mVecName;
        double***	mVec;

        double	mXmin, mXmax;
        int		mXpts;
        double	mDi;

        double	mYmin, mYmax;
        int		mYpts;
        double	mDj;

        double	mZmin, mZmax;
        int		mZpts;
        double	mDk;

        // used by InitDerivative() and Derivative()

        static double mXin, mXi0, mXip, mX0, mXd;
};

// operators

inline double& Vector3D::operator()(const unsigned int i, const unsigned int j, const unsigned int k)
{
  return mVec[i][j][k];
}

#endif
