/** 
*
*   Vector4D.h
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#ifndef _VECTOR4D_H_
#define _VECTOR4D_H_

#include "Global.h"

class Vector4D
{
  public:

    // operator () for reading/writing element of Vector

    inline float& operator()(unsigned int n,unsigned int i, unsigned int j, unsigned int k);

    // it may be required to define operator= here

// CLASS FUNCTIONS
    Vector4D();
    Vector4D(string aName,
	     float aTmin, float aTmax, int aTpts,
	     float aXmin, float aXmax, int aXpts,
	     float aYmin, float aYmax, int aYpts,
	     float aZmin, float aZmax, int aZpts);

    Vector4D(const Vector4D& aVector);
    ~Vector4D();

    string GetName() const;
    float     GetTMin() const;
    float     GetTMax() const;
    int         GetTPts() const;
    float     GetXMin() const;
    float     GetXMax() const;
    int         GetXPts() const;
    float     GetYMin() const;
    float     GetYMax() const;
    int         GetYPts() const;
    float     GetZMin() const;
    float     GetZMax() const;
    int         GetZPts() const;

    float	Interpolate(float aT, float aX, float aY, float aZ);

  private:

    float	Interpolate1D(float aX);
    float	Interpolate2D(float aX, float aY);
    float	Interpolate3D(float aX, float aY, float aZ);
    float	Interpolate4D(float aT, float aX, float aY, float aZ);

//    inline int	  InitDerivative(int aIdx, floataAMin, floataAMax, int aAPts);
//    inline floatDerivative(floataFin, floataFi, floataFip);

    string	mVecName;
    float****	mVec;

    float	mTmin, mTmax;
    int		mTpts;
    float	mDn;

    float	mXmin, mXmax;
    int		mXpts;
    float	mDi;

    float	mYmin, mYmax;
    int		mYpts;
    float	mDj;

    float	mZmin, mZmax;
    int		mZpts;
    float	mDk;

};

// OPERATORS

inline float& Vector4D::operator()(const unsigned int n, const unsigned int i, const unsigned int j, const unsigned int k)
{
  return mVec[n][i][j][k];
}

#endif //_VECTOR4D_H_
