/** 
*
*   Configurator.cxx
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#include "Global.h"

/*!
*
*  \brief Default constructor
*
*/

Configurator::Configurator(){

    PRINT_DEBUG_3("<Configurator::Configurator>");

    mParameters.clear();
}

/*!
*
*  \brief Default destructor
*
*/

Configurator::~Configurator(){

    PRINT_DEBUG_3("<Configurator::~Configurator>");

    mParameters.clear();
}

/*!
 *
 *  \brief Returns the value of the parameter kept in the configuration
 *
 *  \li aKeyword - name of the parameter
 *
 */

string Configurator::GetParameter(string aKeyword){

    PRINT_DEBUG_3("<Configurator::GetParameter>");

    vector<Parameter>::iterator tIterator;
    for (tIterator = mParameters.begin(); tIterator != mParameters.end(); tIterator++)
    if (tIterator->keyword == aKeyword) return tIterator->value;
    // else if the parameter is not found exit
    exit(ProgramStatus(ERROR_INPUT_PARAMETER_MISSING,
    "Configurator::GetParameter: Cannot find parameter for keyword : <" + aKeyword + "> in the configuration."));

}

/*!
 *
 *  \brief Sets the value of the parameter kept in the configuration
 *
 *  \li aKeyword - name of the parameter
 *
 */

void Configurator::SetParameter(string aKeyword, string aVal){
    
    PRINT_DEBUG_3("<Configurator::SetParameter>");
    
    vector<Parameter>::iterator tIterator;
    for (tIterator = mParameters.begin(); tIterator != mParameters.end(); tIterator++)
        if (tIterator->keyword == aKeyword) {
            mParameters.erase(tIterator);
            Parameter p = { aKeyword, aVal };
            AddParameter(&p);
            return;
        }
    // else if the parameter is not found exit
    exit(ProgramStatus(ERROR_INPUT_PARAMETER_MISSING,
                       "Configurator::SetParameter: Cannot find parameter for keyword : <" + aKeyword + "> in the configuration."));
    
}

/*!
 *
 *  \brief Returns the value of the parameter as the integer number
 *
 *  \li aKeyword - name of the parameter
 *
 */

int Configurator::GetInt(string aKeyword){

    return static_cast<int>(atoi(GetParameter(aKeyword).c_str()));

}

/*!
 *
 *  \brief Returns the value of the parameter as the double number
 *
 *  \li aKeyword - name of the parameter
 *
 */

double Configurator::GetDouble(string aKeyword){

    return static_cast<double>(atof(GetParameter(aKeyword).c_str()));

}

/*!
 *
 *  \brief Adds the parameter to the configuration
 *
 *  \li aKeyword - name of the parameter
 *
 */

void Configurator::AddParameter(Parameter* aParameter){

    PRINT_DEBUG_3("<Configurator::AddParameter>");

    mParameters.push_back(*aParameter);
}

/*!
 *
 *  \brief Prints all parameters kept in the configuration
 *
 */

void Configurator::PrintParameters(void){

    PRINT_DEBUG_3("<Configurator::PrintParameters>");

    vector<Parameter>::iterator tIterator;

    PrintRawMessage("Option","Value");
    for (tIterator = mParameters.begin(); tIterator != mParameters.end(); tIterator++)
    PrintRawMessage(tIterator->keyword,tIterator->value);
}

/*!
 *
 *  \brief Returns all parameters in ini file format suitable for output
 *
 */

void Configurator::OutputParametersToFile(ofstream& aFile){
    
    PRINT_DEBUG_3("<Configurator::DumpParameters>");
    
    vector<Parameter>::iterator tIterator;
    
    for (tIterator = mParameters.begin(); tIterator != mParameters.end(); tIterator++)
        aFile << tIterator->keyword << " = " << tIterator->value << endl;
}

