/** 
*
*   GaussianInitialSource.cxx
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#include "Global.h"

GaussianInitialSource::GaussianInitialSource(Configurator *aConfigurator,  Grid *aGrid)
: mConfigurator(aConfigurator), mGrid(aGrid){

    PRINT_DEBUG_3("<GaussianInitialSource::GaussianInitialSource>");

    // InitialTemperatureGauss in [GeV]
    // all variables are transformed to [fm]

    mNormalizationFactor = mConfigurator->GetDouble("InitialTemperature") / hbarc;

    mSigmaX = mConfigurator->GetDouble("SigmaX");
    mSigmaY = mConfigurator->GetDouble("SigmaY");
    mSigmaZ = mConfigurator->GetDouble("SigmaZ");
//
//    mEquationsChoice    = aConfigurator->GetInt("Equations");
//
//    switch(mEquationsChoice){
//
//        case 1:
//            // for PerfectFluidEquations
//            aFunction = &GaussianInitialSource::GetLogTemperature;
//            break;
//
//        case 2:
//            // for AnisotropicFluidEquations
//            aFunction = &GaussianInitialSource::GetTemperature;
//            break;
//
//        default:
//
//            exit(ProgramStatus(ERROR_OPTION_MISSING,
//            "FlatInitialSource::GaussianInitialSource: Cannot find Equations for " + d2s(mEquationsChoice)));
//    }

}

GaussianInitialSource::~GaussianInitialSource(){

    PRINT_DEBUG_3("<GaussianInitialSource::~GaussianInitialSource>");

    // nothing to clean

}

double GaussianInitialSource::GetValue(int i, int j, int k){

    PRINT_DEBUG_3("<GaussianInitialSource::GetValue>");

    return mNormalizationFactor * Gauss3D(mGrid->GetX(i),mGrid->GetY(j),mGrid->GetZ(k));
//
//    return (this->*aFunction)(mGrid->GetX(i),mGrid->GetY(j),mGrid->GetZ(k));

}
//
//double GaussianInitialSource::GetTemperature(double x, double y, double z){
//
//    PRINT_DEBUG_3("<GaussianInitialSource::GetTemperature>");
//
//    return mNormalizationFactor * Gauss3D(x, y, z);
//
//}
//
//double GaussianInitialSource::GetLogTemperature(double x, double y, double z){
//
//    PRINT_DEBUG_3("<GaussianInitialSource::GetLogTemperature>");
//
//    return log(mNormalizationFactor * Gauss3D(x, y, z));
//
//}

double GaussianInitialSource::Gauss3D(double x, double y, double z){

    PRINT_DEBUG_3("<GaussianInitialSource::Gauss3D>");

    return (1 - verysmall) * exp(-0.5 * (pow(x / mSigmaX,2) + pow(y / mSigmaY,2) + pow(z / mSigmaZ,2))) + verysmall;

}

void GaussianInitialSource::PrintInfo(void){

    PRINT_DEBUG_3("<GaussianInitialSource::PrintInfo>");

    PrintRawMessage("-> Gauss 3D");
    PrintRawMessage("\tInitial central Lambda", d2s(mNormalizationFactor) + " [1/fm] (" + d2s(mNormalizationFactor * hbarc) + " [GeV])");
    PrintRawMessage("\tsigmaX", d2s(mSigmaX));
    PrintRawMessage("\tsigmaY", d2s(mSigmaY));
    PrintRawMessage("\tsigmaE", d2s(mSigmaZ));

}
