/** 
*
*   GlobalFunctions.cxx
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#include "GlobalFunctions.h"

int ProgramStatus(Error_Type mError, string mWhere){

 switch(mError){

    case 0 :

        break;

    case 1 :

        PrintMessage("Error! File not found", mWhere);
        break;

    case 2 :

        PrintMessage("Error! Option incorrect", mWhere);
        break;

    case 3 :

        PrintMessage("Error! Memory bad allocation", mWhere);
        break;

    case 4 :

        PrintMessage("Error! Input parameter missing", mWhere);
        break;

    case 5 :

        PrintMessage("Error! Incorrect value(s) obtained", mWhere);
        break;

    case 6 :

        PrintMessage("Error! File corrupted", mWhere);
        break;

    default:

        break;

 }
    return static_cast<int>(mError);
}

void PrintValue(double aValue){
    PRINT_MESSAGE("|\r\t " << aValue << "\r\t\t\t\t\t\t\t\t\t|"			);
}

void PrintRawMessage(string aMessage){
    PRINT_MESSAGE("|\r\t " << aMessage << "\r\t\t\t\t\t\t\t\t\t|"			);
}

void PrintRawMessage(string aMessage1, string aMessage2){
    PRINT_MESSAGE("|\r\t " << aMessage1 << " : " << aMessage2 << "\r\t\t\t\t\t\t\t\t\t|"			);
}

void PrintRawMessage(string aMessage, double aValue){
    PRINT_MESSAGE("|\r\t " << aMessage << " : " << d2s(aValue) << "\r\t\t\t\t\t\t\t\t\t|"			);
}

void PrintMessage(string aMessage){
    PRINT_MESSAGE("*************************************************************************"	);
    PRINT_MESSAGE("*\r\t " << aMessage << "\r\t\t\t\t\t\t\t\t\t*"			);
    PRINT_MESSAGE("*************************************************************************"	);
}

void PrintMessage(string aMessage1, string aMessage2){
    PRINT_MESSAGE("-------------------------------------------------------------------------"	);
    PRINT_MESSAGE("|\r\t " << aMessage1 << "\r\t\t\t\t\t\t\t\t\t|"			);
    PRINT_MESSAGE("|\r\t " << aMessage2 << "\r\t\t\t\t\t\t\t\t\t|"			);
    PRINT_MESSAGE("-------------------------------------------------------------------------"	);
}

void PrintMessage(string aMessage, double aValue){
    PRINT_MESSAGE("-------------------------------------------------------------------------"	);
    PRINT_MESSAGE("|\r\t " << aMessage << "\r\t\t\t\t\t\t\t\t\t|"			);
    PRINT_MESSAGE("|\r\t " << d2s(aValue) << "\r\t\t\t\t\t\t\t\t\t|"			);
    PRINT_MESSAGE("-------------------------------------------------------------------------"	);
}

void PrintEvolutionOutroMessage(string aMessage, const double *aDate){
    PRINT_MESSAGE("*************************************************************************"	);
    PRINT_MESSAGE("*\r\t " << aMessage << "\r\t\t\t\t\t\t\t\t\t*"			);
    PRINT_MESSAGE("*\r\t Time of calculation : " << aDate[0] << " d " << aDate[1] << " h " << aDate[2] << " m " << aDate[3] << "s." << "\r\t\t\t\t\t\t\t\t\t*");
    PRINT_MESSAGE("*************************************************************************"	);
}

bool FileExists(const string& aFileName){
    ifstream infile(aFileName.c_str());
    return infile.good();
}

bool MakeDirectory(string aPath){
    bool tSuccess = false;
    int nRC = mkdir(aPath.c_str(), 0777); // A mkdir return value of 0 indicates successful completion, and -1 indicates failure //http://www.zzee.com/solutions/linux-permissions.shtml#setuid
    // or S_IROTH | S_IWOTH | S_IXOTH | S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP
    if(nRC == -1)
    {
        switch(errno)
        {
            case ENOENT: //parent didn't exist, try to create it // This error is reported when a file referenced as a directory component in the file name doesn't exist, or when a component is a symbolic link whose target file does not exist.

                if(MakeDirectory(aPath.substr(0, aPath.find_last_of('/'))))
                    //Now, try to create again.
                    tSuccess = 0 == mkdir(aPath.c_str(), 0777);
                else
                    tSuccess = false;
                break;

            case EEXIST: // A file named filename already exists.

                tSuccess = true;
                break;

            default:
                tSuccess = false;
                break;
        }
    }
    else{
        tSuccess = true;
    }
    PrintRawMessage("Directory created" , aPath);
    return tSuccess;
}

void ClearDirectory(string aPath){

    const char* path = aPath.c_str();
    // These are data types defined in the "dirent" header
    struct dirent *next_file;

    DIR *theFolder;

    char filepath[256];

    theFolder = opendir(path);

    while ((next_file = readdir(theFolder)))
    {
        // build the full path for each file in the folder
        sprintf(filepath, "%s/%s", path, next_file->d_name);
        remove(filepath);
    }
}

bool CopyFile(string aInputFile, string aOutputFile){

	ifstream inFile(aInputFile.c_str(), ios::in | ios::binary);
	ofstream outFile(aOutputFile.c_str(), ios::out | ios::binary);

	if(inFile.is_open() && outFile.is_open()){
        outFile << inFile.rdbuf();
	}
	else if(!outFile.is_open()){
		PrintRawMessage("Couldn't open for copying", aOutputFile);
		return 0;
	}
	else if(!inFile.is_open()){
        PrintRawMessage("Couldn't open for copying", aInputFile);
		return 0;
	}

	inFile.close();
	outFile.close();

	return 0;
}

string GetFilename(const string& str){

    size_t found = str.find_last_of("/\\");
    return str.substr(found+1);
}

string GetDirectory(const string& str){

    size_t found = str.find_last_of("/\\");
    return str.substr(0,found);
}

// function that converts string to double

double s2d(const string& s){
   istringstream i(s);
   double x;
   if (!(i >> x))
     return 0;
   return x;
 }

// function that converts double to string

string d2s(const double& d){
   stringstream s;
   if (!(s << d)) return 0;
   return s.str();
}

// function deleting spaces and tabs

string delwhite(string str){

   str.erase(std::remove(str.begin(), str.end(), ' '), str.end());
   str.erase(std::remove(str.begin(), str.end(), '\t'), str.end());
   return str;

}
