/** 
*
*   Grid.cxx
*	
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*    
*/

#include "Global.h"

Grid::Grid(Configurator *aConfigurator){

    PRINT_DEBUG_3("<Grid::Grid>");

    // spacing x,y,z

 	mdX = aConfigurator->GetDouble("XSpacing");
	mdY = aConfigurator->GetDouble("YSpacing");
	mdZ = aConfigurator->GetDouble("ZSpacing");

	// number of points x,y,z

    mXPts   = aConfigurator->GetInt("XPts");
    mYPts   = aConfigurator->GetInt("YPts");
    mZPts   = aConfigurator->GetInt("ZPts");

    //optimization // needed for GetX, GetY, GetZ

    mFacX = (mXPts - 1.0) / 2.0;
    mFacY = (mYPts - 1.0) / 2.0;
    mFacZ = (mZPts - 1.0) / 2.0;

    // minimal values x,y,z

	mXMin = GetX(0);
	mYMin = GetY(0);
	mZMin = GetZ(0);

    // maximal values x,y,z

	mXMax = GetX(mXPts - 1);
	mYMax = GetY(mYPts - 1);
	mZMax = GetZ(mZPts - 1);

	// time grid characteristics

	mdT     = aConfigurator->GetDouble("TauStepSize");
	mTMin   = aConfigurator->GetDouble("TauMin");
	mTMax   = aConfigurator->GetDouble("TauMax");
    mTPts   = ceil((mTMax - mTMin) / mdT) + 1;

}

Grid::~Grid(){

    PRINT_DEBUG_3("<Grid::~Grid>");

    // nothing to do

}

void Grid::PrintInfo(void){

    PRINT_DEBUG_3("<Grid::PrintInfo>");

    PrintRawMessage("Time Range","(" + d2s(mTMin) + ", " + d2s(mTMax) + ")");
    PrintRawMessage("Time Step",mdT);
    PrintRawMessage("Time Points",mTPts);
    PrintRawMessage("X range","(" + d2s(mXMin) + "," + d2s(mXMax) + ")");
    PrintRawMessage("Y range","(" + d2s(mYMin) + "," + d2s(mYMax) + ")");
    PrintRawMessage("Z range","(" + d2s(mZMin) + "," + d2s(mZMax) + ")");
    PrintRawMessage("X spacing",GetdX());
    PrintRawMessage("Y spacing",GetdY());
    PrintRawMessage("Z spacing",GetdZ());
    PrintRawMessage("X points",GetXP());
    PrintRawMessage("Y points",GetYP());
    PrintRawMessage("Z points",GetZP());
    PrintRawMessage("Total number of grid points", GetXP() * GetYP() * GetZP());

}
