
/**
 *
 *   Hfuncs.cxx
 *
 *   2017 Mubarak Alqahtani, Mohammad Nopoush, Radoslaw Ryblewski, and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "Global.h"

/*!
 *
 *  \brief Default constructor
 *
 */

Hfuncs::Hfuncs() {
    PRINT_DEBUG_3("<Hfuncs:Hfuncs>");
}


/*!
 *
 *  \brief Destructor
 *
 */

Hfuncs::~Hfuncs(){
    PRINT_DEBUG_3("<Hfuncs::~Hfuncs>");
    pints.loadInterpolations();
}

// Interpolation loading

void Hfuncs::loadInterpolations() {
    
    pints.loadInterpolations();
}

// Interpolation freeing

void Hfuncs::freeInterpolations() {
    
    pints.freeInterpolations();
}

/************************************************
 *  expanded H functions and their derivatives
 ************************************************/


inline double Hfuncs::genI3(double deltax, double deltay, double deltaz, double m, double delta0){


	double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
	double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
	double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
	double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
	double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
	double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;

	return -((676039*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 352716*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 92378*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 48620*deltax9*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 243100*deltay9*deltaz3 + 225225*deltay8*deltaz4 + 6435*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 3432*deltax7*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 216216*deltay7*deltaz5 + 213444*deltay6*deltaz6 + 924*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 504*deltax5*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 216216*deltay5*deltaz7 + 225225*deltay4*deltaz8 + 35*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 20*deltax3*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + 243100*deltay3*deltaz9 + 4*deltax*(deltay + deltaz)*(88179*deltay10 - 41990*deltay9*deltaz + 88179*deltaz10 + 78455*deltay8*deltaz2 - 46280*deltay7*deltaz3 + 76310*deltay6*deltaz4 - 47204*deltay5*deltaz5 + 76310*deltay4*deltaz6 - 46280*deltay3*deltaz7 + 78455*deltay2*deltaz8 - 41990*deltay*deltaz9) + 6*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-11.5,m,delta0))/6.029312e8 + ((88179*deltax11 + 46189*deltax10*(deltay + deltaz) + 12155*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 6435*deltax8*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 858*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 462*deltax6*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 70*deltax4*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 5*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 3*deltax2*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + (deltay + deltaz)*(88179*deltay10 - 41990*deltay9*deltaz + 88179*deltaz10 + 78455*deltay8*deltaz2 - 46280*deltay7*deltaz3 + 76310*deltay6*deltaz4 - 47204*deltay5*deltaz5 + 76310*deltay4*deltaz6 - 46280*deltay3*deltaz7 + 78455*deltay2*deltaz8 - 41990*deltay*deltaz9) + deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-10.5,m,delta0))/6.3307776e7 - ((46189*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 24310*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 6435*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 3432*deltax7*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 462*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 252*deltax5*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 70*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 40*deltax3*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 3*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 2*deltax*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + 24310*deltay*deltaz9)*Pi*pints.pInt(-9.5,m,delta0))/2.6148864e7 + ((12155*deltax9 + 6435*deltax8*deltay + 5148*deltax7*deltay2 + 4620*deltax6*deltay3 + 4410*deltax5*deltay4 + 4410*deltax4*deltay5 + 4620*deltax3*deltay6 + 5148*deltax2*deltay7 + 6435*deltax*deltay8 + 12155*deltay9 + (6435*deltax8 + 3432*deltax7*deltay + 2772*deltax6*deltay2 + 2520*deltax5*deltay3 + 2450*deltax4*deltay4 + 2520*deltax3*deltay5 + 2772*deltax2*deltay6 + 3432*deltax*deltay7 + 6435*deltay8)*deltaz + 12*(deltax + deltay)*(429*deltax6 - 198*deltax5*deltay + 387*deltax4*deltay2 - 212*deltax3*deltay3 + 387*deltax2*deltay4 - 198*deltax*deltay5 + 429*deltay6)*deltaz2 + 20*(231*deltax6 + 126*deltax5*deltay + 105*deltax4*deltay2 + 100*deltax3*deltay3 + 105*deltax2*deltay4 + 126*deltax*deltay5 + 231*deltay6)*deltaz3 + 70*(deltax + deltay)*(63*deltax4 - 28*deltax3*deltay + 58*deltax2*deltay2 - 28*deltax*deltay3 + 63*deltay4)*deltaz4 + 126*(35*deltax4 + 20*deltax3*deltay + 18*deltax2*deltay2 + 20*deltax*deltay3 + 35*deltay4)*deltaz5 + 924*(deltax + deltay)*(5*deltax2 - 2*deltax*deltay + 5*deltay2)*deltaz6 + 1716*(3*deltax2 + 2*deltax*deltay + 3*deltay2)*deltaz7 + 6435*(deltax + deltay)*deltaz8 + 12155*deltaz9)*Pi*pints.pInt(-8.5,m,delta0))/5.292032e6 - ((6435*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 3432*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 924*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 504*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 70*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 40*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 12*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 8*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pInt(-7.5,m,delta0))/2.08896e6 + ((429*deltax7 + 231*deltax6*(deltay + deltaz) + 63*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 35*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 5*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 3*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*Pi*pints.pInt(-6.5,m,delta0))/99840. - ((231*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 126*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 35*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 20*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 3*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 2*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pInt(-5.5,m,delta0))/36608. + ((63*deltax5 + 35*deltax4*(deltay + deltaz) + 10*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 6*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pInt(-4.5,m,delta0))/6336. - ((35*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 20*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 6*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 4*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pInt(-3.5,m,delta0))/2016. + ((5*deltax3 + 3*deltax2*(deltay + deltaz) + deltax*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2))*Pi*pints.pInt(-2.5,m,delta0))/140. - ((3*deltax2 + 3*deltay2 + 2*deltay*deltaz + 2*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pInt(-1.5,m,delta0))/30. + (2*(deltax + deltay + deltaz)*Pi*pints.pInt(-0.5,m,delta0))/3. + 4*Pi*pints.pInt(0.5,m,delta0);
}

inline double Hfuncs::I3_1(double ax, double ay, double az, double m){

	double delta0=1.;
	double deltax=ax*ax-delta0;
	double deltay=ay*ay-delta0;
	double deltaz=az*az-delta0;

	return genI3(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::I3_2(double ax, double ay, double az, double m){


	double delta0=4.;
	double deltax=ax*ax-delta0;
	double deltay=ay*ay-delta0;
	double deltaz=az*az-delta0;

	return genI3(deltax,deltay,deltaz, m, delta0);
}

inline double Hfuncs::H3exp(double ax, double ay, double az, double m){

	double rmin = 1.75;
 	double rmax = 1.85;	
	double r = sqrt(ax*ax+ay*ay+az*az);
	double alpha = ax*ay*az;
	if (r < rmin) return alpha*Nt*I3_1(ax,ay,az,m);
	if (r > rmax) return alpha*Nt*I3_2(ax,ay,az,m);
        else return alpha*Nt*(10.*(rmax-r)*I3_1(ax,ay,az,m) + 10.*(r-rmin)*I3_2(ax,ay,az,m));
}

double Hfuncs::H3(double ax, double ay, double az, double m) {
	return H3exp(ax,ay,az,m);
}

//*********************************************************************
// Derivatives of H3
//*********************************************************************

inline double Hfuncs::genI31(double deltax, double deltay, double deltaz, double m, double delta0){

    
    double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
    double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
    double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
    double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
    double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
    double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;

    
    return Sqrt(delta0 + deltay)*Sqrt(delta0 + deltaz)*(-((676039*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 352716*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 92378*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 243100*deltay9*deltaz3 + 48620*deltax9*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 225225*deltay8*deltaz4 + 6435*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 216216*deltay7*deltaz5 + 3432*deltax7*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 213444*deltay6*deltaz6 + 924*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 216216*deltay5*deltaz7 + 504*deltax5*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 225225*deltay4*deltaz8 + 35*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 243100*deltay3*deltaz9 + 20*deltax3*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 6*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9) + 4*deltax*(88179*deltay11 + 46189*deltay10*deltaz + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 32175*deltay8*deltaz3 + 30030*deltay7*deltaz4 + 29106*deltay6*deltaz5 + 29106*deltay5*deltaz6 + 30030*deltay4*deltaz7 + 32175*deltay3*deltaz8 + 36465*deltay2*deltaz9))*Pi*pints.pInt(-11.5,m,delta0))/6.029312e8 + ((88179*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 46189*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 12155*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 6435*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 858*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 462*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 126*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 70*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 5*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 3*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-10.5,m,delta0))/6.3307776e7 - ((46189*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 24310*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 6435*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 3432*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 462*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 252*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 70*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 40*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 3*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 2*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*Pi*pints.pInt(-9.5,m,delta0))/2.6148864e7 + ((12155*deltax9 + 6435*deltax8*deltay + 5148*deltax7*deltay2 + 4620*deltax6*deltay3 + 4410*deltax5*deltay4 + 4410*deltax4*deltay5 + 4620*deltax3*deltay6 + 5148*deltax2*deltay7 + 6435*deltax*deltay8 + 12155*deltay9 + (6435*deltax8 + 3432*deltax7*deltay + 2772*deltax6*deltay2 + 2520*deltax5*deltay3 + 2450*deltax4*deltay4 + 2520*deltax3*deltay5 + 2772*deltax2*deltay6 + 3432*deltax*deltay7 + 6435*deltay8)*deltaz + 12*(deltax + deltay)*(429*deltax6 - 198*deltax5*deltay + 387*deltax4*deltay2 - 212*deltax3*deltay3 + 387*deltax2*deltay4 - 198*deltax*deltay5 + 429*deltay6)*deltaz2 + 20*(231*deltax6 + 126*deltax5*deltay + 105*deltax4*deltay2 + 100*deltax3*deltay3 + 105*deltax2*deltay4 + 126*deltax*deltay5 + 231*deltay6)*deltaz3 + 70*(deltax + deltay)*(63*deltax4 - 28*deltax3*deltay + 58*deltax2*deltay2 - 28*deltax*deltay3 + 63*deltay4)*deltaz4 + 126*(35*deltax4 + 20*deltax3*deltay + 18*deltax2*deltay2 + 20*deltax*deltay3 + 35*deltay4)*deltaz5 + 924*(deltax + deltay)*(5*deltax2 - 2*deltax*deltay + 5*deltay2)*deltaz6 + 1716*(3*deltax2 + 2*deltax*deltay + 3*deltay2)*deltaz7 + 6435*(deltax + deltay)*deltaz8 + 12155*deltaz9)*Pi*pints.pInt(-8.5,m,delta0))/5.292032e6 - ((6435*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 3432*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 924*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 504*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 70*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 40*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 12*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 8*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pInt(-7.5,m,delta0))/2.08896e6 + ((429*deltax7 + 231*deltax6*(deltay + deltaz) + 63*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 35*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 5*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 3*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*Pi*pints.pInt(-6.5,m,delta0))/99840. - ((231*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 126*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 35*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 20*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 3*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 2*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pInt(-5.5,m,delta0))/36608. + ((63*deltax5 + 35*deltax4*(deltay + deltaz) + 10*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 6*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pInt(-4.5,m,delta0))/6336. - ((35*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 20*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 6*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 4*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pInt(-3.5,m,delta0))/2016. + ((5*deltax3 + 3*deltax2*(deltay + deltaz) + deltax*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2))*Pi*pints.pInt(-2.5,m,delta0))/140. - ((3*deltax2 + 3*deltay2 + 2*deltay*deltaz + 2*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pInt(-1.5,m,delta0))/30. + (2*(deltax + deltay + deltaz)*Pi*pints.pInt(-0.5,m,delta0))/3. + ((delta0 + deltax)*Pi*(-2909907*(2028117*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 969969*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 230945*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 109395*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 12870*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 6006*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 1386*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 630*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 35*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 15*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 3*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*pints.pInt(-11.5,m,delta0) + 6928350*(969969*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 461890*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 109395*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 51480*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 6006*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 2772*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 630*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 280*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 15*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 6*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*pints.pInt(-10.5,m,delta0) - 33547800*(230945*deltax9 + 12155*deltay9 + 6435*deltay8*deltaz + 109395*deltax8*(deltay + deltaz) + 5148*deltay7*deltaz2 + 25740*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 4620*deltay6*deltaz3 + 12012*deltax6*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 4410*deltay5*deltaz4 + 1386*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 4410*deltay4*deltaz5 + 630*deltax4*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 4620*deltay3*deltaz6 + 140*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 5148*deltay2*deltaz7 + 60*deltax2*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 6435*deltay*deltaz8 + 3*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 12155*deltaz9)*pints.pInt(-9.5,m,delta0) + 82882800*(109395*deltax8 + 36036*deltax6*deltay2 + 27720*deltax5*deltay3 + 22050*deltax4*deltay4 + 17640*deltax3*deltay5 + 13860*deltax2*deltay6 + 10296*deltax*deltay7 + 6435*deltay8 + 24024*deltax6*deltay*deltaz + 16632*deltax5*deltay2*deltaz + 12600*deltax4*deltay3*deltaz + 9800*deltax3*deltay4*deltaz + 7560*deltax2*deltay5*deltaz + 5544*deltax*deltay6*deltaz + 3432*deltay7*deltaz + 51480*deltax7*(deltay + deltaz) + 30888*deltax*deltax5*deltaz2 + 5148*deltax6*deltaz2 - 11880*deltax*deltax4*deltay*deltaz2 + 28512*deltax5*deltay*deltaz2 - 11880*deltax4*Power(deltay,2)*deltaz2 + 18576*deltax*deltax3*deltay2*deltaz2 + 4644*deltax4*deltay2*deltaz2 + 18576*deltax3*deltay*deltay2*deltaz2 - 7632*deltax*deltax2*deltay3*deltaz2 - 2544*deltax3*deltay3*deltaz2 - 7632*deltax2*deltay*deltay3*deltaz2 + 9288*Power(deltax,2)*deltay4*deltaz2 + 4644*deltax2*deltay4*deltaz2 + 9288*deltax*deltay*deltay4*deltaz2 - 4752*deltax*deltay5*deltaz2 - 2376*deltay*deltay5*deltaz2 + 5148*deltay6*deltaz2 + 27720*deltax5*deltaz3 + 12600*deltax4*deltay*deltaz3 + 8400*deltax3*deltay2*deltaz3 + 6000*deltax2*deltay3*deltaz3 + 4200*deltax*deltay4*deltaz3 + 2520*deltay5*deltaz3 + 17640*deltax*deltax3*deltaz4 + 4410*deltax4*deltaz4 - 5880*deltax*deltax2*deltay*deltaz4 + 15680*deltax3*deltay*deltaz4 - 5880*deltax2*Power(deltay,2)*deltaz4 + 8120*Power(deltax,2)*deltay2*deltaz4 + 4060*deltax2*deltay2*deltaz4 + 8120*deltax*deltay*deltay2*deltaz4 - 3920*deltax*deltay3*deltaz4 - 1960*deltay*deltay3*deltaz4 + 4410*deltay4*deltaz4 + 17640*deltax3*deltaz5 + 7560*deltax2*deltay*deltaz5 + 4536*deltax*deltay2*deltaz5 + 2520*deltay3*deltaz5 + 9240*Power(deltax,2)*deltaz6 + 4620*deltax2*deltaz6 + 5544*deltax*deltay*deltaz6 - 1848*Power(deltay,2)*deltaz6 + 4620*deltay2*deltaz6 + 10296*deltax*deltaz7 + 3432*deltay*deltaz7 + 6435*deltaz8)*pints.pInt(-8.5,m,delta0) - 1679758080*(6435*deltax7 + 3003*deltax6*(deltay + deltaz) + 693*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 315*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 35*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 3*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*pints.pInt(-7.5,m,delta0) + 4393213440*(3003*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 1386*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 315*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 140*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 15*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 6*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*pints.pInt(-6.5,m,delta0) - 23962982400*(693*deltax5 + 315*deltax4*(deltay + deltaz) + 70*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 30*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 3*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*pints.pInt(-5.5,m,delta0) + 69226393600*(315*deltax4 + 90*deltax2*deltay2 + 60*deltax*deltay*deltay2 + 35*deltay4 + 60*deltax2*deltay*deltaz - 24*deltax*Power(deltay,2)*deltaz + 60*deltax*deltay2*deltaz + 20*deltay3*deltaz - 24*deltax*deltay*Power(deltaz,2) + 140*deltax3*(deltay + deltaz) + 90*deltax2*deltaz2 + 60*deltax*deltay*deltaz2 + 18*deltay2*deltaz2 + 60*deltax*deltaz*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4)*pints.pInt(-4.5,m,delta0) - 870274662400*(35*deltax3 + 9*deltax*deltay2 + 5*deltay*deltay2 + 6*deltax*deltay*deltaz - 2*Power(deltay,2)*deltaz + 5*deltay2*deltaz - 2*deltay*Power(deltaz,2) + 15*deltax2*(deltay + deltaz) + 9*deltax*deltaz2 + 5*deltay*deltaz2 + 5*deltaz*deltaz2)*pints.pInt(-3.5,m,delta0) + 3132988784640*(15*deltax2 + 3*deltay2 + 2*deltay*deltaz + 6*deltax*(deltay + deltaz) + 3*deltaz2)*pints.pInt(-2.5,m,delta0) - 29241228656640*(3*deltax + deltay + deltaz)*pints.pInt(-1.5,m,delta0) + 292412286566400*pints.pInt(-0.5,m,delta0)))/2.193092149248e14 + 4*Pi*pints.pInt(0.5,m,delta0));
}

inline double Hfuncs::I31_1(double ax, double ay, double az, double m){
    
    double delta0=1.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI31(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::I31_2(double ax, double ay, double az, double m){
    
    
    double delta0=4.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI31(deltax,deltay,deltaz, m, delta0);
}

inline double Hfuncs::d1H3exp(double ax, double ay, double az, double m) {

    double rmin = 1.75;
    double rmax = 1.85;
    double r = sqrt(ax*ax+ay*ay+az*az);
    double alpha = ax*ay*az;
    if (r < rmin) return Nt*I31_1(ax,ay,az,m);
    if (r > rmax) return Nt*I31_2(ax,ay,az,m);
    else return Nt*(10.*(rmax-r)*I31_1(ax,ay,az,m) + 10.*(r-rmin)*I31_2(ax,ay,az,m));
    
}

double Hfuncs::d1H3(double ax, double ay, double az, double m) {
	return d1H3exp(ax,ay,az,m);
}

double Hfuncs::d2H3(double ax, double ay, double az, double m) {
	return d1H3exp(ay,ax,az,m);
}

double Hfuncs::d3H3(double ax, double ay, double az, double m) {
	return d1H3exp(az,ay,ax,m);
}

inline double Hfuncs::genI3m(double deltax, double deltay, double deltaz, double m, double delta0){
    
    double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
    double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
    double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
    double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
    double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
    double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;
    
    return -((676039*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 352716*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 92378*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 48620*deltax9*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 243100*deltay9*deltaz3 + 225225*deltay8*deltaz4 + 6435*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 3432*deltax7*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 216216*deltay7*deltaz5 + 213444*deltay6*deltaz6 + 924*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 504*deltax5*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 216216*deltay5*deltaz7 + 225225*deltay4*deltaz8 + 35*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 20*deltax3*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + 243100*deltay3*deltaz9 + 4*deltax*(deltay + deltaz)*(88179*deltay10 - 41990*deltay9*deltaz + 88179*deltaz10 + 78455*deltay8*deltaz2 - 46280*deltay7*deltaz3 + 76310*deltay6*deltaz4 - 47204*deltay5*deltaz5 + 76310*deltay4*deltaz6 - 46280*deltay3*deltaz7 + 78455*deltay2*deltaz8 - 41990*deltay*deltaz9) + 6*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pmInt(-11.5,m,delta0))/6.029312e8 + ((88179*deltax11 + 46189*deltax10*(deltay + deltaz) + 12155*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 6435*deltax8*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 858*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 462*deltax6*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 70*deltax4*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 5*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 3*deltax2*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + (deltay + deltaz)*(88179*deltay10 - 41990*deltay9*deltaz + 88179*deltaz10 + 78455*deltay8*deltaz2 - 46280*deltay7*deltaz3 + 76310*deltay6*deltaz4 - 47204*deltay5*deltaz5 + 76310*deltay4*deltaz6 - 46280*deltay3*deltaz7 + 78455*deltay2*deltaz8 - 41990*deltay*deltaz9) + deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pmInt(-10.5,m,delta0))/6.3307776e7 - ((46189*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 24310*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 6435*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 3432*deltax7*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 462*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 252*deltax5*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 70*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 40*deltax3*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 3*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 2*deltax*(deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8) + 24310*deltay*deltaz9)*Pi*pints.pmInt(-9.5,m,delta0))/2.6148864e7 + ((12155*deltax9 + 6435*deltax8*deltay + 5148*deltax7*deltay2 + 4620*deltax6*deltay3 + 4410*deltax5*deltay4 + 4410*deltax4*deltay5 + 4620*deltax3*deltay6 + 5148*deltax2*deltay7 + 6435*deltax*deltay8 + 12155*deltay9 + (6435*deltax8 + 3432*deltax7*deltay + 2772*deltax6*deltay2 + 2520*deltax5*deltay3 + 2450*deltax4*deltay4 + 2520*deltax3*deltay5 + 2772*deltax2*deltay6 + 3432*deltax*deltay7 + 6435*deltay8)*deltaz + 12*(deltax + deltay)*(429*deltax6 - 198*deltax5*deltay + 387*deltax4*deltay2 - 212*deltax3*deltay3 + 387*deltax2*deltay4 - 198*deltax*deltay5 + 429*deltay6)*deltaz2 + 20*(231*deltax6 + 126*deltax5*deltay + 105*deltax4*deltay2 + 100*deltax3*deltay3 + 105*deltax2*deltay4 + 126*deltax*deltay5 + 231*deltay6)*deltaz3 + 70*(deltax + deltay)*(63*deltax4 - 28*deltax3*deltay + 58*deltax2*deltay2 - 28*deltax*deltay3 + 63*deltay4)*deltaz4 + 126*(35*deltax4 + 20*deltax3*deltay + 18*deltax2*deltay2 + 20*deltax*deltay3 + 35*deltay4)*deltaz5 + 924*(deltax + deltay)*(5*deltax2 - 2*deltax*deltay + 5*deltay2)*deltaz6 + 1716*(3*deltax2 + 2*deltax*deltay + 3*deltay2)*deltaz7 + 6435*(deltax + deltay)*deltaz8 + 12155*deltaz9)*Pi*pints.pmInt(-8.5,m,delta0))/5.292032e6 - ((6435*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 3432*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 924*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 504*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 70*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 40*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 12*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 8*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pmInt(-7.5,m,delta0))/2.08896e6 + ((429*deltax7 + 231*deltax6*(deltay + deltaz) + 63*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 35*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 5*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 3*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*Pi*pints.pmInt(-6.5,m,delta0))/99840. - ((231*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 126*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 35*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 20*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 3*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 2*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pmInt(-5.5,m,delta0))/36608. + ((63*deltax5 + 35*deltax4*(deltay + deltaz) + 10*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 6*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pmInt(-4.5,m,delta0))/6336. - ((35*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 20*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 6*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 4*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pmInt(-3.5,m,delta0))/2016. + ((5*deltax3 + 3*deltax2*(deltay + deltaz) + deltax*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2))*Pi*pints.pmInt(-2.5,m,delta0))/140. - ((3*deltax2 + 3*deltay2 + 2*deltay*deltaz + 2*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pmInt(-1.5,m,delta0))/30. + (2*(deltax + deltay + deltaz)*Pi*pints.pmInt(-0.5,m,delta0))/3. + 4*Pi*pints.pmInt(0.5,m,delta0);
}

inline double Hfuncs::I3m_1(double ax, double ay, double az, double m){
    
    double delta0=1.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3m(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::I3m_2(double ax, double ay, double az, double m){
    
    
    double delta0=4.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3m(deltax,deltay,deltaz, m, delta0);
}

inline double Hfuncs::d4H3exp(double ax, double ay, double az, double m) {
    
    double rmin = 1.75;
    double rmax = 1.85;
    double r = sqrt(ax*ax+ay*ay+az*az);
    double alpha = ax*ay*az;;
    if (r < rmin) return (2.*m)*alpha*Nt*I3m_1(ax,ay,az,m);
    if (r > rmax) return (2.*m)*alpha*Nt*I3m_2(ax,ay,az,m);
    else return (2.*m)*alpha*Nt*(10.*(rmax-r)*I3m_1(ax,ay,az,m) + 10.*(r-rmin)*I3m_2(ax,ay,az,m));
}

double Hfuncs::d4H3(double ax, double ay, double az, double m) {
    return d4H3exp(ax,ay,az,m);
}


//************************************************************************
// New expand H3X functions 
//************************************************************************

inline double Hfuncs::genI3X(double deltax, double deltay, double deltaz, double m, double delta0){

	double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
	double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
	double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
	double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
	double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
	double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;

	return ((16900975*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 8112468*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 1939938*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 243100*deltay9*deltaz3 + 923780*deltax9*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 225225*deltay8*deltaz4 + 109395*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 216216*deltay7*deltaz5 + 51480*deltax7*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 213444*deltay6*deltaz6 + 12012*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 216216*deltay5*deltaz7 + 5544*deltax5*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 225225*deltay4*deltaz8 + 315*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 243100*deltay3*deltaz9 + 140*deltax3*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 30*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9) + 12*deltax*(88179*deltay11 + 46189*deltay10*deltaz + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 32175*deltay8*deltaz3 + 30030*deltay7*deltaz4 + 29106*deltay6*deltaz5 + 29106*deltay5*deltaz6 + 30030*deltay4*deltaz7 + 32175*deltay3*deltaz8 + 36465*deltay2*deltaz9))*Pi*pints.pInt(-12.5,m,delta0))/7.077888e8 - ((2028117*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 969969*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 230945*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 109395*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 12870*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 6006*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 1386*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 630*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 35*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 15*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 3*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-11.5,m,delta0))/7.53664e7 + ((969969*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 461890*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 109395*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 51480*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 6006*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 2772*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 630*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 280*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 15*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 6*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*Pi*pints.pInt(-10.5,m,delta0))/3.1653888e7 + ((-230945*deltax9 - 109395*deltax8*(deltay + deltaz) - 25740*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 12012*deltax6*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) - 1386*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 630*deltax4*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) - 140*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) - 60*deltax2*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) - 3*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) - (deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8))*Pi*pints.pInt(-9.5,m,delta0))/6.537216e6 + ((109395*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 51480*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 12012*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 5544*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 630*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 280*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 60*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 24*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pInt(-8.5,m,delta0))/2.646016e6 + ((-6435*deltax7 - 3003*deltax6*(deltay + deltaz) - 693*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 315*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) - 35*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 15*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) - 3*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) - (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*Pi*pints.pInt(-7.5,m,delta0))/130560. + ((3003*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 1386*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 315*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 140*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 15*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 6*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pInt(-6.5,m,delta0))/49920. - ((693*deltax5 + 315*deltax4*(deltay + deltaz) + 70*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 30*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 3*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pInt(-5.5,m,delta0))/9152. + ((315*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 140*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 30*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 12*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pInt(-4.5,m,delta0))/3168. - ((35*deltax3 + 15*deltax2*(deltay + deltaz) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(9*deltay2 + 6*deltay*deltaz + 9*deltaz2))*Pi*pints.pInt(-3.5,m,delta0))/252. + ((15*deltax2 + 3*deltay2 + 2*deltay*deltaz + 6*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pInt(-2.5,m,delta0))/70. - (2*(3*deltax + deltay + deltaz)*Pi*pints.pInt(-1.5,m,delta0))/15. + (4*Pi*pints.pInt(-0.5,m,delta0))/3.;
}

inline double Hfuncs::I3X_1(double ax, double ay, double az, double m){

	double delta0=1.;
	double deltax=ax*ax-delta0;
	double deltay=ay*ay-delta0;
	double deltaz=az*az-delta0;

	return genI3X(deltax,deltay,deltaz, m,delta0)/2.;
}

inline double Hfuncs::I3X_2(double ax, double ay, double az, double m){


	double delta0=4.;
	double deltax=ax*ax-delta0;
	double deltay=ay*ay-delta0;
	double deltaz=az*az-delta0;

	return genI3X(deltax,deltay,deltaz, m, delta0)/2.;
}

inline double Hfuncs::H3Xexp(double ax, double ay, double az, double m) {

	double rmin = 1.75;
 	double rmax = 1.85;	
	double r = sqrt(ax*ax+ay*ay+az*az);
	double alpha = ax*ay*az;
	if (r < rmin) return 2.*alpha*ax*ax*Nt*I3X_1(ax,ay,az,m);
	if (r > rmax) return 2.*alpha*ax*ax*Nt*I3X_2(ax,ay,az,m);
        else return 2*alpha*ax*ax*Nt*(10.*(rmax-r)*I3X_1(ax,ay,az,m) + 10.*(r-rmin)*I3X_2(ax,ay,az,m));
}

double Hfuncs::H3X(double ax, double ay, double az, double m) {
    return H3Xexp(ax,ay,az,m);
}


double Hfuncs::H3B(double ax, double ay, double az, double m) {

    return (H3(ax,ay,az,m)-H3X(ax,ay,az,m)-H3Y(ax,ay,az,m)-H3L(ax,ay,az,m))/(m*m);   
}


//**********************************************************************************************
// derivatives of H3X
//**********************************************************************************************
inline double Hfuncs::genI3X1(double deltax, double deltay, double deltaz, double m, double delta0){
    

    double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
    double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
    double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
    double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
    double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
    double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;
    
    return Sqrt(delta0 + deltay)*Sqrt(delta0 + deltaz)*((Power(delta0 + deltax,2)*Pi*(7436429*(16900975*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 7436429*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 1616615*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 692835*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 72930*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 30030*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 6006*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 2310*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 105*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 35*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 5*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*pints.pInt(-12.5,m,delta0) - 17459442*(7436429*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 3233230*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 692835*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 291720*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 30030*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 12012*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 2310*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 840*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 35*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 10*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*pints.pInt(-11.5,m,delta0) + 83140200*(1616615*deltax9 + 12155*deltay9 + 6435*deltay8*deltaz + 692835*deltax8*(deltay + deltaz) + 5148*deltay7*deltaz2 + 145860*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 4620*deltay6*deltaz3 + 60060*deltax6*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 4410*deltay5*deltaz4 + 6006*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 4410*deltay4*deltaz5 + 2310*deltax4*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 4620*deltay3*deltaz6 + 420*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 5148*deltay2*deltaz7 + 140*deltax2*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 6435*deltay*deltaz8 + 5*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 12155*deltaz9)*pints.pInt(-10.5,m,delta0) + 201286800*(-692835*deltax8 - 6435*deltay8 - 3432*deltay7*deltaz - 291720*deltax7*(deltay + deltaz) + 24024*deltax5*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 2772*deltay6*deltaz2 - 60060*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 2520*deltay5*deltaz3 + 840*deltax3*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 2450*deltay4*deltaz4 - 2310*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 2520*deltay3*deltaz5 + 40*deltax*(deltay + deltaz)*(-429*deltay6 + 198*deltay5*deltaz - 387*deltay4*deltaz2 + 212*deltay3*deltaz3 - 387*deltay2*deltaz4 + 198*deltay*deltaz5 - 429*deltaz6) - 2772*deltay2*deltaz6 - 140*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) - 3432*deltay*deltaz7 - 6435*deltaz8)*pints.pInt(-9.5,m,delta0) + 3978374400*(36465*deltax7 + 15015*deltax6*(deltay + deltaz) + 3003*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 1155*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 105*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 35*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 5*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*pints.pInt(-8.5,m,delta0) + 10078548480*(-15015*deltax6 - 231*deltay6 - 126*deltay5*deltaz - 6006*deltax5*(deltay + deltaz) + 420*deltax3*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 105*deltay4*deltaz2 - 1155*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 100*deltay3*deltaz3 + 10*deltax*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 105*deltay2*deltaz4 - 35*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 126*deltay*deltaz5 - 231*deltaz6)*pints.pInt(-7.5,m,delta0) + 52718561280*(3003*deltax5 + 1155*deltax4*(deltay + deltaz) + 210*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 70*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 5*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*pints.pInt(-6.5,m,delta0) - 143777894400*(1155*deltax4 + 210*deltax2*deltay2 + 100*deltax*deltay*deltay2 + 35*deltay4 + 140*deltax2*deltay*deltaz - 40*deltax*Power(deltay,2)*deltaz + 100*deltax*deltay2*deltaz + 20*deltay3*deltaz - 40*deltax*deltay*Power(deltaz,2) + 420*deltax3*(deltay + deltaz) + 210*deltax2*deltaz2 + 100*deltax*deltay*deltaz2 + 18*deltay2*deltaz2 + 100*deltax*deltaz*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4)*pints.pInt(-5.5,m,delta0) + 1661433446400*(105*deltax3 + 15*deltax*deltay2 + 5*deltay*deltay2 + 10*deltax*deltay*deltaz - 2*Power(deltay,2)*deltaz + 5*deltay2*deltaz - 2*deltay*Power(deltaz,2) + 35*deltax2*(deltay + deltaz) + 15*deltax*deltaz2 + 5*deltay*deltaz2 + 5*deltaz*deltaz2)*pints.pInt(-4.5,m,delta0) - 5221647974400*(35*deltax2 + 3*deltay2 + 2*deltay*deltaz + 10*deltax*(deltay + deltaz) + 3*deltaz2)*pints.pInt(-3.5,m,delta0) + 37595865415680*(5*deltax + deltay + deltaz)*pints.pInt(-2.5,m,delta0) - 175447371939840*pints.pInt(-1.5,m,delta0)))/2.193092149248e14 + 3*(delta0 + deltax)*(((16900975*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 8112468*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 1939938*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 243100*deltay9*deltaz3 + 923780*deltax9*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 225225*deltay8*deltaz4 + 109395*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 216216*deltay7*deltaz5 + 51480*deltax7*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 213444*deltay6*deltaz6 + 12012*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 216216*deltay5*deltaz7 + 5544*deltax5*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 225225*deltay4*deltaz8 + 315*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 243100*deltay3*deltaz9 + 140*deltax3*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 30*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9) + 12*deltax*(88179*deltay11 + 46189*deltay10*deltaz + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 32175*deltay8*deltaz3 + 30030*deltay7*deltaz4 + 29106*deltay6*deltaz5 + 29106*deltay5*deltaz6 + 30030*deltay4*deltaz7 + 32175*deltay3*deltaz8 + 36465*deltay2*deltaz9))*Pi*pints.pInt(-12.5,m,delta0))/7.077888e8 - ((2028117*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 969969*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 230945*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 109395*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 12870*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 6006*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 1386*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 630*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 35*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 15*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 3*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-11.5,m,delta0))/7.53664e7 + ((969969*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 461890*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 109395*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 51480*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 6006*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 2772*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 630*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 280*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 15*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 6*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*Pi*pints.pInt(-10.5,m,delta0))/3.1653888e7 + ((-230945*deltax9 - 109395*deltax8*(deltay + deltaz) + 12012*deltax6*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 25740*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 630*deltax4*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 1386*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 60*deltax2*(deltay + deltaz)*(-429*deltay6 + 198*deltay5*deltaz - 387*deltay4*deltaz2 + 212*deltay3*deltaz3 - 387*deltay2*deltaz4 + 198*deltay*deltaz5 - 429*deltaz6) - 140*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(-12155*deltay8 + 5720*deltay7*deltaz - 10868*deltay6*deltaz2 + 6248*deltay5*deltaz3 - 10658*deltay4*deltaz4 + 6248*deltay3*deltaz5 - 10868*deltay2*deltaz6 + 5720*deltay*deltaz7 - 12155*deltaz8) - 3*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8))*Pi*pints.pInt(-9.5,m,delta0))/6.537216e6 + ((109395*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 51480*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 12012*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 5544*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 630*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 280*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 60*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 24*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pInt(-8.5,m,delta0))/2.646016e6 + ((-6435*deltax7 - 3003*deltax6*(deltay + deltaz) + 315*deltax4*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 693*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 15*deltax2*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 35*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(-429*deltay6 + 198*deltay5*deltaz - 387*deltay4*deltaz2 + 212*deltay3*deltaz3 - 387*deltay2*deltaz4 + 198*deltay*deltaz5 - 429*deltaz6) - 3*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6))*Pi*pints.pInt(-7.5,m,delta0))/130560. + ((3003*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 1386*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 315*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 140*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 15*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 6*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pInt(-6.5,m,delta0))/49920. - ((693*deltax5 + 315*deltax4*(deltay + deltaz) + 70*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 30*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 3*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pInt(-5.5,m,delta0))/9152. + ((315*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 140*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 30*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 12*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pInt(-4.5,m,delta0))/3168. - ((35*deltax3 + 15*deltax2*(deltay + deltaz) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(9*deltay2 + 6*deltay*deltaz + 9*deltaz2))*Pi*pints.pInt(-3.5,m,delta0))/252. + ((15*deltax2 + 3*deltay2 + 2*deltay*deltaz + 6*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pInt(-2.5,m,delta0))/70. - (2*(3*deltax + deltay + deltaz)*Pi*pints.pInt(-1.5,m,delta0))/15. + (4*Pi*pints.pInt(-0.5,m,delta0))/3.));
}

inline double Hfuncs::I3X1_1(double ax, double ay, double az, double m){
    
    double delta0=1.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3X1(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::I3X1_2(double ax, double ay, double az, double m){
    
    
    double delta0=4.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3X1(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::d1H3Xexp(double ax, double ay, double az, double m) {
    
    double rmin = 1.75;
    double rmax = 1.85;
    double r = sqrt(ax*ax+ay*ay+az*az);
    double alpha = ax*ay*az;
    if (r < rmin) return Nt*I3X1_1(ax,ay,az,m);
    if (r > rmax) return Nt*I3X1_2(ax,ay,az,m);
    else return Nt*(10.*(rmax-r)*I3X1_1(ax,ay,az,m) + 10.*(r-rmin)*I3X1_2(ax,ay,az,m));
    
}

double Hfuncs::d1H3X(double ax, double ay, double az, double m) {
     return d1H3Xexp(ax,ay,az,m);

}

//*******************************************************************


inline double Hfuncs::genI3X2(double deltax, double deltay, double deltaz, double m, double delta0){
    
    
    double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
    double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
    double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
    double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
    double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
    double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;
    
    return Power(delta0 + deltax,1.5)*Sqrt(delta0 + deltaz)*(((16900975*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 8112468*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 1939938*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 243100*deltay9*deltaz3 + 923780*deltax9*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 225225*deltay8*deltaz4 + 109395*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 216216*deltay7*deltaz5 + 51480*deltax7*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 213444*deltay6*deltaz6 + 12012*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 216216*deltay5*deltaz7 + 5544*deltax5*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 225225*deltay4*deltaz8 + 315*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 243100*deltay3*deltaz9 + 140*deltax3*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 30*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9) + 12*deltax*(88179*deltay11 + 46189*deltay10*deltaz + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 32175*deltay8*deltaz3 + 30030*deltay7*deltaz4 + 29106*deltay6*deltaz5 + 29106*deltay5*deltaz6 + 30030*deltay4*deltaz7 + 32175*deltay3*deltaz8 + 36465*deltay2*deltaz9))*Pi*pints.pInt(-12.5,m,delta0))/7.077888e8 - ((2028117*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 969969*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 230945*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 109395*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 12870*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 6006*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 1386*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 630*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 35*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 15*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 3*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pInt(-11.5,m,delta0))/7.53664e7 + ((969969*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 461890*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 109395*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 51480*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 6006*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 2772*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 630*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 280*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 15*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 6*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*Pi*pints.pInt(-10.5,m,delta0))/3.1653888e7 + ((-230945*deltax9 - 109395*deltax8*(deltay + deltaz) + 12012*deltax6*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 25740*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 630*deltax4*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 1386*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 60*deltax2*(deltay + deltaz)*(-429*deltay6 + 198*deltay5*deltaz - 387*deltay4*deltaz2 + 212*deltay3*deltaz3 - 387*deltay2*deltaz4 + 198*deltay*deltaz5 - 429*deltaz6) - 140*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + (deltay + deltaz)*(-12155*deltay8 + 5720*deltay7*deltaz - 10868*deltay6*deltaz2 + 6248*deltay5*deltaz3 - 10658*deltay4*deltaz4 + 6248*deltay3*deltaz5 - 10868*deltay2*deltaz6 + 5720*deltay*deltaz7 - 12155*deltaz8) - 3*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8))*Pi*pints.pInt(-9.5,m,delta0))/6.537216e6 + ((109395*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 51480*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 12012*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 5544*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 630*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 280*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 60*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 24*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pInt(-8.5,m,delta0))/2.646016e6 + ((-6435*deltax7 - 3003*deltax6*(deltay + deltaz) + 315*deltax4*(deltay + deltaz)*(-5*deltay2 + 2*deltay*deltaz - 5*deltaz2) - 693*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 15*deltax2*(deltay + deltaz)*(-63*deltay4 + 28*deltay3*deltaz - 58*deltay2*deltaz2 + 28*deltay*deltaz3 - 63*deltaz4) - 35*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(-429*deltay6 + 198*deltay5*deltaz - 387*deltay4*deltaz2 + 212*deltay3*deltaz3 - 387*deltay2*deltaz4 + 198*deltay*deltaz5 - 429*deltaz6) - 3*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6))*Pi*pints.pInt(-7.5,m,delta0))/130560. + ((3003*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 1386*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 315*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 140*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 15*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 6*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pInt(-6.5,m,delta0))/49920. - ((693*deltax5 + 315*deltax4*(deltay + deltaz) + 70*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 30*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 3*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pInt(-5.5,m,delta0))/9152. + ((315*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 140*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 30*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 12*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pInt(-4.5,m,delta0))/3168. - ((35*deltax3 + 15*deltax2*(deltay + deltaz) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(9*deltay2 + 6*deltay*deltaz + 9*deltaz2))*Pi*pints.pInt(-3.5,m,delta0))/252. + ((15*deltax2 + 3*deltay2 + 2*deltay*deltaz + 6*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pInt(-2.5,m,delta0))/70. + ((delta0 + deltay)*Pi*(7436429*(676039*deltax11 + 969969*deltax*deltay10 + 676039*deltay11 + 1154725*deltax9*deltay2 + 1276275*deltax8*deltay3 + 1351350*deltax7*deltay4 + 1387386*deltax6*deltay5 + 1387386*deltax5*deltay6 + 1351350*deltax4*deltay7 + 1276275*deltax3*deltay8 + 1154725*deltax2*deltay9 + 461890*deltax9*deltay*deltaz + 323323*deltay10*deltaz + 546975*deltax8*deltay2*deltaz + 600600*deltax7*deltay3*deltaz + 630630*deltax6*deltay4*deltaz + 640332*deltax5*deltay5*deltaz + 630630*deltax4*deltay6*deltaz + 600600*deltax3*deltay7*deltaz + 546975*deltax2*deltay8*deltaz + 461890*deltax*deltay9*deltaz + 323323*deltax10*(3*deltay + deltaz) + 46189*deltax*deltaz10 + 46189*deltay*deltaz10 + 29393*deltaz11 + 230945*deltax9*deltaz2 + 328185*deltax8*deltay*deltaz2 + 386100*deltax7*deltay2*deltaz2 + 420420*deltax6*deltay3*deltaz2 + 436590*deltax5*deltay4*deltaz2 + 436590*deltax4*deltay5*deltaz2 + 420420*deltax3*deltay6*deltaz2 + 386100*deltax2*deltay7*deltaz2 + 328185*deltax*deltay8*deltaz2 + 230945*deltay9*deltaz2 + 182325*deltax8*deltaz3 + 257400*deltax7*deltay*deltaz3 + 300300*deltax6*deltay2*deltaz3 + 323400*deltax5*deltay3*deltaz3 + 330750*deltax4*deltay4*deltaz3 + 323400*deltax3*deltay5*deltaz3 + 300300*deltax2*deltay6*deltaz3 + 257400*deltax*deltay7*deltaz3 + 182325*deltay8*deltaz3 + 150150*deltax7*deltaz4 + 210210*deltax6*deltay*deltaz4 + 242550*deltax5*deltay2*deltaz4 + 257250*deltax4*deltay3*deltaz4 + 257250*deltax3*deltay4*deltaz4 + 242550*deltax2*deltay5*deltaz4 + 210210*deltax*deltay6*deltaz4 + 150150*deltay7*deltaz4 + 126126*deltax6*deltaz5 + 174636*deltax5*deltay*deltaz5 + 198450*deltax4*deltay2*deltaz5 + 205800*deltax3*deltay3*deltaz5 + 198450*deltax2*deltay4*deltaz5 + 174636*deltax*deltay5*deltaz5 + 126126*deltay6*deltaz5 + 106722*deltax5*deltaz6 + 145530*deltax4*deltay*deltaz6 + 161700*deltax3*deltay2*deltaz6 + 161700*deltax2*deltay3*deltaz6 + 145530*deltax*deltay4*deltaz6 + 106722*deltay5*deltaz6 + 90090*deltax4*deltaz7 + 120120*deltax3*deltay*deltaz7 + 128700*deltax2*deltay2*deltaz7 + 120120*deltax*deltay3*deltaz7 + 90090*deltay4*deltaz7 + 75075*deltax3*deltaz8 + 96525*deltax2*deltay*deltaz8 + 96525*deltax*deltay2*deltaz8 + 75075*deltay3*deltaz8 + 60775*deltax2*deltaz9 + 72930*deltax*deltay*deltaz9 + 60775*deltay2*deltaz9)*pints.pInt(-12.5,m,delta0) - 5819814*(969969*deltax10 + 969969*deltay10 + 1640925*deltax8*deltay2 + 1801800*deltax7*deltay3 + 1891890*deltax6*deltay4 + 1920996*deltax5*deltay5 + 1891890*deltax4*deltay6 + 1801800*deltax3*deltay7 + 1640925*deltax2*deltay8 + 1385670*deltax*deltay9 + 656370*deltax8*deltay*deltaz + 772200*deltax7*deltay2*deltaz + 840840*deltax6*deltay3*deltaz + 873180*deltax5*deltay4*deltaz + 873180*deltax4*deltay5*deltaz + 840840*deltax3*deltay6*deltaz + 772200*deltax2*deltay7*deltaz + 656370*deltax*deltay8*deltaz + 461890*deltay9*deltaz + 461890*deltax9*(3*deltay + deltaz) + 46189*deltaz10 + 328185*deltax8*deltaz2 + 463320*deltax7*deltay*deltaz2 + 540540*deltax6*deltay2*deltaz2 + 582120*deltax5*deltay3*deltaz2 + 595350*deltax4*deltay4*deltaz2 + 582120*deltax3*deltay5*deltaz2 + 540540*deltax2*deltay6*deltaz2 + 463320*deltax*deltay7*deltaz2 + 328185*deltay8*deltaz2 + 257400*deltax7*deltaz3 + 360360*deltax6*deltay*deltaz3 + 415800*deltax5*deltay2*deltaz3 + 441000*deltax4*deltay3*deltaz3 + 441000*deltax3*deltay4*deltaz3 + 415800*deltax2*deltay5*deltaz3 + 360360*deltax*deltay6*deltaz3 + 257400*deltay7*deltaz3 + 210210*deltax6*deltaz4 + 291060*deltax5*deltay*deltaz4 + 330750*deltax4*deltay2*deltaz4 + 343000*deltax3*deltay3*deltaz4 + 330750*deltax2*deltay4*deltaz4 + 291060*deltax*deltay5*deltaz4 + 210210*deltay6*deltaz4 + 174636*deltax5*deltaz5 + 238140*deltax4*deltay*deltaz5 + 264600*deltax3*deltay2*deltaz5 + 264600*deltax2*deltay3*deltaz5 + 238140*deltax*deltay4*deltaz5 + 174636*deltay5*deltaz5 + 145530*deltax4*deltaz6 + 194040*deltax3*deltay*deltaz6 + 207900*deltax2*deltay2*deltaz6 + 194040*deltax*deltay3*deltaz6 + 145530*deltay4*deltaz6 + 120120*deltax3*deltaz7 + 154440*deltax2*deltay*deltaz7 + 154440*deltax*deltay2*deltaz7 + 120120*deltay3*deltaz7 + 96525*deltax2*deltaz8 + 115830*deltax*deltay*deltaz8 + 96525*deltay2*deltaz8 + 72930*deltax*deltaz9 + 72930*deltay*deltaz9)*pints.pInt(-11.5,m,delta0) + 27713400*(230945*deltax9 + 386100*deltax7*deltay2 + 420420*deltax6*deltay3 + 436590*deltax5*deltay4 + 436590*deltax4*deltay5 + 420420*deltax3*deltay6 + 386100*deltax2*deltay7 + 328185*deltax*deltay8 + 230945*deltay9 + 154440*deltax7*deltay*deltaz + 180180*deltax6*deltay2*deltaz + 194040*deltax5*deltay3*deltaz + 198450*deltax4*deltay4*deltaz + 194040*deltax3*deltay5*deltaz + 180180*deltax2*deltay6*deltaz + 154440*deltax*deltay7*deltaz + 109395*deltay8*deltaz + 109395*deltax8*(3*deltay + deltaz) + 77220*deltax7*deltaz2 + 108108*deltax6*deltay*deltaz2 + 124740*deltax5*deltay2*deltaz2 + 132300*deltax4*deltay3*deltaz2 + 132300*deltax3*deltay4*deltaz2 + 124740*deltax2*deltay5*deltaz2 + 108108*deltax*deltay6*deltaz2 + 77220*deltay7*deltaz2 + 60060*deltax6*deltaz3 + 83160*deltax5*deltay*deltaz3 + 94500*deltax4*deltay2*deltaz3 + 98000*deltax3*deltay3*deltaz3 + 94500*deltax2*deltay4*deltaz3 + 83160*deltax*deltay5*deltaz3 + 60060*deltay6*deltaz3 + 48510*deltax5*deltaz4 + 66150*deltax4*deltay*deltaz4 + 73500*deltax3*deltay2*deltaz4 + 73500*deltax2*deltay3*deltaz4 + 66150*deltax*deltay4*deltaz4 + 48510*deltay5*deltaz4 + 39690*deltax4*deltaz5 + 52920*deltax3*deltay*deltaz5 + 56700*deltax2*deltay2*deltaz5 + 52920*deltax*deltay3*deltaz5 + 39690*deltay4*deltaz5 + 32340*deltax3*deltaz6 + 41580*deltax2*deltay*deltaz6 + 41580*deltax*deltay2*deltaz6 + 32340*deltay3*deltaz6 + 25740*deltax2*deltaz7 + 30888*deltax*deltay*deltaz7 + 25740*deltay2*deltaz7 + 19305*deltax*deltaz8 + 19305*deltay*deltaz8 + 12155*deltaz9)*pints.pInt(-10.5,m,delta0) - 67095600*(109395*deltax8 + 180180*deltax6*deltay2 + 194040*deltax5*deltay3 + 158760*deltax4*deltay*deltay3 + 39690*deltax4*deltay4 + 194040*deltax3*deltay5 + 154440*deltax2*deltay*deltay5 + 25740*deltax2*deltay6 + 154440*deltax*deltay7 + 97240*deltay*deltay7 + 12155*deltay8 + 72072*deltax6*deltay*deltaz + 83160*deltax5*deltay2*deltaz - 52920*deltax4*deltay*deltay2*deltaz + 141120*deltax4*deltay3*deltaz + 88200*deltax3*deltay4*deltaz - 59400*deltax2*deltay*deltay4*deltaz + 142560*deltax2*deltay5*deltaz + 72072*deltax*deltay6*deltaz - 40040*deltay*deltay6*deltaz + 91520*deltay7*deltaz + 51480*deltax7*(3*deltay + deltaz) + 36036*deltax6*deltaz2 + 49896*deltax5*deltay*deltaz2 + 56700*deltax4*deltay2*deltaz2 + 58800*deltax3*deltay3*deltaz2 + 92880*deltax2*deltay*deltay3*deltaz2 - 36180*deltax2*deltay4*deltaz2 + 49896*deltax*deltay5*deltaz2 + 65208*deltay*deltay5*deltaz2 - 29172*deltay6*deltaz2 + 73080*deltax4*deltay*deltaz*deltaz2 + 92880*deltax2*deltay3*deltaz*deltaz2 + 65208*deltay5*deltaz*deltaz2 + 27720*deltax5*deltaz3 - 35280*deltax4*deltay*deltaz3 + 42000*deltax3*deltay2*deltaz3 - 38160*deltax2*deltay*deltay2*deltaz3 - 12720*deltax2*deltay3*deltaz3 + 37800*deltax*deltay4*deltaz3 - 31240*deltay*deltay4*deltaz3 - 6248*deltay5*deltaz3 - 17640*deltax4*deltaz*deltaz3 - 38160*deltax2*deltay2*deltaz*deltaz3 - 31240*deltay4*deltaz*deltaz3 + 39690*deltax4*deltaz4 + 29400*deltax3*deltay*deltaz4 + 69660*deltax2*deltay2*deltaz4 + 29400*deltax*deltay3*deltaz4 + 42632*deltay*deltay3*deltaz4 + 10658*deltay4*deltaz4 + 46440*deltax2*deltay*deltaz*deltaz4 + 42632*deltay3*deltaz*deltaz4 + 17640*deltax3*deltaz5 - 23760*deltax2*deltay*deltaz5 + 22680*deltax*deltay2*deltaz5 - 18744*deltay*deltay2*deltaz5 - 6248*deltay3*deltaz5 - 11880*deltax2*deltaz*deltaz5 - 18744*deltay2*deltaz*deltaz5 + 25740*deltax2*deltaz6 + 16632*deltax*deltay*deltaz6 + 32604*deltay2*deltaz6 + 21736*deltay*deltaz*deltaz6 + 10296*deltax*deltaz7 - 11440*deltay*deltaz7 - 5720*deltaz*deltaz7 + 12155*deltaz8)*pints.pInt(-9.5,m,delta0) + 1326124800*(6435*deltax7 + 10395*deltax5*deltay2 + 11025*deltax4*deltay3 + 8820*deltax3*deltay*deltay3 + 2205*deltax3*deltay4 + 10395*deltax2*deltay5 + 7722*deltax*deltay*deltay5 + 1287*deltax*deltay6 + 6435*deltay7 + 4158*deltax5*deltay*deltaz + 4725*deltax4*deltay2*deltaz - 2940*deltax3*deltay*deltay2*deltaz + 7840*deltax3*deltay3*deltaz + 4725*deltax2*deltay4*deltaz - 2970*deltax*deltay*deltay4*deltaz + 7128*deltax*deltay5*deltaz + 3003*deltay6*deltaz + 3003*deltax6*(3*deltay + deltaz) + 2079*deltax5*deltaz2 + 2835*deltax4*deltay*deltaz2 + 3150*deltax3*deltay2*deltaz2 + 3150*deltax2*deltay3*deltaz2 + 4644*deltax*deltay*deltay3*deltaz2 - 1809*deltax*deltay4*deltaz2 + 2079*deltay5*deltaz2 + 4060*deltax3*deltay*deltaz*deltaz2 + 4644*deltax*deltay3*deltaz*deltaz2 + 1575*deltax4*deltaz3 - 1960*deltax3*deltay*deltaz3 + 2250*deltax2*deltay2*deltaz3 - 1908*deltax*deltay*deltay2*deltaz3 - 636*deltax*deltay3*deltaz3 + 1575*deltay4*deltaz3 - 980*deltax3*deltaz*deltaz3 - 1908*deltax*deltay2*deltaz*deltaz3 + 2205*deltax3*deltaz4 + 1575*deltax2*deltay*deltaz4 + 3483*deltax*deltay2*deltaz4 + 1225*deltay3*deltaz4 + 2322*deltax*deltay*deltaz*deltaz4 + 945*deltax2*deltaz5 - 1188*deltax*deltay*deltaz5 + 945*deltay2*deltaz5 - 594*deltax*deltaz*deltaz5 + 1287*deltax*deltaz6 + 693*deltay*deltaz6 + 429*deltaz7)*pints.pInt(-8.5,m,delta0) - 3359516160*(3003*deltax6 + 4725*deltax4*deltay2 + 4900*deltax3*deltay3 + 3780*deltax2*deltay*deltay3 + 945*deltax2*deltay4 + 4158*deltax*deltay5 + 2574*deltay*deltay5 + 429*deltay6 + 1890*deltax4*deltay*deltaz + 2100*deltax3*deltay2*deltaz - 1260*deltax2*deltay*deltay2*deltaz + 3360*deltax2*deltay3*deltaz + 1890*deltax*deltay4*deltaz - 990*deltay*deltay4*deltaz + 2376*deltay5*deltaz + 1386*deltax5*(3*deltay + deltaz) + 945*deltax4*deltaz2 + 1260*deltax3*deltay*deltaz2 + 1350*deltax2*deltay2*deltaz2 + 1260*deltax*deltay3*deltaz2 + 1548*deltay*deltay3*deltaz2 - 603*deltay4*deltaz2 + 1740*deltax2*deltay*deltaz*deltaz2 + 1548*deltay3*deltaz*deltaz2 + 700*deltax3*deltaz3 - 840*deltax2*deltay*deltaz3 + 900*deltax*deltay2*deltaz3 - 636*deltay*deltay2*deltaz3 - 212*deltay3*deltaz3 - 420*deltax2*deltaz*deltaz3 - 636*deltay2*deltaz*deltaz3 + 945*deltax2*deltaz4 + 630*deltax*deltay*deltaz4 + 1161*deltay2*deltaz4 + 774*deltay*deltaz*deltaz4 + 378*deltax*deltaz5 - 396*deltay*deltaz5 - 198*deltaz*deltaz5 + 429*deltaz6)*pints.pInt(-7.5,m,delta0) + 17572853760*(693*deltax5 + 1050*deltax3*deltay2 + 1050*deltax2*deltay3 + 756*deltax*deltay*deltay3 + 189*deltax*deltay4 + 693*deltay5 + 420*deltax3*deltay*deltaz + 450*deltax2*deltay2*deltaz - 252*deltax*deltay*deltay2*deltaz + 672*deltax*deltay3*deltaz + 315*deltay4*deltaz + 315*deltax4*(3*deltay + deltaz) + 210*deltax3*deltaz2 + 270*deltax2*deltay*deltaz2 + 270*deltax*deltay2*deltaz2 + 210*deltay3*deltaz2 + 348*deltax*deltay*deltaz*deltaz2 + 150*deltax2*deltaz3 - 168*deltax*deltay*deltaz3 + 150*deltay2*deltaz3 - 84*deltax*deltaz*deltaz3 + 189*deltax*deltaz4 + 105*deltay*deltaz4 + 63*deltaz5)*pints.pInt(-6.5,m,delta0) - 47925964800*(315*deltax4 + 450*deltax2*deltay2 + 420*deltax*deltay3 + 252*deltay*deltay3 + 63*deltay4 + 180*deltax2*deltay*deltaz + 180*deltax*deltay2*deltaz - 84*deltay*deltay2*deltaz + 224*deltay3*deltaz + 140*deltax3*(3*deltay + deltaz) + 90*deltax2*deltaz2 + 108*deltax*deltay*deltaz2 + 90*deltay2*deltaz2 + 116*deltay*deltaz*deltaz2 + 60*deltax*deltaz3 - 56*deltay*deltaz3 - 28*deltaz*deltaz3 + 63*deltaz4)*pints.pInt(-5.5,m,delta0) + 553811148800*(35*deltax3 + 45*deltax*deltay2 + 35*deltay3 + 18*deltax*deltay*deltaz + 15*deltay2*deltaz + 15*deltax2*(3*deltay + deltaz) + 9*deltax*deltaz2 + 9*deltay*deltaz2 + 5*deltaz3)*pints.pInt(-4.5,m,delta0) - 1740549324800*(15*deltax2 + 15*deltay2 + 6*deltay*deltaz + 6*deltax*(3*deltay + deltaz) + 3*deltaz2)*pints.pInt(-3.5,m,delta0) + 12531955138560*(3*deltax + 3*deltay + deltaz)*pints.pInt(-2.5,m,delta0) - 58482457313280*pints.pInt(-1.5,m,delta0)))/2.193092149248e14 - (2*(3*deltax + deltay + deltaz)*Pi*pints.pInt(-1.5,m,delta0))/15. + (4*Pi*pints.pInt(-0.5,m,delta0))/3.);
    
    
}

inline double Hfuncs::I3X2_1(double ax, double ay, double az, double m){
    
    double delta0=1.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3X2(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::I3X2_2(double ax, double ay, double az, double m){
    
    
    double delta0=4.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3X2(deltax,deltay,deltaz, m,delta0);
}

inline double Hfuncs::d2H3Xexp(double ax, double ay, double az, double m) {
    
    double rmin = 1.75;
    double rmax = 1.85;
    double r = sqrt(ax*ax+ay*ay+az*az);
    double alpha = ax*ay*az;
    if (r < rmin) return Nt*I3X2_1(ax,ay,az,m);
    if (r > rmax) return Nt*I3X2_2(ax,ay,az,m);
    else return Nt*(10.*(rmax-r)*I3X2_1(ax,ay,az,m) + 10.*(r-rmin)*I3X2_2(ax,ay,az,m));
    
}

double Hfuncs::d2H3X(double ax, double ay, double az, double m) {
    return d2H3Xexp(ax,ay,az,m);

}

double Hfuncs::d3H3X(double ax, double ay, double az, double m) {
    return d2H3Xexp(ax,az,ay,m);

}

//****************************************

inline double Hfuncs::genI3Xm(double deltax, double deltay, double deltaz, double m, double delta0){
    
    double deltax2=deltax*deltax, deltax3=deltax2*deltax, deltax4=deltax3*deltax, deltax5=deltax4*deltax, deltax6=deltax5*deltax, deltax7=deltax6*deltax, deltax8=deltax7*deltax;
    double deltax9=deltax8*deltax, deltax10=deltax9*deltax, deltax11=deltax10*deltax, deltax12=deltax11*deltax;
    double deltay2=deltay*deltay, deltay3=deltay2*deltay, deltay4=deltay3*deltay, deltay5=deltay4*deltay, deltay6=deltay5*deltay, deltay7=deltay6*deltay, deltay8=deltay7*deltay;
    double deltay9=deltay8*deltay, deltay10=deltay9*deltay, deltay11=deltay10*deltay, deltay12=deltay11*deltay;
    double deltaz2=deltaz*deltaz, deltaz3=deltaz2*deltaz, deltaz4=deltaz3*deltaz, deltaz5=deltaz4*deltaz, deltaz6=deltaz5*deltaz, deltaz7=deltaz6*deltaz, deltaz8=deltaz7*deltaz;
    double deltaz9=deltaz8*deltaz, deltaz10=deltaz9*deltaz, deltaz11=deltaz10*deltaz, deltaz12=deltaz11*deltaz;
    
    return ((16900975*deltax12 + 676039*deltay12 + 352716*deltay11*deltaz + 8112468*deltax11*(deltay + deltaz) + 277134*deltay2*deltaz10 + 352716*deltay*deltaz11 + 676039*deltaz12 + 277134*deltay10*deltaz2 + 1939938*deltax10*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 243100*deltay9*deltaz3 + 923780*deltax9*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 225225*deltay8*deltaz4 + 109395*deltax8*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 216216*deltay7*deltaz5 + 51480*deltax7*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 213444*deltay6*deltaz6 + 12012*deltax6*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 216216*deltay5*deltaz7 + 5544*deltax5*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 225225*deltay4*deltaz8 + 315*deltax4*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 243100*deltay3*deltaz9 + 140*deltax3*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 30*deltax2*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9) + 12*deltax*(88179*deltay11 + 46189*deltay10*deltaz + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 32175*deltay8*deltaz3 + 30030*deltay7*deltaz4 + 29106*deltay6*deltaz5 + 29106*deltay5*deltaz6 + 30030*deltay4*deltaz7 + 32175*deltay3*deltaz8 + 36465*deltay2*deltaz9))*Pi*pints.pmInt(-12.5,m,delta0))/7.077888e8 - ((2028117*deltax11 + 88179*deltay11 + 46189*deltay10*deltaz + 969969*deltax10*(deltay + deltaz) + 46189*deltay*deltaz10 + 88179*deltaz11 + 36465*deltay9*deltaz2 + 230945*deltax9*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 32175*deltay8*deltaz3 + 109395*deltax8*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 30030*deltay7*deltaz4 + 12870*deltax7*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 29106*deltay6*deltaz5 + 6006*deltax6*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 29106*deltay5*deltaz6 + 1386*deltax5*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 30030*deltay4*deltaz7 + 630*deltax4*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 32175*deltay3*deltaz8 + 35*deltax3*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 36465*deltay2*deltaz9 + 15*deltax2*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9) + 3*deltax*(46189*deltay10 + 24310*deltay9*deltaz + 46189*deltaz10 + 19305*deltay8*deltaz2 + 17160*deltay7*deltaz3 + 16170*deltay6*deltaz4 + 15876*deltay5*deltaz5 + 16170*deltay4*deltaz6 + 17160*deltay3*deltaz7 + 19305*deltay2*deltaz8 + 24310*deltay*deltaz9))*Pi*pints.pmInt(-11.5,m,delta0))/7.53664e7 + ((969969*deltax10 + 46189*deltay10 + 24310*deltay9*deltaz + 461890*deltax9*(deltay + deltaz) + 46189*deltaz10 + 19305*deltay8*deltaz2 + 109395*deltax8*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 17160*deltay7*deltaz3 + 51480*deltax7*(5*deltay3 + 3*deltay2*deltaz + 3*deltay*deltaz2 + 5*deltaz3) + 16170*deltay6*deltaz4 + 6006*deltax6*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 15876*deltay5*deltaz5 + 2772*deltax5*(63*deltay5 + 35*deltay4*deltaz + 30*deltay3*deltaz2 + 30*deltay2*deltaz3 + 35*deltay*deltaz4 + 63*deltaz5) + 16170*deltay4*deltaz6 + 630*deltax4*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 17160*deltay3*deltaz7 + 280*deltax3*(429*deltay7 + 231*deltay6*deltaz + 189*deltay5*deltaz2 + 175*deltay4*deltaz3 + 175*deltay3*deltaz4 + 189*deltay2*deltaz5 + 231*deltay*deltaz6 + 429*deltaz7) + 19305*deltay2*deltaz8 + 15*deltax2*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) + 24310*deltay*deltaz9 + 6*deltax*(12155*deltay9 + 6435*deltay8*deltaz + 5148*deltay7*deltaz2 + 4620*deltay6*deltaz3 + 4410*deltay5*deltaz4 + 4410*deltay4*deltaz5 + 4620*deltay3*deltaz6 + 5148*deltay2*deltaz7 + 6435*deltay*deltaz8 + 12155*deltaz9))*Pi*pints.pmInt(-10.5,m,delta0))/3.1653888e7 + ((-230945*deltax9 - 109395*deltax8*(deltay + deltaz) - 25740*deltax7*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 12012*deltax6*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) - 1386*deltax5*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 630*deltax4*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) - 140*deltax3*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) - 60*deltax2*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) - 3*deltax*(6435*deltay8 + 3432*deltay7*deltaz + 2772*deltay6*deltaz2 + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 3432*deltay*deltaz7 + 6435*deltaz8) - (deltay + deltaz)*(12155*deltay8 - 5720*deltay7*deltaz + 10868*deltay6*deltaz2 - 6248*deltay5*deltaz3 + 10658*deltay4*deltaz4 - 6248*deltay3*deltaz5 + 10868*deltay2*deltaz6 - 5720*deltay*deltaz7 + 12155*deltaz8))*Pi*pints.pmInt(-9.5,m,delta0))/6.537216e6 + ((109395*deltax8 + 6435*deltay8 + 3432*deltay7*deltaz + 51480*deltax7*(deltay + deltaz) + 2772*deltay6*deltaz2 + 12012*deltax6*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 5544*deltax5*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 2520*deltay5*deltaz3 + 2450*deltay4*deltaz4 + 630*deltax4*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 280*deltax3*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 2520*deltay3*deltaz5 + 2772*deltay2*deltaz6 + 60*deltax2*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) + 24*deltax*(deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6) + 3432*deltay*deltaz7 + 6435*deltaz8)*Pi*pints.pmInt(-8.5,m,delta0))/2.646016e6 + ((-6435*deltax7 - 3003*deltax6*(deltay + deltaz) - 693*deltax5*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) - 315*deltax4*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) - 35*deltax3*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) - 15*deltax2*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) - 3*deltax*(231*deltay6 + 126*deltay5*deltaz + 105*deltay4*deltaz2 + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 126*deltay*deltaz5 + 231*deltaz6) - (deltay + deltaz)*(429*deltay6 - 198*deltay5*deltaz + 387*deltay4*deltaz2 - 212*deltay3*deltaz3 + 387*deltay2*deltaz4 - 198*deltay*deltaz5 + 429*deltaz6))*Pi*pints.pmInt(-7.5,m,delta0))/130560. + ((3003*deltax6 + 231*deltay6 + 126*deltay5*deltaz + 1386*deltax5*(deltay + deltaz) + 105*deltay4*deltaz2 + 315*deltax4*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 140*deltax3*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 100*deltay3*deltaz3 + 105*deltay2*deltaz4 + 15*deltax2*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + 6*deltax*(deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4) + 126*deltay*deltaz5 + 231*deltaz6)*Pi*pints.pmInt(-6.5,m,delta0))/49920. - ((693*deltax5 + 315*deltax4*(deltay + deltaz) + 70*deltax3*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 30*deltax2*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 3*deltax*(35*deltay4 + 20*deltay3*deltaz + 18*deltay2*deltaz2 + 20*deltay*deltaz3 + 35*deltaz4) + (deltay + deltaz)*(63*deltay4 - 28*deltay3*deltaz + 58*deltay2*deltaz2 - 28*deltay*deltaz3 + 63*deltaz4))*Pi*pints.pmInt(-5.5,m,delta0))/9152. + ((315*deltax4 + 35*deltay4 + 20*deltay3*deltaz + 140*deltax3*(deltay + deltaz) + 18*deltay2*deltaz2 + 30*deltax2*(3*deltay2 + 2*deltay*deltaz + 3*deltaz2) + 12*deltax*(deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + 20*deltay*deltaz3 + 35*deltaz4)*Pi*pints.pmInt(-4.5,m,delta0))/3168. - ((35*deltax3 + 15*deltax2*(deltay + deltaz) + (deltay + deltaz)*(5*deltay2 - 2*deltay*deltaz + 5*deltaz2) + deltax*(9*deltay2 + 6*deltay*deltaz + 9*deltaz2))*Pi*pints.pmInt(-3.5,m,delta0))/252. + ((15*deltax2 + 3*deltay2 + 2*deltay*deltaz + 6*deltax*(deltay + deltaz) + 3*deltaz2)*Pi*pints.pmInt(-2.5,m,delta0))/70. - (2*(3*deltax + deltay + deltaz)*Pi*pints.pmInt(-1.5,m,delta0))/15. + (4*Pi*pints.pmInt(-0.5,m,delta0))/3.;
}

inline double Hfuncs::I3Xm_1(double ax, double ay, double az, double m){
    
    double delta0=1.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3Xm(deltax,deltay,deltaz, m,delta0)/2.;
}

inline double Hfuncs::I3Xm_2(double ax, double ay, double az, double m){
    
    
    double delta0=4.;
    double deltax=ax*ax-delta0;
    double deltay=ay*ay-delta0;
    double deltaz=az*az-delta0;
    
    return genI3Xm(deltax,deltay,deltaz, m, delta0)/2.;
}

inline double Hfuncs::d4H3Xexp(double ax, double ay, double az, double m) {
    
    double rmin = 1.75;
    double rmax = 1.85;
    double r = sqrt(ax*ax+ay*ay+az*az);
    double alpha = ax*ay*az;
    if (r < rmin) return (2.*m)*2.*alpha*ax*ax*Nt*I3Xm_1(ax,ay,az,m);
    if (r > rmax) return (2.*m)*2.*alpha*ax*ax*Nt*I3Xm_2(ax,ay,az,m);
    else return (2.*m)*2*alpha*ax*ax*Nt*(10.*(rmax-r)*I3Xm_1(ax,ay,az,m) + 10.*(r-rmin)*I3Xm_2(ax,ay,az,m));
    
}

double Hfuncs::d4H3X(double ax, double ay, double az, double m) {
    return d4H3Xexp(ax,ay,az,m);

}

//****************************************************************************************************
//  H3Y and H3L and their derivatives
//****************************************************************************************************


double Hfuncs::H3Y(double ax, double ay, double az, double m) {
    return H3Xexp(ay,ax,az,m);
}

double Hfuncs::d1H3Y(double ax, double ay, double az, double m) {
    return d2H3X(ay,ax,az,m);
}

double Hfuncs::d2H3Y(double ax, double ay, double az, double m) {
    return d1H3X(ay,ax,az,m);
}

double Hfuncs::d3H3Y(double ax, double ay, double az, double m) {
    return d3H3X(ay,ax,az,m);
}

double Hfuncs::d4H3Y(double ax, double ay, double az, double m) {
    return d4H3X(ay,ax,az,m);
}

//*************

double Hfuncs::H3L(double ax, double ay, double az, double m) {
    return H3Xexp(az,ay,ax,m);
}

double Hfuncs::d1H3L(double ax, double ay, double az, double m) {
    return d3H3X(az,ay,ax,m);
}

double Hfuncs::d2H3L(double ax, double ay, double az, double m) {
    return d2H3X(az,ay,ax,m);
}

double Hfuncs::d3H3L(double ax, double ay, double az, double m) {
    return d1H3X(az,ay,ax,m);
}

double Hfuncs::d4H3L(double ax, double ay, double az, double m) {
    return d4H3X(az,ay,ax,m);
}



