/**
*
*   HyperCube.cxx
*
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#include "Global.h"

HyperCube::HyperCube(){

  mHyp = new double*** [2];
    for (int i1 = 0; i1 < 2; ++i1) {
      mHyp[i1] = new double** [2];
      for (int i2 = 0; i2 < 2; ++i2) {
        mHyp[i1][i2] = new double* [2];
        for (int i3 = 0; i3 < 2; ++i3) {
          mHyp[i1][i2][i3] = new double [2];
          for (int i4 = 0; i4 < 2; ++i4)
            mHyp[i1][i2][i3][i4] = 0;
        }
      }
    }

}

HyperCube::~HyperCube(){

if(mHyp) {

    for (int i1 = 0; i1 < 2; ++i1) {
        for (int i2 = 0; i2 < 2; ++i2) {
            for (int i3 = 0; i3 < 2; ++i3){
                delete[] mHyp[i1][i2][i3];
            }
            delete[] mHyp[i1][i2];
        }
        delete[] mHyp[i1];
    }
    delete[] mHyp;
}
}

void HyperCube::SetHyperCube(Vector4D *aVec, int x1, int x2, int x3, int x4){

    for (int i1 = 0; i1 < 2; ++i1) {
      for (int i2 = 0; i2 < 2; ++i2) {
        for (int i3 = 0; i3 < 2; ++i3) {
          for (int i4 = 0; i4 < 2; ++i4) mHyp[i1][i2][i3][i4] = (*aVec)(x1 + i1, x2 + i2, x3 + i3, x4 + i4);
        }
      }
    }

}

double**** HyperCube::GetHyperCube(void){

    return mHyp;

}
