
/**
 *
 *   pintegrals.cxx
 *
 *   Copyright (C) 2016 Mubarak Alqahtani, and Michael Strickland
 *
 *   This file is part of aHydro3p1.
 *
 *   aHydro3p1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   aHydro3p1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "Global.h"

/*!
 *
 *  \brief Default constructor
 *
 */

Pintegrals::Pintegrals() {
    PRINT_DEBUG_3("<Pintegrals:Pintegrals>");
}


/*!
 *
 *  \brief Destructor
 *
 */

Pintegrals::~Pintegrals(){
    PRINT_DEBUG_3("<Pintegrals::~Pintegrals>");

}

double X[INTPTS];
double Y[INTPTS];

double offset = 100;

double dM = (MMAX-MMIN)/(INTPTS-1.);

double Pintegrals::integrate(functionWrapperPint* fw, double n, double m, double delta0) {
    
    double result, error;
    struct integralParamsPint int_params = {n, m, delta0};
    gsl_function F;
    F.function = fw->function;
    F.params = &int_params;
    gsl_integration_workspace *w = gsl_integration_workspace_alloc(intNPTS);
    gsl_integration_qagiu(&F, 0, 0, intTOLERANCE, intNPTS, w, &result, &error);
    gsl_integration_workspace_free(w);
    return result;
}

double Pintegrals::pInt_integrand(double p, void *params) {
    
    struct integralParamsPint *my_params = (struct integralParamsPint *) params;
    double n = my_params->n;
    double m = my_params->m;
    double delta0 = my_params->delta0;
    
    return pow(p,3.-2.*n)*pow(m*m+delta0*p*p,n)*exp(-sqrt(p*p+m*m));
}

double Pintegrals::pmInt_integrand(double p, void *params) {
    
    struct integralParamsPint *my_params = (struct integralParamsPint *) params;
    double n = my_params->n;
    double m = my_params->m;
    double delta0 = my_params->delta0;
    
    return -(Power(p,3 - 2*n)*Power(Power(m,2) + delta0*Power(p,2),-1 + n)*(Power(m,2) + delta0*Power(p,2) - 2*n*Sqrt(Power(m,2) + Power(p,2))))/(2.*exp(Sqrt(m*m+p*p))*Sqrt(Power(m,2) + Power(p,2)));
}

double Pintegrals::pIntegral(double n, double m, double delta0) {
    
    functionWrapperPint fw;
    fw.function = &Pintegrals::pInt_integrand;
    return integrate(&fw, n, m, delta0);
}

double Pintegrals::pmIntegral(double n, double m, double delta0) {
    
    functionWrapperPint fw;
    fw.function = &Pintegrals::pmInt_integrand;
    return integrate(&fw, n, m, delta0);
}

void Pintegrals::loadpInterpolation(int region, double n) {
    
    double d0;
    
    if (region==0) d0=1;
    if (region==1) d0=4;
    
    // pInt
    
    for (int i=0;i<INTPTS;i++) {
        X[i] = MMIN + i*dM;
        Y[i] = log(pIntegral(n,X[i],d0));
    }
    
    gsl_interp_accel *a;
    gsl_spline *s;
    
    a = gsl_interp_accel_alloc();
    s = gsl_spline_alloc(gsl_interp_cspline, INTPTS);
    gsl_spline_init(s, X, Y, INTPTS);
    
    interpolationData intdata = {a,s};
    pintDataVector.push_back(intdata);
    
}


void Pintegrals::loadpmInterpolation(int region, double n) {
    
    double d0;
    
    if (region==0) d0=1;
    if (region==1) d0=4;
    
    // pmInt
    
    for (int i=0;i<INTPTS;i++) {
        X[i] = MMIN + i*dM;
        Y[i] = log(pmIntegral(n,X[i],d0)+offset);
    }
    
    gsl_interp_accel *a;
    gsl_spline *s;
    
    a = gsl_interp_accel_alloc();
    s = gsl_spline_alloc(gsl_interp_cspline, INTPTS);
    gsl_spline_init(s, X, Y, INTPTS);
    
    interpolationData intdata = {a,s};
    pmintDataVector.push_back(intdata);
    
}

double Pintegrals::pInt(double n, double m, double d0) {
    
#if USE_INTERPOLATIONS==0
    return Pintegrals::pIntegral(n,m,d0);
#else
    if (m<MMIN || m>MMAX) return pIntegral(n,m,d0);
    
    int region = 0;
    if (d0==1) region = 0;
    if (d0==4) region = 1;
    // compute the index of the appropriate interpolating function
    int idx = (int)( n-(1-2*EXPORDER)/2. ) + region*(EXPORDER+1);
    //cout << "Interpolation " << idx << endl;
    interpolationData id = pintDataVector[idx];
    return exp(gsl_spline_eval(id.s, m, id.a));
#endif
    
}

double Pintegrals::pmInt(double n, double m, double d0) {
    
#if USE_INTERPOLATIONS==0
    return Pintegrals::pmIntegral(n,m,d0);
#else
    if (m<MMIN || m>MMAX) return pmIntegral(n,m,d0);
    
    int region = 0;
    if (d0==1) region = 0;
    if (d0==4) region = 1;
    // compute the index of the appropriate interpolating function
    int idx = (int)( n-(1-2*EXPORDER)/2. ) + region*(EXPORDER+1);
    //cout << "Interpolation " << idx << endl;
    interpolationData id = pmintDataVector[idx];
    return exp(gsl_spline_eval(id.s, m, id.a))-offset;
#endif
}

void Pintegrals::freeInterpolations() {
    
    //PrintRawMessage("-> Freeing 1d interpolations for pInt and pmInt");
    
    for (int i=0; i<pintDataVector.size(); i++)
    {
        interpolationData id = pintDataVector[i];
        gsl_spline_free(id.s);
        gsl_interp_accel_free(id.a);
    }
    for (int i=0; i<pmintDataVector.size(); i++)
    {
        interpolationData id = pmintDataVector[i];
        gsl_spline_free(id.s);
        gsl_interp_accel_free(id.a);
    }
}

void Pintegrals::loadInterpolations() {
    
    //PrintRawMessage("-> Loading 1d interpolations for pInt and pmInt");
    
    double n = (1-2*EXPORDER)/2.;
    for (int i=0;i<=EXPORDER;i++) loadpInterpolation(0, n+i);
    for (int i=0;i<=EXPORDER;i++) loadpInterpolation(1, n+i);
    for (int i=0;i<=EXPORDER;i++) loadpmInterpolation(0, n+i);
    for (int i=0;i<=EXPORDER;i++) loadpmInterpolation(1, n+i);
}




