/**
*
*   main.cxx
*
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#include "Global.h"

// static variables

string sConfigurationFileDirectory;
string sGeneralConfigurationFileName;

// functions

void readParametersFromCommandLine(int argc, char** argv, Configurator *aConfigurator, int echo);
string loadINIFileFromCommandLine(int argc, char** argv, int echo);

// main function
// command line arguments: 1(program name) 2(bin file directory)
// running options:
// 1 - perform full analysis
// 2 - skip hydro stage, perform hypersurface extraction using .bin file

int main(int argc, char **argv){

     
	/*Teq Teq;
        double x=1.2;
        cout << Teq.teq(x) << endl;

  Matching Matching;
   // cout << "DEBUG" << endl << endl;
    //if (USE_HFUNC_INTERPOLATIONS) hfuncs.loadInterpolations();
    double ax=0.65;
    double ay=0.72;
    double az=0.82;
    double T=0.2;
    double lambda=0.2;


        cout << Matching.findT(lambda,ax,ay,az) << endl;
        cout << Matching.findLambda(T,ax,ay,az) << endl;*/

    /*double ax=1.5;
    double ay=1.2;
    double az=1.8;
    double m=2.9;

    cout << hfuncs.H3e(ax,ay,az,m) << endl;
    cout << hfuncs.d1H3e(ax,ay,az,m) << endl;
    cout << hfuncs.d2H3e(ax,ay,az,m) << endl;
    cout << hfuncs.d3H3e(ax,ay,az,m) << endl;
    cout << hfuncs.d4H3e(ax,ay,az,m) << endl;
    
    cout << endl;
    
    cout << hfuncs.H3x(ax,ay,az) << endl;
    cout << hfuncs.H3y(ax,ay,az) << endl;
    cout << hfuncs.H3L(ax,ay,az) << endl;
    
    cout << endl;
    
    cout << hfuncs.d1H3x(ax,ay,az) << endl;
    cout << hfuncs.d2H3x(ax,ay,az) << endl;
    cout << hfuncs.d3H3x(ax,ay,az) << endl;
    
    cout << endl;
    
    cout << hfuncs.d1H3y(ax,ay,az) << endl;
    cout << hfuncs.d2H3y(ax,ay,az) << endl;
    cout << hfuncs.d3H3y(ax,ay,az) << endl;

    cout << endl;
    
    cout << hfuncs.d1H3L(ax,ay,az) << endl;
    cout << hfuncs.d2H3L(ax,ay,az) << endl;
    cout << hfuncs.d3H3L(ax,ay,az) << endl;
    
    exit(0);*/

    
/*
     
    clock_t startTime = clock();
    int ntrials = 1000000;
    double max = 3.9;
    for (int i=0; i<ntrials; i++) {
        double ax = max*rand()/(RAND_MAX+1.0)+0.1;
        double ay = max*rand()/(RAND_MAX+1.0)+0.1;
        double az = max*rand()/(RAND_MAX+1.0)+0.1;
        hfuncs.d2H3L(ax,ay,az);

    }
    cout << double( clock() - startTime ) / (double)CLOCKS_PER_SEC<< " seconds." << endl;
    exit(0);
*/

    Configurator *tConfigurator = 0;
    Configurator *tConfigurator1 = 0;
    Configurator *tConfigurator2 = 0;
    Parser *tParser = 0;
    Solver *tSolver = 0;
    string tHydroDirectory = "";
    Freezer *tFreezer = 0;

    string tVersion = _AHYDRO3P1_VERSION;

    // print intro-message
    PrintMessage("aHydro3p1 (3+1d Anisotropic Hydrodynamics)", "ver. " + tVersion);
    PrintMessage("CONFIGURATION");

    sConfigurationFileDirectory     = "./ini/";
    sGeneralConfigurationFileName   = "general.ini";

    // check for commandline override of ini file
    string INIoverride = loadINIFileFromCommandLine(argc,argv,1);
    if (INIoverride.size()>0) {
        tParser = new Parser(INIoverride);
        tConfigurator1 = tParser->ReadINI();
        delete tParser;
    } else {
        tParser = new Parser(sConfigurationFileDirectory + sGeneralConfigurationFileName);
        tConfigurator1 = tParser->ReadINI();
        delete tParser;
    }

    // commandline overrides

    readParametersFromCommandLine(argc,argv,tConfigurator1,1);

    // run code modules according to the evaluationmode chosen

    bitset<2> tMode (tConfigurator1->GetParameter("EvaluationMode"));

    if(tMode.size() != 2) ProgramStatus(ERROR_INCORRECT_VALUE, "In main function.");

    if(tMode[0]){// hydro

        PrintMessage("STAGE 1: HYDRODYNAMIC EVOLUTION");

        tSolver = new Solver(tConfigurator1);
        sConfigurationFileDirectory = tSolver->SolveEquations();
        tConfigurator = tConfigurator1;
    }else{
            PrintMessage("Skipping STAGE 1.");

            sConfigurationFileDirectory = tConfigurator1->GetParameter("BINDir");
            sGeneralConfigurationFileName = GetFilename(GetDirectory(sConfigurationFileDirectory));
            Parser *tParser2 = new Parser(sConfigurationFileDirectory + sGeneralConfigurationFileName + ".ini");
            tConfigurator2  = tParser2->ReadINI();
            delete tParser2;

            // process commandline overrides
            readParametersFromCommandLine(argc,argv,tConfigurator2,1);

            tConfigurator = tConfigurator2;
        
//            if (!ifstream((sConfigurationFileDirectory + sGeneralConfigurationFileName).c_str())){
//                PrintRawMessage("File not found! Specify \'BINDir\' in the command line.");
//                return ProgramStatus(EXIT_OK, "main");
//            }

    }

    if(tMode[1]){// freezeout

        PrintMessage("STAGE 2: FREEZE-OUT HYPERSURFACE EXTRACTION");

        switch(tConfigurator->GetInt("FreezeOutMethod")){

                case 1:
                    tFreezer = new ParametrizationFreezer(tConfigurator, sConfigurationFileDirectory);
                    break;

                case 2:
                    //tFreezer = new ParametrizationFreezer(tConfigurator, sConfigurationFileDirectory);
                    tFreezer = new TriangulationFreezer(tConfigurator, sConfigurationFileDirectory);
                    break;

                default:
                    return ProgramStatus(ERROR_OPTION_MISSING, "In main function.");
            }

            tFreezer->ExtractFOHS();
    }

	delete tSolver;
	delete tFreezer;
    delete tConfigurator1;
    delete tConfigurator2;
    return ProgramStatus(EXIT_OK, "main");

}

//
//  checks to see if there is a special parameter called "INIFile" is on the commandline and,
//  if so override the default
//
string loadINIFileFromCommandLine(int argc, char** argv, int echo) {
    if (argc<=1) return "";
    int optind = 1;
    PrintRawMessage("");
    string key="",value="";
    PrintRawMessage("Checking for INI File override in commandline arguments");
    while (optind < argc)
    {
        key = argv[optind];
        if (key.compare("-INIFile")==0) {
            key = key.substr(1,key.length()); // remove '-'
            value = argv[optind+1]; // load value
            if (echo) PrintRawMessage(key,value);
            optind++;
        }
        optind++;
    }
    PrintRawMessage("");
    return value;
}

//
// Read all parameters from commandline
//
void readParametersFromCommandLine(int argc, char** argv, Configurator *aConfigurator, int echo) {
    if (argc<=1) return;
    int optind = 1;
    PrintRawMessage("");
    PrintRawMessage("Processing commandline arguments");
    while (optind < argc)
    {
        if (argv[optind][0]=='-') {
            string key = argv[optind];
            if (key.compare("-INIFile")!=0) { // Don't load INIFile, it is special
                key= key.substr(1,key.length()); // remove '-'
                string value = argv[optind+1]; // load value
                if (echo) PrintRawMessage(key,value);
                aConfigurator->SetParameter(key.c_str(),value.c_str());
                optind++;
            }
        }
        optind++;
    }
    PrintRawMessage("");
    return;
}
