/**
*
*   tsv_maker.cxx
*
*   Copyright (C) 2017 Radoslaw Ryblewski and Michael Strickland
*
*   This file is part of aHydro3p1.
*
*   aHydro3p1 is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   aHydro3p1 is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with aHydro3p1.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#include "Global.h"

// static variables

// main function definition

// 0th arg is always the program name
// 1st arg is the name of the specific hydro output directory - 8 characters (string)
// 2nd arg is specification of the cut direction : t (for tau), x, y, z (for eta) (string)
// 3rd arg is the value for cut (float)

// 0th arg is always the program name
// 1st arg is Tmin
// 2nd arg is Tmax
// 3rd arg is dT
// 4th arg is output directory

struct Record3D{

  double X;
  double Y;
  double Z;

};

vector<Record3D>* ImportArray2D(string FileName);

int main(int argc, char **argv){

    // leave if the number of parameters is different from the expected one

    if(argc > 5 || argc < 4){
    PrintMessage("Wrong number of parameters! Exit!\n");
    return 0;
    }

    // if there is 4 args and to radek section

if(argc == 4){

    PrintMessage("Making .tsv from ahydro3p1 output.");

    //change cstrings to strings
    string tFileName = argv[1];
    string tCut = argv[2];
    double tValue = s2d(argv[3]);

    cout << "Directory: " << tFileName;
    cout << "\nCut: " << tCut << " = " << tValue << endl;

    string tTSVDir = "./output/tsv/" + tFileName + "/";
    string tBINDir = "./output/bin/" + tFileName + "/";
    string tConfFileName = tFileName + ".ini";
    ifstream tBINFile;
    ofstream tTSVFile;

    // create .tsv directory

    struct stat tFileStatistics;   // needed for specification of number of t points
    const int tSizeOfType = sizeof(float);
    Vector4D *tField = 0;

    // open confifuration file with configurator

    Configurator *tConfigurator = 0;
    Parser *tParser = 0;
    tParser = new Parser(tBINDir + tConfFileName);       // construct Parser with the given ini file
    tConfigurator = tParser->ReadINI();                // parse file and send result to configurator
    delete tParser;
    int tRec = tConfigurator->GetInt("RecordingFrequency");
    if(tRec < 0) tRec = -tRec;
    Grid *tGrid  = new Grid(tConfigurator);

    // number of fields according to the equations type used

    int tEq = static_cast<usi>(tConfigurator->GetInt("Equations"));

    int tNumberOfFields = 0;
    string *tFieldNames;

    switch(tEq){

        case 1:

            tNumberOfFields = 4;
            tFieldNames = new string [tNumberOfFields];
            tFieldNames[0] = "lnt";
            tFieldNames[1] = "rap";
            tFieldNames[2] = "ux";
            tFieldNames[3] = "uy";

            break;

        case 2:

            tNumberOfFields = 5;
            tFieldNames = new string [tNumberOfFields];
            tFieldNames[0] = "lambda";
            tFieldNames[1] = "rap";
            tFieldNames[2] = "ux";
            tFieldNames[3] = "uy";
            tFieldNames[4] = "xi";

            break;

        case 3:

            tNumberOfFields = 8;
            tFieldNames = new string [tNumberOfFields];
            tFieldNames[0] = "ax";
            tFieldNames[1] = "ay";
            tFieldNames[2] = "az";
            tFieldNames[3] = "ux";
            tFieldNames[4] = "uy";
            tFieldNames[5] = "rap";
            tFieldNames[6] = "lambda";
            tFieldNames[7] = "temp";
            
            break;

        default:

            PrintMessage("Cannot find Equations for " + d2s(tEq) + "! Exit!\n");
    }

    // ---------- LOOP OVER ALL FIELDS

    for(int i = 0; i < tNumberOfFields; ++i){

    // define the file names

    string tBINFileName = tBINDir + tFieldNames[i] + ".bin";
    string tTSVFileName = tTSVDir + tFieldNames[i] + "_" + tCut + "_" + d2s(tValue) + ".tsv";

    // open .bin file

    try{
        tBINFile.open(tBINFileName.c_str(), ios::in | ios::binary); // here add some error hhandler if the file does not exist
    }catch(const ios::failure &problem){
        cout << problem.what();
        return 0;
    }
        if(i == 0){
            MakeDirectory(tTSVDir);
        }
    // find out what the number of t points contains the .bin file // number of t points exported to .bin file is less than the number of taken steps

    stat(tBINFileName.c_str(),&tFileStatistics);
    double tNumberOfPoints = tFileStatistics.st_size / tSizeOfType;

    // size of the file in Bajts / size of number type in Bajts = total number of points (has to be integer number)
    // if not:

    if(tSizeOfType -  static_cast<int>(tSizeOfType))
    {
        cout << "Binary file has wrong format. It is not multiple of a size of float.\n" << flush;
        return 0;
    }

    // number of tau points in file can be evaluated in such a way:

    double tTPts = tNumberOfPoints / (tGrid->GetXP() * tGrid->GetYP() * tGrid->GetZP());// number of t points exported to .bin file is less than the number of taken steps
    if(tTPts -  static_cast<int>(tTPts))
    {
        cout << "Number of tau points has to be an integer number.\n" << flush;
        return 0;
    }

    if(i == 0) cout << "File contains " << tTPts << " time points.\n" << flush;

    const int tTP =  static_cast<int>(tTPts);
    const double tTmin = tGrid->GetT(0);

    // find Tmax // tGrid->GetT(tGrid->GetTP()) can be also give different value from the last eksported t point
    // maximal value of T is calculated in such a way:

    const double tTmax = tTmin + tRec * (tTP - 1) * tGrid->GetdT();// DEF1

    // define table containing values of time points:

    double tT[tTP];

    // here FILL IT

    for(usi n = 0; n < tTP; ++n) tT[n] = tTmin + tRec * n * tGrid->GetdT(); // see DEF1

    if(i == 0) {
        cout << "Tmin: " << tTmin << " , Tmax:" << tTmax << " , Tpts:" << tTP << "\n" << flush;
        cout << "Xmin: " << tGrid->GetXMin() << " , Xmax:" << tGrid->GetXMax() << " , Xpts:" << tGrid->GetXP() << "\n" << flush;
        cout << "Ymin: " << tGrid->GetYMin() << " , Ymax:" << tGrid->GetYMax() << " , Ypts:" << tGrid->GetYP() << "\n" << flush;
        cout << "Zmin: " << tGrid->GetZMin() << " , Zmax:" << tGrid->GetZMax() << " , Zpts:" << tGrid->GetZP() << "\n" << flush;
    }

    // define the appropriate Vector4D
// string aName,
//	     double aTmin, double aTmax, int aTpts,
//	     double aXmin, double aXmax, int aXpts,
//	     double aYmin, double aYmax, int aYpts,
//	     double aZmin, double aZmax, int aZpts

    if(i == 0) tField = new Vector4D("Field", tTmin, tTmax, tTP, tGrid->GetXMin(),tGrid->GetXMax(),tGrid->GetXP(), tGrid->GetYMin(),tGrid->GetYMax(),tGrid->GetYP(), tGrid->GetZMin(),tGrid->GetZMax(),tGrid->GetZP());

    //read .bin file into the Vector4D object

    for (usi n = 0; n < tTP; n++)
        for (usi i = 0; i < tGrid->GetXP(); i++)
            for (usi j = 0; j < tGrid->GetYP(); j++)
                for (usi k = 0; k < tGrid->GetZP(); k++){
                  tBINFile.read(reinterpret_cast<char*>(&((*tField)(n,i,j,k))),tSizeOfType);
                }

    // write the required data to .tsv. file
	tTSVFile.open(tTSVFileName.c_str(), ios::trunc | ios::out);
    tTSVFile.precision(6);

    stringstream tBuffer;
    // check what happens when there is only one t point !!! -> the interpolation may not work properly !

    if(tTP == 1){
    cout << "\nOnly ONE time point available. Exporting the IC data for "<< tTmin << ".\n" << flush;
    for (usi i = 0; i < tGrid->GetXP(); i++)
        for (usi j = 0; j < tGrid->GetYP(); j++)
            for (usi k = 0; k < tGrid->GetZP(); k++){
			tBuffer << tGrid->GetX(i) << "\t" << tGrid->GetY(j) << "\t" << tGrid->GetZ(k) << "\t" << tField->Interpolate(tValue,tGrid->GetX(i),tGrid->GetY(j),tGrid->GetZ(k)) << endl;
            }
    } else {
        if(tCut == "t"){
        if(tValue > tTmax || tValue < tTmin) goto OutOfRange;
        for (usi i = 0; i < tGrid->GetXP(); i++)
            for (usi j = 0; j < tGrid->GetYP(); j++)
                for (usi k = 0; k < tGrid->GetZP(); k++){
                tBuffer << tGrid->GetX(i) << "\t" << tGrid->GetY(j) << "\t" << tGrid->GetZ(k) << "\t" << tField->Interpolate(tValue,tGrid->GetX(i),tGrid->GetY(j),tGrid->GetZ(k)) << endl;
                }
        } else if(tCut == "x"){
        if(tValue > tGrid->GetX(tGrid->GetXP()) || tValue < tGrid->GetX(0)) goto OutOfRange;
        for (usi n = 0; n < tTP; n++)
            for (usi j = 0; j < tGrid->GetYP(); j++)
                for (usi k = 0; k < tGrid->GetZP(); k++){
                tBuffer << tT[n] << "\t" << tGrid->GetY(j) << "\t" << tGrid->GetZ(k) << "\t" << tField->Interpolate(tT[n],tValue,tGrid->GetY(j),tGrid->GetZ(k)) << endl;
                }
        } else if(tCut == "y"){
        if(tValue > tGrid->GetY(tGrid->GetYP()) || tValue < tGrid->GetY(0)) goto OutOfRange;
        for (usi n = 0; n < tTP; n++)
            for (usi i = 0; i < tGrid->GetXP(); i++)
                for (usi k = 0; k < tGrid->GetZP(); k++){
                tBuffer << tT[n] << "\t" << tGrid->GetX(i) << "\t" << tGrid->GetZ(k) << "\t" << tField->Interpolate(tT[n],tGrid->GetX(i),tValue,tGrid->GetZ(k)) << endl;
                }
        } else if(tCut == "z"){
        if(tValue > tGrid->GetZ(tGrid->GetZP()) || tValue < tGrid->GetZ(0)) goto OutOfRange;
        for (usi n = 0; n < tTP; n++)
            for (usi i = 0; i < tGrid->GetXP(); i++)
                for (usi j = 0; j < tGrid->GetYP(); j++){
                tBuffer << tT[n] << "\t" << tGrid->GetX(i) << "\t" << tGrid->GetY(j) << "\t" << tField->Interpolate(tT[n],tGrid->GetX(i),tGrid->GetY(j),tValue) << endl;
                }
        } else {
            cout << "\nWrongly defined cut direction.\n" << flush;
            return 0;
        }
    }

    tTSVFile << tBuffer.rdbuf(); // faster than tBuffer.str();

    //close both files

    tBINFile.close();
    tTSVFile.close();
    }
    // ---------- LOOP OVER ALL FIELDS

    delete tField;
    delete tConfigurator;
    cout << "Done.\n" << flush;
    return 1;

OutOfRange:

    cout << "\nCut defined out of " + tCut + " range.\n" << flush;
    return 0;

}else{

    // if there is any 5th arg then go to the mikes section

    PrintMessage("Making .tsv from ahydro2p1 output.");

    double tTmin = s2d(argv[1]);
    double tTmax = s2d(argv[2]);
    double tdT   = s2d(argv[3]);
    string tTSVDir2 = argv[4];
    int tTPts    = 1 + (tTmax - tTmin) / tdT;

    cout << "Tmin : " << tTmin << ", Tmax : " << tTmax << ", dT : " << tdT << ", TPts : " << tTPts << "\n\n" << flush;

    int tNumberOfFields2 = 4;
    string *tFieldNames2 = new string [tNumberOfFields2];
    tFieldNames2[0] = "phard";
    tFieldNames2[1] = "ux";
    tFieldNames2[2] = "uy";
    tFieldNames2[3] = "xi";
    string tTSVFileName2;
    stringstream str;
    double tT;
    vector<Record3D> *Lines;        //  holds the data from TSV file as a Records
    int ArrSize;
    double tXmin, tXmax;
    int tXPts;
    double tYmin, tYmax;
    int tYPts;
    double tdX, tdY;

    Vector3D *tField2 = 0;

      for(int m = 0; m < tNumberOfFields2;++m){
        for (usi n = 0; n < tTPts; ++n){
            tT = tTmin + n * tdT;
            str.str(string());
            str.clear();
            str << fixed << setprecision(3) << tT;
            tTSVFileName2 = tTSVDir2 + tFieldNames2[m] + "_" + str.str() + ".dat";

            Lines = ImportArray2D(tTSVFileName2);   //  reading the data from TSV to tRecords
            ArrSize = Lines->size();

            if(m == 0 && n == 0){
                vector<Record3D>::iterator i;
                tXmin = Lines->begin()->X;
                tXmax = (Lines->end()-1)->X;
                tYmin = Lines->begin()->Y;
                tYPts = 0;
                for (i = Lines->begin(); i < Lines->end(); i++){
                    if(i->X != tXmin) {
                        i--;
                        tYmax = i->Y;
                        break;
                    }
                    tYPts++;
                }
                tXPts = ArrSize / tYPts;
                tdX = (tXmax - tXmin) / (tXPts - 1);
                tdY = (tYmax - tYmin) / (tYPts - 1);
                cout << "\nTotal Pts : " << ArrSize << flush;
                cout << "\nXmin : " << tXmin << ", Xmax : " << tXmax << ", XPts : " << tXPts << ", dX : " << tdX << "" << flush;
                cout << "\nYmin : " << tYmin << ", Ymax : " << tYmax << ", YPts : " << tYPts << ", dY : " << tdY << "\n\n" << flush;

                tField2 = new Vector3D("Field",tTmin,tTmax,tTPts,tXmin,tXmax,tXPts,tYmin,tYmax,tYPts);
            }

            for (usi i = 0; i < tXPts; i++)
                for (usi j = 0; j < tYPts; j++)
                      (*tField2)(n,i,j) = (Lines->begin() + i * tXPts + j)->Z;

        }

        // export Vector3D to file f(t,x,y)

        ofstream tTSVFile2;

        tTSVFileName2 = tTSVDir2 + tFieldNames2[m] + ".tsv";
        tTSVFile2.open(tTSVFileName2.c_str(), ios::trunc | ios::out);
        tTSVFile2.precision(6);
        stringstream tBuffer;
        cout << "Exporting to file " <<  tTSVFileName2 << endl;
        for (usi n = 0; n < tTPts; n++)
            for (usi i = 0; i < tXPts; i++)
                for (usi j = 0; j < tYPts; j++){
                tBuffer << tTmin + n * tdT << "\t" << tXmin + i * tdX << "\t" << tYmin + j * tdY << "\t" << tField2->Interpolate(tTmin + n * tdT,tXmin + i * tdX,tYmin + j * tdY) << endl;
                }
        tTSVFile2 << tBuffer.rdbuf();
        tTSVFile2.close();
      }

    delete tField2;
}
}

vector<Record3D>* ImportArray2D(string FileName){

    ifstream File;
    File.open(FileName.c_str());
    if((!File) && (!File.is_open())){
    cout << " File " << FileName << " not found. Exit" << endl; exit(0);
    };
    cout << " File " << FileName << " read." << endl;

 	Record3D TSVLine;

	vector<Record3D>* Lines = new vector<Record3D>;

 	while ((File >> TSVLine.X >> TSVLine.Y >> TSVLine.Z)){
 		Lines->push_back (TSVLine);
    }

    File.close();

    return Lines;

}
// --------------------------------------------------------------------------------------------
