#!/bin/bash

############################################################################################################
#SBATCH --job-name=ahydro3p1
#SBATCH --nodes=2
#SBATCH --ntasks=11
#SBATCH --ntasks-per-node=6
############################################################################################################

# list of b's
# <b> in standard centrality classes
blist=("2.3154" "4.23354" "5.98713" "7.75305" "9.18112" "10.4139" "11.515" "12.5194" "13.4488" "14.3335" "15.6079")
nb=$((${#blist[@]}-1))

############################################################################################################
############################################################################################################

function printLine {
echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
}

printLine
echo "Launching runs"
printLine

# check for existence of 'runs' directory; create it if it doesn't exist
id=`eval date +%Y%m%d`"_"$SLURM_JOB_ID
rundir="runs_"$id
if [ -d $rundir ]; then
	echo "==> $rundir directory exists"
else 
	echo "==> $rundir directory does not exist, creating it"
	mkdir $rundir
fi

for i in `seq 0 $nb`;
  do
    b=${blist[$i]}
    # check for existence of run specfic directory
    dir=$rundir"/"$i
    if [ -d $dir ]; then
	    echo "==> run directory "$dir" directory exists already ... cleaning it out"
	    rm -rf $dir
	    mkdir $dir
    else 
	    echo "==> creating run directory "$dir
	    mkdir $dir
    fi
  
    # copy necessary files and create output directories
    echo "==> creating run directory structure"
  
    mkdir $dir"/output"
    mkdir $dir"/ini"
    cp aHydro3p1 $dir
    cp ini/general.ini $dir"/ini/" 
    cp -rf input $dir
    cd $dir
    echo "==> launching job "$i
    srun -n1 -N1 --exclusive ./aHydro3p1 -ImpactParameter $b > log.out &
    sleep 2
    cd ../..
done
wait
mv slurm-${SLURM_JOB_ID}.out $rundir/slurm-out.log
