#!/bin/bash

############################################################################################################
#SBATCH --job-name=ahydro3p1
#SBATCH --nodes=4
#SBATCH --ntasks=27
#SBATCH --ntasks-per-node=7
############################################################################################################

# list of T0
tlist=("0.57" "0.58" "0.59")
nt=$((${#tlist[@]}-1))

# list of fpietabar
eblist=("1")
neb=$((${#eblist[@]}-1))

# list of slope1
slope1=("0" "0.1" "0.2")
nslope1=$((${#slope1[@]}-1))

# list of slope2
slope2=("0.1" "0.2" "0.3")
nslope2=$((${#slope2[@]}-1))

############################################################################################################
############################################################################################################

function printLine {
echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
}

printLine
echo "Launching runs"
printLine

# check for existence of 'runs' directory; create it if it doesn't exist
id=`eval date +%Y%m%d`"_"$SLURM_JOB_ID
rundir="runs_"$id
if [ -d $rundir ]; then
	echo "==> $rundir directory exists"
else 
	echo "==> $rundir directory does not exist, creating it"
	mkdir $rundir
fi

cnt=0
for i in `seq 0 $nt`;
  do
    t=${tlist[$i]}
    for j in `seq 0 $neb`;
      do
        fpieb=${eblist[$j]}
    	for k in `seq 0 $nslope1`;
      	  do
            s1=${slope1[$k]}
    	    for l in `seq 0 $nslope2`;
      	      do
                s2=${slope2[$l]}

                eb=$(echo "scale=6; $fpieb/12.5663706144" | bc)
                # check for existence of run specfic directory
                dir=$rundir"/"$cnt
                if [ -d $dir ]; then
	          echo "==> run directory "$dir" directory exists already ... cleaning it out"
	          rm -rf $dir
	          mkdir $dir
                else 
	          echo "==> creating run directory "$dir
	          mkdir $dir
                fi
    
                # copy necessary files and create output directories
                echo "==> creating run directory structure"
      
                mkdir $dir"/output"
                mkdir $dir"/ini"
                cp aHydro3p1 $dir
                cp ini/general.ini $dir"/ini/" 
                cp -rf input $dir
                cd $dir
                echo "==> launching job "$cnt
                echo "==> T0="$t
                echo "==> fpiebar="$fpieb
                echo "==> ebar="$eb
                echo "==> slope1="$s1
                echo "==> slope2="$s2
    	        srun -n1 -N1 --exclusive ./aHydro3p1 -InitialTemperature $t -etabar $eb -Slope2 $s2 > log.out &
                sleep 5
                cd ../..
                cnt=$((cnt+1))
	  done
	done
      done
    done 
wait
mv slurm-${SLURM_JOB_ID}.out $rundir/slurm-out.log
