#!/bin/bash

############################################################################################################
#SBATCH --job-name=ahydro3p1
#SBATCH --nodes=4
#SBATCH --ntasks=25
#SBATCH --ntasks-per-node=7
############################################################################################################

# list of T0
tlist=("0.56" "0.57" "0.58" "0.59" "0.60")
nt=$((${#tlist[@]}-1))

# list of fpietabar
eblist=("1.5" "2" "2.5" "3" "3.5")
neb=$((${#eblist[@]}-1))

############################################################################################################
############################################################################################################

function printLine {
echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
}

printLine
echo "Launching runs"
printLine

# check for existence of 'runs' directory; create it if it doesn't exist
id=`eval date +%Y%m%d`"_"$SLURM_JOB_ID
rundir="runs_"$id
if [ -d $rundir ]; then
	echo "==> $rundir directory exists"
else 
	echo "==> $rundir directory does not exist, creating it"
	mkdir $rundir
fi

cnt=0
for i in `seq 0 $nt`;
  do
    t=${tlist[$i]}
    for j in `seq 0 $neb`;
      do
        fpieb=${eblist[$j]}
        eb=$(echo "scale=6; $fpieb/12.5663706144" | bc)
          # check for existence of run specfic directory
          dir=$rundir"/"$cnt
          if [ -d $dir ]; then
	    echo "==> run directory "$dir" directory exists already ... cleaning it out"
	    rm -rf $dir
	    mkdir $dir
          else 
	    echo "==> creating run directory "$dir
	    mkdir $dir
          fi
  
          # copy necessary files and create output directories
          echo "==> creating run directory structure"
  
          mkdir $dir"/output"
          mkdir $dir"/ini"
          cp aHydro3p1 $dir
          cp ini/general.ini $dir"/ini/" 
          cp -rf input $dir
          cd $dir
          echo "==> launching job "$cnt
          echo "==> T0="$t
          echo "==> fpiebar="$fpieb
          echo "==> ebar="$eb
    	  srun -n1 -N1 --exclusive ./aHydro3p1 -InitialTemperature $t -etabar $eb > log.out &
          sleep 5
          cd ../..
          cnt=$((cnt+1))
      done
    done 
wait
mv slurm-${SLURM_JOB_ID}.out $rundir/slurm-out.log
