(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[         0,          0]
NotebookDataLength[    190699,       4735]
NotebookOptionsPosition[    178847,       4463]
NotebookOutlinePosition[    179258,       4479]
CellTagsIndexPosition[    179215,       4476]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 StyleBox[" 3+1D ahydro equations\n",
  FontSize->14],
 "\n",
 StyleBox[" ",
  FontWeight->"Plain"],
 "\n",
 StyleBox["written by: \nRadoslaw Ryblewski (rryblews@kent.edu)\nMichael \
Strickland (mstrick6@kent.edu)\n\nThis notebook contains LO anisotropic fluid \
evolution equations for aHydro3p1. \nIt generates the expressions for: ",
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Tau]"], "\[CapitalTheta]"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Tau]"], 
    SubscriptBox["u", "x"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Tau]"], 
    SubscriptBox["u", "y"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Tau]"], "\[CapitalLambda]"}], 
   TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Tau]"], "\[Xi]"}], TraditionalForm]]],
 " ",
 StyleBox["which are used by Runge-Kutta algorithm in C++ code aHydro3p1.\n",
  FontWeight->"Plain"]
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.5959377759181356`*^9, 3.59593799749175*^9}, {
   3.595938029411251*^9, 3.595938039123149*^9}, {3.595938075247025*^9, 
   3.595938241403592*^9}, {3.595940124401556*^9, 3.5959401345249443`*^9}, {
   3.5969931945208893`*^9, 3.5969932009811563`*^9}, {3.596993271699087*^9, 
   3.5969932726910896`*^9}, {3.596993332462383*^9, 3.5969933335934114`*^9}, {
   3.598049108028962*^9, 3.5980491962253523`*^9}},
 TextAlignment->Center,
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[CellGroupData[{

Cell["(* Operators *)", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, 
   3.595941026510354*^9},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell["\<\
(* 
Functions which the equations are solved for:

ux = ux[\[Tau],\[Eta],x,y]
uy = uy[\[Tau],\[Eta],x,y]
\[CapitalTheta] = \[CapitalTheta][\[Tau],\[Eta],x,y]
\[Xi] = \[Xi][\[Tau],\[Eta],x,y]
\[CapitalLambda] = \[CapitalLambda][\[Tau],\[Eta],x,y].

Independent variables: \[Tau], \[Eta], x, y.

For derivation of the operators defined below see:  arXiv:1207.0629. See also \
\"operators.nb\".
 *)\
\>", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.},
 CellChangeTimes->{{3.595938343108375*^9, 3.595938484690908*^9}, {
   3.59593856225112*^9, 3.595938575541687*^9}, {3.595938606463481*^9, 
   3.595938616639481*^9}, {3.595939897725975*^9, 3.59593991517458*^9}, {
   3.595940023395308*^9, 3.5959400517655888`*^9}, {3.5959401021610327`*^9, 
   3.595940108869153*^9}, {3.595940752607624*^9, 3.5959407853703613`*^9}, {
   3.595940959775938*^9, 3.595940966255937*^9}, {3.5959409973543262`*^9, 
   3.595941026510405*^9}, 3.598049216440502*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"up", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    " ", 
    SqrtBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"ux", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"], 
      "+", 
      SuperscriptBox[
       RowBox[{"uy", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"], 
      " "}]]}], ";"}], " ", 
  RowBox[{"(*", " ", 
   SubscriptBox["u", "\[UpTee]"], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"u0", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    " ", 
    SqrtBox[
     RowBox[{"1", "+", 
      SuperscriptBox[
       RowBox[{"up", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       "2"]}]]}], ";"}], 
  RowBox[{"(*", " ", 
   SubscriptBox["u", "0"], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Y", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{"\[Eta]", "-", 
     RowBox[{"\[CapitalTheta]", "[", 
      RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}], ";"}],
   "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"L1", "[", "\[ScriptCapitalF]_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "\[Tau]"]}], 
     RowBox[{"Sinh", "[", 
      RowBox[{"Y", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
     " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", "\[Eta]"], "\[ScriptCapitalF]"}]}], "+", 
    RowBox[{
     RowBox[{"Cosh", "[", 
      RowBox[{"Y", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
     RowBox[{
      SubscriptBox["\[PartialD]", "\[Tau]"], "\[ScriptCapitalF]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"L2", "[", "\[ScriptCapitalF]_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"+", 
       FractionBox["1", "\[Tau]"]}], " ", 
      RowBox[{"Cosh", "[", 
       RowBox[{"Y", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Eta]"], "\[ScriptCapitalF]"}]}], "-", 
     RowBox[{
      RowBox[{"Sinh", "[", 
       RowBox[{"Y", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      " ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Tau]"], "\[ScriptCapitalF]"}]}]}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DpUp", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "x"], 
      RowBox[{"ux", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
     RowBox[{
      SubscriptBox["\[PartialD]", "y"], 
      RowBox[{"uy", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]}], 
   ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    OverscriptBox["\[Del]", "\[RightVector]"], " ", 
    OverscriptBox[
     SubscriptBox["u", "\[UpTee]"], "\[RightVector]"]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"UpDp", "[", "\[ScriptCapitalF]_", "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"ux", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], " ", 
      RowBox[{
       SubscriptBox["\[PartialD]", "x"], "\[ScriptCapitalF]"}]}], "+", 
     RowBox[{
      RowBox[{"uy", " ", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      RowBox[{
       SubscriptBox["\[PartialD]", "y"], "\[ScriptCapitalF]"}]}]}]}], ";"}], 
  RowBox[{"(*", "  ", 
   RowBox[{
    OverscriptBox[
     SubscriptBox["u", "\[UpTee]"], "\[RightVector]"], " ", 
    OverscriptBox["\[Del]", "\[RightVector]"]}], " ", "*)"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"UD", "[", "\[ScriptCapitalF]_", "]"}], ":=", " ", 
    RowBox[{
     RowBox[{"UpDp", "[", "\[ScriptCapitalF]", "]"}], "+", 
     RowBox[{
      RowBox[{"u0", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      RowBox[{"L1", "[", "\[ScriptCapitalF]", "]"}]}]}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SuperscriptBox["U", "\[Mu]"], 
    SubscriptBox["\[PartialD]", "\[Mu]"]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"VD", "[", "\[ScriptCapitalF]_", "]"}], ":=", 
    RowBox[{"L2", "[", "\[ScriptCapitalF]", "]"}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SuperscriptBox["V", "\[Mu]"], 
    SubscriptBox["\[PartialD]", "\[Mu]"]}], " ", "*)"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DU", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"DpUp", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], " ", "+", 
      RowBox[{"L1", "[", 
       RowBox[{"u0", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "+", 
      RowBox[{
       RowBox[{"u0", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       RowBox[{"L2", "[", 
        RowBox[{"\[CapitalTheta]", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
        "]"}]}]}], "]"}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Mu]"], 
    SuperscriptBox["U", "\[Mu]"]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DV", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{"L1", "[", 
      RowBox[{"\[CapitalTheta]", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
     "]"}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SubscriptBox["\[PartialD]", "\[Mu]"], 
    SuperscriptBox["V", "\[Mu]"]}], " ", "*)"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]1", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"u0", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      RowBox[{"L2", "[", 
       RowBox[{"\[CapitalTheta]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}]}],
      "]"}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SubscriptBox["U", "\[Nu]"], 
    SuperscriptBox["V", "\[Mu]"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SuperscriptBox["V", "\[Nu]"]}]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]2", "[", 
     RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"u0", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"UpDp", "[", 
         RowBox[{"\[CapitalTheta]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}],
         "+", 
        RowBox[{
         RowBox[{"u0", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
         RowBox[{"L1", "[", 
          RowBox[{"\[CapitalTheta]", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
          "]"}]}]}], ")"}]}], "]"}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    SubscriptBox["V", "\[Nu]"], 
    SuperscriptBox["U", "\[Mu]"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SuperscriptBox["U", "\[Nu]"]}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]"}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10002.},
 CellChangeTimes->{{3.5843330672395*^9, 3.58433308236639*^9}, {
   3.584333551420419*^9, 3.584333601103108*^9}, {3.584333665360273*^9, 
   3.584333694714584*^9}, {3.584333993453297*^9, 3.584334080513718*^9}, {
   3.592124540422938*^9, 3.592124611050702*^9}, {3.592124641262281*^9, 
   3.592124664900694*^9}, {3.592124695272218*^9, 3.592124698512785*^9}, {
   3.592125929087735*^9, 3.592125937438864*^9}, {3.592126013790105*^9, 
   3.592126027609721*^9}, {3.592126156680561*^9, 3.592126181148512*^9}, {
   3.592126272281247*^9, 3.592126274165404*^9}, 3.594743727403545*^9, {
   3.595938316764769*^9, 3.595938324057412*^9}, {3.59594020307592*^9, 
   3.595940218045492*^9}, {3.5959402595008163`*^9, 3.595940262401919*^9}, {
   3.595940330017214*^9, 3.595940384938987*^9}, {3.595940415423855*^9, 
   3.595940465776135*^9}, 3.595940815904538*^9, {3.595940877304138*^9, 
   3.595940914079331*^9}, {3.595940952865994*^9, 3.5959409571485357`*^9}, 
   3.595941026510475*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["(* Equation of state *)", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10001.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"thermodynamic", " ", "variables"}], ",", " ", 
    RowBox[{
    "P", " ", "and", " ", "\[CurlyEpsilon]", " ", "are", " ", "isotropic", 
     " ", "thermodynamic", " ", "variables"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"PL", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     " ", 
     RowBox[{
      RowBox[{"P", "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      RowBox[{"RL", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"PT", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      RowBox[{"P", "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      RowBox[{"RT", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Epsilon]", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     " ", 
     RowBox[{
      RowBox[{"\[CurlyEpsilon]", "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      RowBox[{"R", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       "]"}]}]}], ";"}]}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10001.},
 CellChangeTimes->{{3.56490870736257*^9, 3.564908717503238*^9}, {
   3.564908823258741*^9, 3.564908823627633*^9}, {3.56497864762816*^9, 
   3.564978671339451*^9}, {3.564980326466891*^9, 3.564980353455334*^9}, {
   3.584335000676806*^9, 3.584335001164825*^9}, {3.584335041436735*^9, 
   3.584335049052154*^9}, {3.584336400213577*^9, 3.584336404903622*^9}, {
   3.584337389738687*^9, 3.584337431271009*^9}, {3.584337465114624*^9, 
   3.584337466903268*^9}, {3.592050793610105*^9, 3.592050797256536*^9}, {
   3.592050992497602*^9, 3.592051003231515*^9}, {3.59205105188782*^9, 
   3.592051056150435*^9}, {3.592051123792599*^9, 3.592051162721387*^9}, {
   3.592051784338253*^9, 3.592051890879368*^9}, 3.592051936720448*^9, 
   3.59205212747558*^9, {3.5947419882766743`*^9, 3.59474199334851*^9}, 
   3.5947437274038267`*^9, {3.595940547634246*^9, 3.5959405739032288`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["(* Hydrodynamic equations *)", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, 3.59594104835845*^9},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"hydrodynamic", " ", "equations"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"\[CapitalGamma]", "[", 
     RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], " ", "is", 
    " ", "the", " ", "inverse", " ", "equilibration", " ", 
    RowBox[{"time", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "0", "th", " ", "moment", " ", "of", " ", "the", " ", "Boltzmann", " ", 
    "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Eqn1LHS", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      FractionBox[
       RowBox[{"UD", "[", 
        RowBox[{"\[Xi]", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
       RowBox[{"1", "+", 
        RowBox[{"\[Xi]", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]], "-", 
      FractionBox[
       RowBox[{"6", 
        RowBox[{"UD", "[", 
         RowBox[{"\[CapitalLambda]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
         "]"}]}], 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "-", 
      RowBox[{"2", 
       RowBox[{"DU", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], " ", 
      "-", 
      RowBox[{"2", " ", 
       RowBox[{"\[CapitalGamma]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"R", "[", 
            RowBox[{"\[Xi]", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
            "]"}], 
           FractionBox["3", "4"]], 
          SqrtBox[
           RowBox[{"1", "+", 
            RowBox[{"\[Xi]", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
             "]"}]}]]}]}], ")"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"LHS", " ", "of", " ", 
      SubscriptBox["U", "\[Nu]"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Mu]"], 
       SuperscriptBox["T", "\[Mu]\[Nu]"]}]}], "  ", "=", " ", "0"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Eqn2LHS", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      RowBox[{"UD", "[", 
       RowBox[{"\[Epsilon]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Epsilon]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "+", 
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"DU", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "-", 
      " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "-", 
         RowBox[{"PL", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"\[CapitalDelta]1", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"LHS", " ", "of", " ", 
      SubscriptBox["V", "\[Nu]"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Mu]"], 
       SuperscriptBox["T", "\[Mu]\[Nu]"]}]}], "  ", "=", " ", "0"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Eqn3LHS", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      RowBox[{"VD", "[", 
       RowBox[{"PL", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Epsilon]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "+", 
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"\[CapitalDelta]2", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "-", 
      " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "-", 
         RowBox[{"PL", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"DV", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"LHS", " ", "of", " ", 
      SubscriptBox[
       SuperscriptBox["\[CapitalDelta]", "1"], "\[Nu]"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Mu]"], 
       SuperscriptBox["T", "\[Mu]\[Nu]"]}]}], " ", "=", " ", "0"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Eqn4LHS", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Epsilon]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "+", 
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"UD", "[", 
        RowBox[{"ux", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
        "]"}]}], " ", "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", "x"], 
       RowBox[{"PT", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"ux", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"UD", "[", 
          RowBox[{"PT", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
          "]"}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PT", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
            "-", 
            RowBox[{"PL", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
           ")"}], 
          RowBox[{"\[CapitalDelta]1", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}], 
        ")"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{"LHS", " ", "of", " ", 
      SubscriptBox[
       SuperscriptBox["\[CapitalDelta]", "2"], "\[Nu]"], 
      RowBox[{
       SubscriptBox["\[PartialD]", "\[Mu]"], 
       SuperscriptBox["T", "\[Mu]\[Nu]"]}]}], " ", "=", " ", "0"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Eqn5LHS", "[", 
      RowBox[{"\[Tau]_", ",", "\[Eta]_", ",", "x_", ",", "y_"}], "]"}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Epsilon]", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "+", 
         RowBox[{"PT", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
        ")"}], 
       RowBox[{"UD", "[", 
        RowBox[{"uy", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
        "]"}]}], " ", "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", "y"], 
       RowBox[{"PT", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"uy", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"UD", "[", 
          RowBox[{"PT", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
          "]"}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PT", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
            "-", 
            RowBox[{"PL", "[", 
             RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
           ")"}], 
          RowBox[{"\[CapitalDelta]1", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}], 
        ")"}]}]}]}], ";"}]}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10002.},
 CellChangeTimes->{{3.592047117982747*^9, 3.592047122541643*^9}, {
   3.592047158057968*^9, 3.592047244511454*^9}, {3.59204735131525*^9, 
   3.592047363011761*^9}, {3.592047467459888*^9, 3.592047469652076*^9}, {
   3.592047570618974*^9, 3.592047581871917*^9}, {3.592047669281968*^9, 
   3.592047674874717*^9}, {3.592047721613835*^9, 3.59204772190033*^9}, {
   3.592047760829012*^9, 3.59204781409741*^9}, {3.59204823057209*^9, 
   3.59204828320384*^9}, {3.592048754704242*^9, 3.592048762894891*^9}, {
   3.592049434587759*^9, 3.592049470456192*^9}, 3.592049507568486*^9, 
   3.592049554480911*^9, {3.592049627409471*^9, 3.592049704240957*^9}, 
   3.592049799670066*^9, {3.592049836723803*^9, 3.592049838926814*^9}, {
   3.592049984478686*^9, 3.592050016200452*^9}, {3.592050052173359*^9, 
   3.59205007113427*^9}, {3.592050104117317*^9, 3.59205016723869*^9}, {
   3.592050331355992*^9, 3.592050333504898*^9}, 3.592050419796643*^9, {
   3.592050463911436*^9, 3.592050503943142*^9}, 3.5947437274039183`*^9, {
   3.5959406194286823`*^9, 3.5959406633365383`*^9}, {3.5959407943406057`*^9, 
   3.595940801101321*^9}, {3.5959410483585176`*^9, 3.595941071271699*^9}, {
   3.595941107249441*^9, 3.5959411254202757`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["(* Simplify equations *)", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, {3.5959407095943537`*^9, 3.595940717144854*^9}, 
   3.59594203222393*^9, 3.595942542180497*^9, 3.5963681808654137`*^9},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[TextData[{
 "(* \nThis routine takes each equation (1-5) and simplifies it in such a way \
to obtain the form: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["C", "ij"], 
    RowBox[{
     SubscriptBox["\[PartialD]", "i"], 
     SubscriptBox["X", "j"]}]}], TraditionalForm]]],
 " + C = 0 (i = \[Tau], x, y, \[Eta], ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["X", "j"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"=", 
     SubscriptBox["u", "x"]}], ",", 
    SubscriptBox["u", "y"], ","}], TraditionalForm]]],
 " \[CapitalLambda], \[Xi], ",
 "\[CapitalTheta]",
 ").\n*)"
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.595941278450656*^9, 3.595941294053019*^9}, {
   3.595941400996941*^9, 3.595941550620653*^9}, {3.595941586828558*^9, 
   3.595941594403351*^9}, {3.5959416890738297`*^9, 3.595941912641343*^9}, 
   3.5959420322240057`*^9, 3.5959425421805553`*^9, 3.596368180865456*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "partial", " ", "derivatives", " ", "appearing", " ", "in", " ", "the", 
    " ", "equations"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"DerivAll", "=", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      "\[IndentingNewLine]", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"TauDeriv", "=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"NameT", "[", "i_", "]"}], ":=", 
     RowBox[{"Switch", "[", 
      RowBox[{
      "i", ",", "1", ",", "\"\<Lt\>\"", ",", "2", ",", "\"\<Xt\>\"", ",", "3",
        ",", "\"\<Uxt\>\"", ",", "4", ",", "\"\<Uyt\>\"", ",", "5", ",", 
       "\"\<Tt\>\""}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"SpaceDeriv", "=", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["ux", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["uy", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalLambda]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[Xi]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], ",", 
       RowBox[{
        SuperscriptBox["\[CapitalTheta]", 
         TagBox[
          RowBox[{"(", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
          Derivative],
         MultilineFunction->None], "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      "\[IndentingNewLine]", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"NameS", "[", "i_", "]"}], ":=", 
     RowBox[{"Switch", "[", 
      RowBox[{
      "i", ",", "\[IndentingNewLine]", "1", ",", "\"\<Uxe\>\"", ",", "2", 
       ",", "\"\<Uxx\>\"", ",", "3", ",", "\"\<Uxy\>\"", ",", 
       "\[IndentingNewLine]", "4", ",", "\"\<Uye\>\"", ",", "5", ",", 
       "\"\<Uyx\>\"", ",", "6", ",", "\"\<Uyy\>\"", ",", 
       "\[IndentingNewLine]", "7", ",", "\"\<Le\>\"", ",", "8", ",", 
       "\"\<Lx\>\"", ",", "9", ",", "\"\<Ly\>\"", ",", "\[IndentingNewLine]", 
       "10", ",", "\"\<Xe\>\"", ",", "11", ",", "\"\<Xx\>\"", ",", "12", ",", 
       "\"\<Xy\>\"", ",", "\[IndentingNewLine]", "13", ",", "\"\<Te\>\"", ",",
        "14", ",", "\"\<Tx\>\"", ",", "15", ",", "\"\<Ty\>\""}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.59212864388506*^9, 3.592128847520707*^9}, {
   3.592131236335462*^9, 3.592131242009772*^9}, 3.592133010925791*^9, 
   3.5947437274041653`*^9, {3.595941152956851*^9, 3.5959411796742887`*^9}, 
   3.595942032224169*^9, 3.595942542180655*^9, {3.5963029172295427`*^9, 
   3.596302963260594*^9}, 3.5963190033130007`*^9, 3.596365299180089*^9, {
   3.596365917729837*^9, 3.59636591877773*^9}, {3.5963677286418552`*^9, 
   3.5963677291492977`*^9}, 3.596368180865491*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"these", " ", "are", " ", "for", " ", "all", " ", "equations"}], 
   " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"EqArr", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"RemArr", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"FreeTermExplicit", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoefTau", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoefTau0", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoefSpace", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoefSpace0", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"Timing", "[", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
      RowBox[{"go", " ", "through", " ", "all", " ", "equations"}], ",", " ", 
      RowBox[{"j", " ", "-", " ", 
       RowBox[{"number", " ", "the", " ", "equation"}]}]}], " ", "*)"}], 
    "\[IndentingNewLine]", 
    RowBox[{"Do", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"A", "=", "0"}], ";", " ", 
       RowBox[{"(*", " ", 
        RowBox[{
        "contains", " ", "new", " ", "form", " ", "of", " ", "the", " ", 
         "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"B", "=", 
        RowBox[{"ToExpression", "[", 
         RowBox[{"\"\<Eqn\>\"", "<>", 
          RowBox[{"ToString", "[", "j", "]"}], "<>", 
          "\"\<LHS[\[Tau],\[Eta],x,y]\>\""}], "]"}]}], ";", " ", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"contains", " ", "non"}], "-", 
         RowBox[{
         "simplified", " ", "form", " ", "of", " ", "the", " ", 
          "equation"}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"go", " ", "through", " ", "tau", " ", "derivatives"}], ",", 
         " ", 
         RowBox[{"i", " ", "-", " ", 
          RowBox[{"number", " ", "the", " ", "time", " ", "derivatives"}]}]}],
         " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "these", " ", "are", " ", "for", " ", "each", " ", "equation", " ", 
         "independently"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"CoefT", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"CoefS", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"AExplicit", "=", "0"}], ";", "\[IndentingNewLine]", 
       RowBox[{"RemExplicit", "=", "0"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"name", "=", 
            RowBox[{"ToExpression", "[", 
             RowBox[{"\"\<C\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"NameT", "[", "i", "]"}], "]"}], "<>", 
              RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"value", "=", 
            RowBox[{"FullSimplify", "[", 
             RowBox[{"Coefficient", "[", 
              RowBox[{"B", ",", 
               RowBox[{
               "TauDeriv", "\[LeftDoubleBracket]", "i", 
                "\[RightDoubleBracket]"}]}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"value", "\[Equal]", "0"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"CoefTau0", ",", 
               RowBox[{"name", "\[Rule]", " ", "value"}]}], "]"}]}], "]"}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"CoefTau", ",", 
             RowBox[{"{", 
              RowBox[{"name", ",", "value"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"CoefT", ",", 
             RowBox[{"name", "->", "value"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"A", "+=", 
            RowBox[{
             RowBox[{
             "TauDeriv", "\[LeftDoubleBracket]", "i", 
              "\[RightDoubleBracket]"}], "name"}]}], ";"}], 
          "\[IndentingNewLine]", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"Length", "[", "TauDeriv", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"AExplicit", "+=", 
        RowBox[{"(", 
         RowBox[{"A", " ", "/.", "CoefT"}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"go", " ", "through", " ", "space", " ", "derivatives"}], 
         ",", " ", 
         RowBox[{"i", " ", "-", " ", 
          RowBox[{
          "number", " ", "the", " ", "space", " ", "derivatives"}]}]}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Rem", "=", "0"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"name", "=", 
            RowBox[{"ToExpression", "[", 
             RowBox[{"\"\<C\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"NameS", "[", "i", "]"}], "]"}], "<>", 
              RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"value", "=", 
            RowBox[{"FullSimplify", "[", 
             RowBox[{"Coefficient", "[", 
              RowBox[{"B", ",", 
               RowBox[{
               "SpaceDeriv", "\[LeftDoubleBracket]", "i", 
                "\[RightDoubleBracket]"}]}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"value", "\[Equal]", "0"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"CoefSpace0", ",", 
               RowBox[{"name", "\[Rule]", " ", "value"}]}], "]"}]}], "]"}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"CoefSpace", ",", 
             RowBox[{"{", 
              RowBox[{"name", ",", "value"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"CoefS", ",", 
             RowBox[{"name", "->", "value"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Rem", "+=", 
            RowBox[{
             RowBox[{
             "SpaceDeriv", "\[LeftDoubleBracket]", "i", 
              "\[RightDoubleBracket]"}], "name"}]}], ";"}], 
          "\[IndentingNewLine]", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"Length", "[", "SpaceDeriv", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"AExplicit", "+=", 
        RowBox[{"(", 
         RowBox[{"Rem", "/.", "CoefS"}], ")"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"general", " ", "form"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"RemArr", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ToExpression", "[", 
            RowBox[{"\"\<R\>\"", "<>", 
             RowBox[{"ToString", "[", "j", "]"}]}], "]"}], ",", "Rem"}], 
          "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"A", "+=", 
        RowBox[{"ToExpression", "[", 
         RowBox[{"\"\<R\>\"", "<>", 
          RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"EqArr", ",", "A"}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "calculate", " ", "the", " ", "free", " ", "term", " ", "for", " ", 
         "each", " ", "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"FreeTermExplicit", ",", 
         RowBox[{"FullSimplify", "[", 
          RowBox[{
           RowBox[{"Expand", "[", "B", "]"}], "-", "AExplicit"}], "]"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
        "FreeTermExplicit", "\[LeftDoubleBracket]", "j", 
         "\[RightDoubleBracket]"}], "]"}], ";"}], "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "5"}], "}"}]}], "]"}], 
    "\[IndentingNewLine]", "]"}]}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596302945837695*^9, 3.5963029915833187`*^9}, {
   3.5963030302708197`*^9, 3.5963033965651293`*^9}, {3.596319008131233*^9, 
   3.596319265141944*^9}, {3.5963193102640867`*^9, 3.596319313919016*^9}, {
   3.596319483321004*^9, 3.5963194837879744`*^9}, {3.596319607591012*^9, 
   3.596319672039578*^9}, {3.596319702480007*^9, 3.59631972957661*^9}, {
   3.59631976429036*^9, 3.596319764690858*^9}, {3.5963198336290617`*^9, 
   3.5963198382927513`*^9}, {3.5963202067614193`*^9, 3.596320219369665*^9}, {
   3.596320302758081*^9, 3.596320315609269*^9}, {3.596320372011347*^9, 
   3.596320432437316*^9}, {3.5963205526094913`*^9, 3.596320589922166*^9}, {
   3.596320660141994*^9, 3.596320680133305*^9}, {3.596320725381651*^9, 
   3.596320816679289*^9}, 3.596321097913673*^9, {3.5963211551263866`*^9, 
   3.596321333536831*^9}, {3.596322375193001*^9, 3.5963223765113993`*^9}, {
   3.596364741501418*^9, 3.5963647691448793`*^9}, {3.596364901609928*^9, 
   3.5963649340389214`*^9}, {3.596365143808375*^9, 3.596365182446103*^9}, {
   3.596365253018372*^9, 3.596365283751519*^9}, {3.596365328264678*^9, 
   3.5963653286147413`*^9}, {3.59636541565545*^9, 3.596365533054961*^9}, {
   3.596365566220401*^9, 3.596365601159873*^9}, {3.596365643727715*^9, 
   3.596366013226449*^9}, {3.596366048691934*^9, 3.596366102877318*^9}, {
   3.5963661397794113`*^9, 3.596366308826079*^9}, {3.596366342863537*^9, 
   3.596366347606902*^9}, {3.596366490461772*^9, 3.5963665133458347`*^9}, {
   3.5963665782911663`*^9, 3.596366690311458*^9}, {3.596366731778966*^9, 
   3.596366846997861*^9}, {3.596367002248555*^9, 3.596367041792933*^9}, {
   3.596367075202859*^9, 3.59636707549891*^9}, {3.596367112245317*^9, 
   3.596367118564271*^9}, {3.5963671526425333`*^9, 3.5963671544609528`*^9}, 
   3.596367208526898*^9, {3.5963672589899063`*^9, 3.596367296006893*^9}, {
   3.596367368428321*^9, 3.5963674274910316`*^9}, {3.596367726635873*^9, 
   3.596367837958633*^9}, {3.596367870429282*^9, 3.596367906037788*^9}, {
   3.596367958162212*^9, 3.5963679585384808`*^9}, {3.596367996547229*^9, 
   3.596368007014966*^9}, {3.5963680638034678`*^9, 3.596368097425579*^9}, 
   3.596368180865528*^9, {3.596368378684002*^9, 3.596368510439281*^9}, {
   3.596368601409543*^9, 3.596368728352118*^9}, {3.596368763427413*^9, 
   3.596368832073454*^9}, {3.59636886737409*^9, 3.5963689083104267`*^9}, {
   3.596368950341857*^9, 3.596368997087263*^9}}],

Cell[BoxData[
 RowBox[{"2", " ", 
  RowBox[{"\[CapitalGamma]", "[", 
   RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"R", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      RowBox[{"3", "/", "4"}]], " ", 
     SqrtBox[
      RowBox[{"1", "+", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]]}]}], 
   ")"}]}]], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.596368110052752*^9, {3.596368180865684*^9, 3.596368182679747*^9}, {
   3.596368912259781*^9, 3.59636892623029*^9}, 3.59636899762612*^9, 
   3.596373343918845*^9, 3.596456631958435*^9, 3.596993384452129*^9, 
   3.598049240474807*^9}],

Cell[BoxData["0"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.596368110052752*^9, {3.596368180865684*^9, 3.596368182679747*^9}, {
   3.596368912259781*^9, 3.59636892623029*^9}, 3.59636899762612*^9, 
   3.596373343918845*^9, 3.596456631958435*^9, 3.596993384452129*^9, 
   3.598049242519353*^9}],

Cell[BoxData["0"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.596368110052752*^9, {3.596368180865684*^9, 3.596368182679747*^9}, {
   3.596368912259781*^9, 3.59636892623029*^9}, 3.59636899762612*^9, 
   3.596373343918845*^9, 3.596456631958435*^9, 3.596993384452129*^9, 
   3.598049243520903*^9}],

Cell[BoxData["0"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.596368110052752*^9, {3.596368180865684*^9, 3.596368182679747*^9}, {
   3.596368912259781*^9, 3.59636892623029*^9}, 3.59636899762612*^9, 
   3.596373343918845*^9, 3.596456631958435*^9, 3.596993384452129*^9, 
   3.598049244753234*^9}],

Cell[BoxData["0"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.596368110052752*^9, {3.596368180865684*^9, 3.596368182679747*^9}, {
   3.596368912259781*^9, 3.59636892623029*^9}, 3.59636899762612*^9, 
   3.596373343918845*^9, 3.596456631958435*^9, 3.596993384452129*^9, 
   3.598049245675776*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.544409999999999`", ",", "Null"}], "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{
  3.5963669598745337`*^9, {3.596367027700568*^9, 3.596367046899541*^9}, 
   3.596367081242375*^9, 3.596367122601386*^9, 3.596367160631606*^9, 
   3.5963672122742233`*^9, 3.596367263242791*^9, {3.596367302617344*^9, 
   3.596367316969898*^9}, {3.59636740888944*^9, 3.596367436585064*^9}, 
   3.596367627144017*^9, 3.59636776913874*^9, 3.596367800885915*^9, 
   3.596367842877787*^9, 3.596367883039772*^9, 3.596367913483803*^9, 
   3.596368039603284*^9, {3.5963681654684753`*^9, 3.596368180865789*^9}, 
   3.596368213754608*^9, {3.5963689129919024`*^9, 3.59636892647149*^9}, 
   3.596368997816263*^9, 3.596373345088457*^9, 3.596456633142316*^9, 
   3.5969933928281813`*^9, 3.5980492459326773`*^9}],

Cell[BoxData[{"CoefSpace0", "\[IndentingNewLine]", "CoefTau0"}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369560165206*^9, 3.5963695657282753`*^9}, 
   3.5969934328774137`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"CUxy1", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyx1", "\[Rule]", "0"}], ",", 
   RowBox[{"CTx1", "\[Rule]", "0"}], ",", 
   RowBox[{"CTy1", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxy2", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyx2", "\[Rule]", "0"}], ",", 
   RowBox[{"CTx2", "\[Rule]", "0"}], ",", 
   RowBox[{"CTy2", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxe3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxx3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxy3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUye3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyx3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyy3", "\[Rule]", "0"}], ",", 
   RowBox[{"CLx3", "\[Rule]", "0"}], ",", 
   RowBox[{"CLy3", "\[Rule]", "0"}], ",", 
   RowBox[{"CXx3", "\[Rule]", "0"}], ",", 
   RowBox[{"CXy3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUye4", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyx4", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyy4", "\[Rule]", "0"}], ",", 
   RowBox[{"CTx4", "\[Rule]", "0"}], ",", 
   RowBox[{"CTy4", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxe5", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxx5", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxy5", "\[Rule]", "0"}], ",", 
   RowBox[{"CTx5", "\[Rule]", "0"}], ",", 
   RowBox[{"CTy5", "\[Rule]", "0"}]}], "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369555020281*^9, 3.596369576014613*^9}, 
   3.596373359375464*^9, 3.5964566340261087`*^9, 3.5969934328774137`*^9, 
   3.598049246252749*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"CUxt3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyt3", "\[Rule]", "0"}], ",", 
   RowBox[{"CUyt4", "\[Rule]", "0"}], ",", 
   RowBox[{"CUxt5", "\[Rule]", "0"}]}], "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369555020281*^9, 3.596369576014613*^9}, 
   3.596373359375464*^9, 3.5964566340261087`*^9, 3.5969934328774137`*^9, 
   3.598049246264702*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"EqArrs", "=", 
   RowBox[{"EqArr", "/.", "CoefTau0"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Eq1", "=", 
  RowBox[{
  "EqArrs", "\[LeftDoubleBracket]", "1", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Eq2", "=", 
  RowBox[{
  "EqArrs", "\[LeftDoubleBracket]", "2", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Eq3", "=", 
  RowBox[{
  "EqArrs", "\[LeftDoubleBracket]", "3", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Eq4", "=", 
  RowBox[{
  "EqArrs", "\[LeftDoubleBracket]", "4", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Eq5", "=", 
  RowBox[{
  "EqArrs", "\[LeftDoubleBracket]", "5", 
   "\[RightDoubleBracket]"}]}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.5963210805372057`*^9, 3.596321080736601*^9}, {
   3.596321443128709*^9, 3.596321483031415*^9}, {3.596322400972788*^9, 
   3.596322439629457*^9}, {3.596369054297123*^9, 3.5963690958254833`*^9}, {
   3.596369431320302*^9, 3.596369481411497*^9}, 3.5969934328774137`*^9}],

Cell[BoxData[
 RowBox[{"R1", "+", 
  RowBox[{"CUxt1", " ", 
   RowBox[{
    SuperscriptBox["ux", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CUyt1", " ", 
   RowBox[{
    SuperscriptBox["uy", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CTt1", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalTheta]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CLt1", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalLambda]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CXt1", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369464253817*^9, 3.596369481894071*^9}, 
   3.596369577446813*^9, 3.596373361318131*^9, 3.596456635185569*^9, 
   3.5969934328774137`*^9, 3.596993478161522*^9, 3.598049246289968*^9}],

Cell[BoxData[
 RowBox[{"R2", "+", 
  RowBox[{"CUxt2", " ", 
   RowBox[{
    SuperscriptBox["ux", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CUyt2", " ", 
   RowBox[{
    SuperscriptBox["uy", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CTt2", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalTheta]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CLt2", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalLambda]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CXt2", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369464253817*^9, 3.596369481894071*^9}, 
   3.596369577446813*^9, 3.596373361318131*^9, 3.596456635185569*^9, 
   3.5969934328774137`*^9, 3.596993478161522*^9, 3.598049246310604*^9}],

Cell[BoxData[
 RowBox[{"R3", "+", 
  RowBox[{"CTt3", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalTheta]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CLt3", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalLambda]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CXt3", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369464253817*^9, 3.596369481894071*^9}, 
   3.596369577446813*^9, 3.596373361318131*^9, 3.596456635185569*^9, 
   3.5969934328774137`*^9, 3.596993478161522*^9, 3.598049246325534*^9}],

Cell[BoxData[
 RowBox[{"R4", "+", 
  RowBox[{"CUxt4", " ", 
   RowBox[{
    SuperscriptBox["ux", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CTt4", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalTheta]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CLt4", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalLambda]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CXt4", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369464253817*^9, 3.596369481894071*^9}, 
   3.596369577446813*^9, 3.596373361318131*^9, 3.596456635185569*^9, 
   3.5969934328774137`*^9, 3.596993478161522*^9, 3.598049246339966*^9}],

Cell[BoxData[
 RowBox[{"R5", "+", 
  RowBox[{"CUyt5", " ", 
   RowBox[{
    SuperscriptBox["uy", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CTt5", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalTheta]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CLt5", " ", 
   RowBox[{
    SuperscriptBox["\[CapitalLambda]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
  RowBox[{"CXt5", " ", 
   RowBox[{
    SuperscriptBox["\[Xi]", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], 
    "]"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369464253817*^9, 3.596369481894071*^9}, 
   3.596369577446813*^9, 3.596373361318131*^9, 3.596456635185569*^9, 
   3.5969934328774137`*^9, 3.596993478161522*^9, 3.598049246356271*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"RemArrs", "=", 
   RowBox[{"RemArr", "/.", "CoefSpace0"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"RemArrs", "//", "MatrixForm"}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369372188517*^9, 3.596369380533084*^9}, {
   3.596369419791689*^9, 3.5963694217241583`*^9}, {3.5963695220041637`*^9, 
   3.596369531003915*^9}, {3.5963701146991663`*^9, 3.596370129793767*^9}, 
   3.5969934328774137`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"R1", 
      RowBox[{
       RowBox[{"CUyy1", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLy1", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXy1", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxx1", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLx1", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXx1", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxe1", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUye1", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTe1", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLe1", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXe1", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]},
     {"R2", 
      RowBox[{
       RowBox[{"CUyy2", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLy2", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXy2", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxx2", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLx2", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXx2", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxe2", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUye2", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTe2", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLe2", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXe2", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]},
     {"R3", 
      RowBox[{
       RowBox[{"CTy3", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTx3", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTe3", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLe3", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXe3", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]},
     {"R4", 
      RowBox[{
       RowBox[{"CUxy4", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLy4", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXy4", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxx4", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLx4", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXx4", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUxe4", " ", 
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTe4", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLe4", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXe4", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]},
     {"R5", 
      RowBox[{
       RowBox[{"CUyy5", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLy5", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXy5", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUyx5", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLx5", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXx5", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CUye5", " ", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CTe5", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CLe5", " ", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], "+", 
       RowBox[{"CXe5", " ", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.},
 CellChangeTimes->{{3.596369375173786*^9, 3.596369381089519*^9}, {
   3.596369523537093*^9, 3.5963695314086523`*^9}, 3.5963695904390078`*^9, 
   3.596370131027452*^9, 3.596373363019459*^9, 3.596456636828483*^9, 
   3.5969934328774137`*^9, 3.596993480541525*^9, 3.5980492465905437`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "(* Derive the expressions for time derivatives ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", "i"], 
    SubscriptBox["X", "ij"]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " *)"
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, {3.5959407095943537`*^9, 3.595940717144854*^9}, 
   3.59594203222393*^9, {3.595942542180497*^9, 3.5959425683564987`*^9}, 
   3.596370253355792*^9, 3.5969935779317875`*^9, {3.5980495581745167`*^9, 
   3.598049603100524*^9}},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Sol", " ", "=", " ", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Eq1", "==", " ", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Eq2", "==", " ", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Eq3", "==", " ", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Eq4", "==", " ", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Eq5", "==", " ", "0"}]}], "\[IndentingNewLine]", "}"}], ",", 
      RowBox[{"{", "\n", 
       RowBox[{
        RowBox[{
         SuperscriptBox["ux", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}],
          "]"}], ",", " ", "\n", 
        RowBox[{
         SuperscriptBox["uy", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}],
          "]"}], ",", " ", "\n", 
        RowBox[{
         SuperscriptBox["\[CapitalLambda]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}],
          "]"}], ",", "  ", "\n", 
        RowBox[{
         SuperscriptBox["\[Xi]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}],
          "]"}], ",", "\n", 
        RowBox[{
         SuperscriptBox["\[CapitalTheta]", 
          TagBox[
           RowBox[{"(", 
            RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
           Derivative],
          MultilineFunction->None], "[", 
         RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}],
          "]"}]}], "\[IndentingNewLine]", "}"}]}], "]"}], 
    "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Sol", "\[LeftDoubleBracket]", 
  RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"Sol", "\[LeftDoubleBracket]", 
  RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"Sol", "\[LeftDoubleBracket]", 
  RowBox[{"3", ",", "1"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"Sol", "\[LeftDoubleBracket]", 
  RowBox[{"4", ",", "1"}], "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"Sol", "\[LeftDoubleBracket]", 
  RowBox[{"5", ",", "1"}], "\[RightDoubleBracket]"}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596321708414171*^9, 3.596321715797854*^9}, {
  3.5963224552110662`*^9, 3.5963224715445747`*^9}, {3.5963702533558617`*^9, 
  3.596370256942141*^9}, {3.5969935501297255`*^9, 3.5969935779317875`*^9}}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["ux", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596993566116761*^9, 3.5969935779317875`*^9}, 
   3.598049259512051*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["uy", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596993566116761*^9, 3.5969935779317875`*^9}, 
   3.598049259520797*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[CapitalLambda]", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596993566116761*^9, 3.5969935779317875`*^9}, 
   3.598049259528873*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[Xi]", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596993566116761*^9, 3.5969935779317875`*^9}, 
   3.598049259537856*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["\[CapitalTheta]", 
   TagBox[
    RowBox[{"(", 
     RowBox[{"1", ",", "0", ",", "0", ",", "0"}], ")"}],
    Derivative],
   MultilineFunction->None], "[", 
  RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596993566116761*^9, 3.5969935779317875`*^9}, 
   3.598049259546976*^9}],

Cell[BoxData[
 RowBox[{"uxSol", "=", 
  RowBox[{
   RowBox[{"Sol", "\[LeftDoubleBracket]", 
    RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], "//", 
   "Simplify"}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.5963217196993847`*^9, 3.596321743191835*^9}, {
   3.596321810981676*^9, 3.596321831478396*^9}, 3.5963219038985567`*^9, 
   3.596321976489719*^9, 3.596322078734948*^9, 3.5963222027429047`*^9, {
   3.596322489831925*^9, 3.596322501747718*^9}, {3.596322558912961*^9, 
   3.5963225740902033`*^9}, {3.596323137878951*^9, 3.596323158529068*^9}, 
   3.596369317005888*^9, {3.596369620263299*^9, 3.596369644067586*^9}, {
   3.596369957443859*^9, 3.596369959178669*^9}, {3.596370994046447*^9, 
   3.596370995563774*^9}, 3.596372496555923*^9, 3.59699414986125*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"CLt5", " ", "CTt4", " ", "CUyt2", " ", "CXt3", " ", "R1"}], "-", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUyt5", " ", "CXt3", " ", "R1"}], "-", 
    RowBox[{"CLt5", " ", "CTt3", " ", "CUyt2", " ", "CXt4", " ", "R1"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUyt5", " ", "CXt4", " ", "R1"}], "-", 
    RowBox[{"CLt5", " ", "CTt4", " ", "CUyt1", " ", "CXt3", " ", "R2"}], "+", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUyt5", " ", "CXt3", " ", "R2"}], "+", 
    RowBox[{"CLt5", " ", "CTt3", " ", "CUyt1", " ", "CXt4", " ", "R2"}], "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUyt5", " ", "CXt4", " ", "R2"}], "-", 
    RowBox[{"CLt5", " ", "CTt4", " ", "CUyt2", " ", "CXt1", " ", "R3"}], "+", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUyt5", " ", "CXt1", " ", "R3"}], "+", 
    RowBox[{"CLt5", " ", "CTt4", " ", "CUyt1", " ", "CXt2", " ", "R3"}], "-", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUyt5", " ", "CXt2", " ", "R3"}], "-", 
    RowBox[{"CLt5", " ", "CTt2", " ", "CUyt1", " ", "CXt4", " ", "R3"}], "+", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUyt1", " ", "CXt4", " ", "R3"}], "+", 
    RowBox[{"CLt5", " ", "CTt1", " ", "CUyt2", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUyt2", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUyt5", " ", "CXt4", " ", "R3"}], "+", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUyt5", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUyt1", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUyt2", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt5", " ", "CTt3", " ", "CUyt2", " ", "CXt1", " ", "R4"}], "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUyt5", " ", "CXt1", " ", "R4"}], "-", 
    RowBox[{"CLt5", " ", "CTt3", " ", "CUyt1", " ", "CXt2", " ", "R4"}], "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUyt5", " ", "CXt2", " ", "R4"}], "+", 
    RowBox[{"CLt5", " ", "CTt2", " ", "CUyt1", " ", "CXt3", " ", "R4"}], "-", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUyt1", " ", "CXt3", " ", "R4"}], "-", 
    RowBox[{"CLt5", " ", "CTt1", " ", "CUyt2", " ", "CXt3", " ", "R4"}], "+", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUyt2", " ", "CXt3", " ", "R4"}], "+", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUyt5", " ", "CXt3", " ", "R4"}], "-", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUyt5", " ", "CXt3", " ", "R4"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUyt1", " ", "CXt5", " ", "R4"}], "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUyt2", " ", "CXt5", " ", "R4"}], "+", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUyt1", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUyt2", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUyt1", " ", "CXt4", " ", "R5"}], "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUyt2", " ", "CXt4", " ", "R5"}], "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "CTt2"}], " ", "CUyt5", " ", "CXt4", " ", "R1"}], "+", 
       RowBox[{"CTt1", " ", "CUyt5", " ", "CXt4", " ", "R2"}], "+", 
       RowBox[{"CTt2", " ", "CUyt5", " ", "CXt1", " ", "R4"}], "-", 
       RowBox[{"CTt1", " ", "CUyt5", " ", "CXt2", " ", "R4"}], "-", 
       RowBox[{"CTt2", " ", "CUyt1", " ", "CXt5", " ", "R4"}], "+", 
       RowBox[{"CTt1", " ", "CUyt2", " ", "CXt5", " ", "R4"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUyt2", " ", "CXt4", " ", "R1"}], "-", 
          RowBox[{"CUyt1", " ", "CXt4", " ", "R2"}], "-", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R4"}], "+", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R4"}]}], ")"}]}], "+", 
       RowBox[{"CTt2", " ", "CUyt1", " ", "CXt4", " ", "R5"}], "-", 
       RowBox[{"CTt1", " ", "CUyt2", " ", "CXt4", " ", "R5"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUyt5", " ", "CXt2", " ", "R1"}], "-", 
          RowBox[{"CUyt2", " ", "CXt5", " ", "R1"}], "-", 
          RowBox[{"CUyt5", " ", "CXt1", " ", "R2"}], "+", 
          RowBox[{"CUyt1", " ", "CXt5", " ", "R2"}], "+", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R5"}], "-", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R5"}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{"CLt4", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CTt2", " ", "CUyt5", " ", "CXt3", " ", "R1"}], "-", 
       RowBox[{"CTt1", " ", "CUyt5", " ", "CXt3", " ", "R2"}], "-", 
       RowBox[{"CTt2", " ", "CUyt5", " ", "CXt1", " ", "R3"}], "+", 
       RowBox[{"CTt1", " ", "CUyt5", " ", "CXt2", " ", "R3"}], "+", 
       RowBox[{"CTt2", " ", "CUyt1", " ", "CXt5", " ", "R3"}], "-", 
       RowBox[{"CTt1", " ", "CUyt2", " ", "CXt5", " ", "R3"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUyt2"}], " ", "CXt3", " ", "R1"}], "+", 
          RowBox[{"CUyt1", " ", "CXt3", " ", "R2"}], "+", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R3"}], "-", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R3"}]}], ")"}]}], "-", 
       RowBox[{"CTt2", " ", "CUyt1", " ", "CXt3", " ", "R5"}], "+", 
       RowBox[{"CTt1", " ", "CUyt2", " ", "CXt3", " ", "R5"}], "+", 
       RowBox[{"CTt3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUyt5"}], " ", "CXt2", " ", "R1"}], "+", 
          RowBox[{"CUyt2", " ", "CXt5", " ", "R1"}], "+", 
          RowBox[{"CUyt5", " ", "CXt1", " ", "R2"}], "-", 
          RowBox[{"CUyt1", " ", "CXt5", " ", "R2"}], "-", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R5"}], "+", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R5"}]}], ")"}]}]}], ")"}]}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "CLt4"}], " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", 
     "CXt1"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt2"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt2", " ", "CUyt1", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt4", " ", "CUyt1", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt1", " ", "CUyt2", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt4", " ", "CUyt2", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], 
    "+", 
    RowBox[{"CLt5", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt1"}], "-", 
          RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1"}], "-", 
          RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt2"}], "+", 
          RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2"}]}], ")"}], " ", 
        "CXt3"}], "+", 
       RowBox[{"CTt3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt4"}], " ", "CUyt2", " ", "CXt1"}], "+", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "-", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CUyt1", " ", "CXt5"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt2", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "CTt2"}], " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], "+", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], "+", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt1"}], "-", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "+", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}], "+", 
       RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], "-", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt1"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt2"}], "-", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt5"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
      ")"}]}]}], ")"}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{3.596372438123333*^9, 3.596372497035407*^9, 
  3.5964566692344847`*^9, 3.596993953806793*^9, 3.59699414986125*^9, 
  3.598049267885459*^9}],

Cell[BoxData[
 RowBox[{"uySol", "=", 
  RowBox[{
   RowBox[{"Sol", "\[LeftDoubleBracket]", 
    RowBox[{"2", ",", "2"}], "\[RightDoubleBracket]"}], "//", 
   "Simplify"}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596369964426653*^9, 3.596369966961493*^9}, 
   3.596372355396862*^9, {3.5963725101807413`*^9, 3.5963725105534163`*^9}, 
   3.59699414986125*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt2", " ", "CXt3", " ", "R1"}], "-", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt4", " ", "CXt3", " ", "R1"}], "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CXt5", " ", "R1"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CXt5", " ", "R1"}], "-", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt1", " ", "CXt3", " ", "R2"}], "+", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt4", " ", "CXt3", " ", "R2"}], "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CXt5", " ", "R2"}], "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CXt5", " ", "R2"}], "-", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt2", " ", "CXt1", " ", "R3"}], "+", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt4", " ", "CXt1", " ", "R3"}], "+", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt1", " ", "CXt2", " ", "R3"}], "-", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt4", " ", "CXt2", " ", "R3"}], "-", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt1", " ", "CXt4", " ", "R3"}], "+", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt2", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CLt4", " ", "CTt2", " ", "CUxt1", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUxt1", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt4", " ", "CTt1", " ", "CUxt2", " ", "CXt5", " ", "R3"}], "-", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUxt2", " ", "CXt5", " ", "R3"}], "-", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUxt4", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUxt4", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt1", " ", "CXt3", " ", "R4"}], "-", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt2", " ", "CXt3", " ", "R4"}], "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt1", " ", "CXt5", " ", "R4"}], "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt2", " ", "CXt5", " ", "R4"}], "+", 
    RowBox[{"CLt5", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CTt2", " ", "CUxt4", " ", "CXt3", " ", "R1"}], "-", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CXt3", " ", "R2"}], "-", 
       RowBox[{"CTt2", " ", "CUxt4", " ", "CXt1", " ", "R3"}], "+", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CXt2", " ", "R3"}], "+", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CXt4", " ", "R3"}], "-", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CXt4", " ", "R3"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt2"}], " ", "CXt3", " ", "R1"}], "+", 
          RowBox[{"CUxt1", " ", "CXt3", " ", "R2"}], "+", 
          RowBox[{"CUxt2", " ", "CXt1", " ", "R3"}], "-", 
          RowBox[{"CUxt1", " ", "CXt2", " ", "R3"}]}], ")"}]}], "-", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CXt3", " ", "R4"}], "+", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CXt3", " ", "R4"}], "+", 
       RowBox[{"CTt3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt4"}], " ", "CXt2", " ", "R1"}], "+", 
          RowBox[{"CUxt2", " ", "CXt4", " ", "R1"}], "+", 
          RowBox[{"CUxt4", " ", "CXt1", " ", "R2"}], "-", 
          RowBox[{"CUxt1", " ", "CXt4", " ", "R2"}], "-", 
          RowBox[{"CUxt2", " ", "CXt1", " ", "R4"}], "+", 
          RowBox[{"CUxt1", " ", "CXt2", " ", "R4"}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CXt1", " ", "R5"}], "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CXt1", " ", "R5"}], "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CXt2", " ", "R5"}], "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CXt2", " ", "R5"}], "+", 
    RowBox[{"CLt4", " ", "CTt2", " ", "CUxt1", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUxt1", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CLt4", " ", "CTt1", " ", "CUxt2", " ", "CXt3", " ", "R5"}], "+", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUxt2", " ", "CXt3", " ", "R5"}], "+", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUxt4", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUxt4", " ", "CXt3", " ", "R5"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt1", " ", "CXt4", " ", "R5"}], "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt2", " ", "CXt4", " ", "R5"}], "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "CTt2"}], " ", "CUxt4", " ", "CXt5", " ", "R1"}], "+", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CXt5", " ", "R2"}], "+", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CXt5", " ", "R4"}], "-", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CXt5", " ", "R4"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt4", " ", "CXt2", " ", "R1"}], "-", 
          RowBox[{"CUxt2", " ", "CXt4", " ", "R1"}], "-", 
          RowBox[{"CUxt4", " ", "CXt1", " ", "R2"}], "+", 
          RowBox[{"CUxt1", " ", "CXt4", " ", "R2"}], "+", 
          RowBox[{"CUxt2", " ", "CXt1", " ", "R4"}], "-", 
          RowBox[{"CUxt1", " ", "CXt2", " ", "R4"}]}], ")"}]}], "+", 
       RowBox[{"CTt2", " ", "CUxt4", " ", "CXt1", " ", "R5"}], "-", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CXt2", " ", "R5"}], "-", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CXt4", " ", "R5"}], "+", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CXt4", " ", "R5"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt2", " ", "CXt5", " ", "R1"}], "-", 
          RowBox[{"CUxt1", " ", "CXt5", " ", "R2"}], "-", 
          RowBox[{"CUxt2", " ", "CXt1", " ", "R5"}], "+", 
          RowBox[{"CUxt1", " ", "CXt2", " ", "R5"}]}], ")"}]}]}], ")"}]}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "CLt4"}], " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", 
     "CXt1"}], "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt2"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt2", " ", "CUyt1", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt4", " ", "CUyt1", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt1", " ", "CUyt2", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt4", " ", "CUyt2", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], 
    "+", 
    RowBox[{"CLt5", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt1"}], "-", 
          RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1"}], "-", 
          RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt2"}], "+", 
          RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2"}]}], ")"}], " ", 
        "CXt3"}], "+", 
       RowBox[{"CTt3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt4"}], " ", "CUyt2", " ", "CXt1"}], "+", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "-", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CUyt1", " ", "CXt5"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt2", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "CTt2"}], " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], "+", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], "+", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt1"}], "-", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "+", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}], "+", 
       RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], "-", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt1"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt2"}], "-", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt5"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
      ")"}]}]}], ")"}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{3.596369969045207*^9, 3.596372356611495*^9, 
  3.5963725118483753`*^9, 3.596994070327067*^9, 3.59699414986125*^9, 
  3.598049276878344*^9}],

Cell[BoxData[
 RowBox[{"LSol", "=", 
  RowBox[{
   RowBox[{"Sol", "\[LeftDoubleBracket]", 
    RowBox[{"3", ",", "2"}], "\[RightDoubleBracket]"}], "//", 
   "Simplify"}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.5963699755465527`*^9, 3.596369980346613*^9}, 
   3.596371023046789*^9, 3.596372723202506*^9, 3.59699414986125*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "CTt4"}], " ", "CUxt2", " ", "CUyt5", " ", "CXt3", " ", 
     "R1"}], "+", 
    RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt3", " ", "R1"}], "+", 
    RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt5", " ", "CXt3", " ", "R2"}], "-", 
    RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt3", " ", "R2"}], "+", 
    RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt5", " ", "CXt1", " ", "R3"}], "-", 
    RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt1", " ", "R3"}], "-", 
    RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt5", " ", "CXt2", " ", "R3"}], "+", 
    RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt2", " ", "R3"}], "+", 
    RowBox[{"CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt4", " ", "R3"}], "-", 
    RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt1", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1", " ", "CXt5", " ", "R3"}], "+", 
    RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt2", " ", "CXt5", " ", "R3"}], "-", 
    RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2", " ", "CXt5", " ", "R3"}], "-", 
    RowBox[{"CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt3", " ", "R4"}], "+", 
    RowBox[{"CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt3", " ", "R4"}], "+", 
    RowBox[{"CTt5", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CUxt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUyt2"}], " ", "CXt3", " ", "R1"}], "+", 
          RowBox[{"CUyt1", " ", "CXt3", " ", "R2"}], "+", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R3"}], "-", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R3"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt2", " ", "CUyt1"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt2"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CXt4", " ", "R3"}], "-", 
          RowBox[{"CXt3", " ", "R4"}]}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt1", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1", " ", "CXt3", " ", "R5"}], "-", 
    RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt2", " ", "CXt3", " ", "R5"}], "+", 
    RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2", " ", "CXt3", " ", "R5"}], "+", 
    RowBox[{"CTt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CUxt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUyt5"}], " ", "CXt2", " ", "R1"}], "+", 
          RowBox[{"CUyt2", " ", "CXt5", " ", "R1"}], "+", 
          RowBox[{"CUyt5", " ", "CXt1", " ", "R2"}], "-", 
          RowBox[{"CUyt1", " ", "CXt5", " ", "R2"}], "-", 
          RowBox[{"CUyt2", " ", "CXt1", " ", "R5"}], "+", 
          RowBox[{"CUyt1", " ", "CXt2", " ", "R5"}]}], ")"}]}], "+", 
       RowBox[{"CUxt2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUyt5", " ", "CXt4", " ", "R1"}], "-", 
          RowBox[{"CUyt5", " ", "CXt1", " ", "R4"}], "+", 
          RowBox[{"CUyt1", " ", "CXt5", " ", "R4"}], "-", 
          RowBox[{"CUyt1", " ", "CXt4", " ", "R5"}]}], ")"}]}], "+", 
       RowBox[{"CUxt1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUyt5"}], " ", "CXt4", " ", "R2"}], "+", 
          RowBox[{"CUyt5", " ", "CXt2", " ", "R4"}], "-", 
          RowBox[{"CUyt2", " ", "CXt5", " ", "R4"}], "+", 
          RowBox[{"CUyt2", " ", "CXt4", " ", "R5"}]}], ")"}]}]}], ")"}]}]}], 
   ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", "CXt1"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt2"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt2", " ", "CUyt1", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt5", " ", "CUxt4", " ", "CUyt1", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt5", " ", "CUxt1", " ", "CUyt2", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt5", " ", "CUxt4", " ", "CUyt2", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt2", " ", "CTt4", " ", "CUxt1", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt1", " ", "CTt4", " ", "CUxt2", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], 
    "+", 
    RowBox[{"CLt5", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CTt4"}], " ", "CUxt2", " ", "CUyt1"}], "+", 
          RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1"}], "+", 
          RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt2"}], "-", 
          RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2"}]}], ")"}], " ", 
        "CXt3"}], "+", 
       RowBox[{"CTt3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt1"}], "-", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "+", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}]}], ")"}]}], 
    "-", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt2", " ", "CUyt1", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt2", " ", "CTt3", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt4", " ", "CTt3", " ", "CUxt1", " ", "CUyt2", " ", "CXt5"}], 
    "-", 
    RowBox[{"CLt1", " ", "CTt3", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], 
    "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5", " ", "CXt2"}], "-", 
       RowBox[{"CTt2", " ", "CUxt1", " ", "CUyt5", " ", "CXt4"}], "+", 
       RowBox[{"CTt1", " ", "CUxt2", " ", "CUyt5", " ", "CXt4"}], "+", 
       RowBox[{"CTt5", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt4"}], " ", "CUyt2", " ", "CXt1"}], "+", 
          RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt2"}], "-", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt4"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt4"}]}], ")"}]}], "-", 
       RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt1", " ", "CXt5"}], "+", 
       RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt2", " ", "CXt5"}], "+", 
       RowBox[{"CTt4", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "CUxt2"}], " ", "CUyt5", " ", "CXt1"}], "+", 
          RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt2"}], "+", 
          RowBox[{"CUxt2", " ", "CUyt1", " ", "CXt5"}], "-", 
          RowBox[{"CUxt1", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
      ")"}]}]}], ")"}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{3.5963699827892838`*^9, 3.596371023939478*^9, 
  3.596372723726927*^9, 3.596372755441601*^9, 3.5969940714070683`*^9, 
  3.59699414986125*^9, 3.598049283572199*^9}],

Cell[BoxData[
 RowBox[{"XSol", "=", 
  RowBox[{
   RowBox[{"Sol", "\[LeftDoubleBracket]", 
    RowBox[{"4", ",", "2"}], "\[RightDoubleBracket]"}], "//", 
   "Simplify"}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596369993216948*^9, 3.59636999961063*^9}, {
   3.59637233400517*^9, 3.596372336963835*^9}, {3.596372800683099*^9, 
   3.596372801775156*^9}, {3.596388641223503*^9, 3.596388690534914*^9}, {
   3.5963887527830153`*^9, 3.5963887571327753`*^9}, {3.5964566817521048`*^9, 
   3.59645669094*^9}, 3.59699414986125*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
        RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
        RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}], " ", "R3"}], 
    "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "CUxt4"}], " ", "CUyt5", " ", "R1"}], "+", 
       RowBox[{"CUxt1", " ", "CUyt5", " ", "R4"}], "+", 
       RowBox[{"CUxt4", " ", "CUyt1", " ", "R5"}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"CLt5", " ", "CTt3", " ", "CUxt4", " ", "CUyt1"}], "+", 
         RowBox[{"CTt3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"CLt4", " ", "CUxt1"}], "-", 
            RowBox[{"CLt1", " ", "CUxt4"}]}], ")"}], " ", "CUyt5"}], "-", 
         RowBox[{"CLt3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"CTt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
            RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
            RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}]}], ")"}], 
       " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
                RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
                RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
              "CXt3"}], "+", 
             RowBox[{"CLt3", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt2"}], "-", 
                RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt4"}], "-", 
                RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
            ")"}]}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
               RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
               RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
             "R3"}], "+", 
            RowBox[{"CLt3", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CUxt4", " ", "CUyt5", " ", "R1"}], "-", 
               RowBox[{"CUxt1", " ", "CUyt5", " ", "R4"}], "-", 
               RowBox[{"CUxt4", " ", "CUyt1", " ", "R5"}]}], ")"}]}]}], 
           ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
               RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
               RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
             "CXt3"}], "+", 
            RowBox[{"CLt3", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
               RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
               RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt5"}]}], ")"}]}]}], 
           ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
               RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
               RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
             "R3"}], "+", 
            RowBox[{"CLt3", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"CUxt4", " ", "CUyt5", " ", "R2"}], "-", 
               RowBox[{"CUxt2", " ", "CUyt5", " ", "R4"}], "-", 
               RowBox[{"CUxt4", " ", "CUyt2", " ", "R5"}]}], ")"}]}]}], 
           ")"}]}]}], ")"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CLt5", " ", "CTt3", " ", "CUxt4", " ", "CUyt2"}], "+", 
          RowBox[{"CTt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CLt4", " ", "CUxt2"}], "-", 
             RowBox[{"CLt2", " ", "CUxt4"}]}], ")"}], " ", "CUyt5"}], "-", 
          RowBox[{"CLt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CTt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
             RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
             RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}]}], ")"}],
         " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
             RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
             RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
           "CXt3"}], "+", 
          RowBox[{"CLt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
             RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
             RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt5"}]}], ")"}]}]}], 
         ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"CLt5", " ", "CTt3", " ", "CUxt4", " ", "CUyt1"}], "+", 
          RowBox[{"CTt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CLt4", " ", "CUxt1"}], "-", 
             RowBox[{"CLt1", " ", "CUxt4"}]}], ")"}], " ", "CUyt5"}], "-", 
          RowBox[{"CLt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CTt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
             RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
             RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}]}], ")"}],
         " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
             RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
             RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
           "CXt3"}], "+", 
          RowBox[{"CLt3", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt2"}], "-", 
             RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt4"}], "-", 
             RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
         ")"}]}]}], ")"}]}]}], ")"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
       RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
       RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", "CXt3"}], 
    "+", 
    RowBox[{"CLt3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
       RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
       RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt5"}]}], ")"}]}]}], 
   ")"}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{
  3.596372337874343*^9, 3.5963728024689503`*^9, {3.5963886241829023`*^9, 
   3.596388691176107*^9}, 3.596388757612402*^9, {3.596456674915537*^9, 
   3.5964566971785173`*^9}, 3.596459200958584*^9, 3.596994091559105*^9, 
   3.59699414986125*^9, 3.5980493114714403`*^9}],

Cell[BoxData[
 RowBox[{"TSol", "=", 
  RowBox[{
   RowBox[{"Sol", "\[LeftDoubleBracket]", 
    RowBox[{"5", ",", "2"}], "\[RightDoubleBracket]"}], "//", 
   "Simplify"}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{{3.596370013985223*^9, 3.596370015545307*^9}, 
   3.596371072757362*^9, 3.596372809722912*^9, 3.59699414987125*^9}],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
            RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
            RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
          "CXt3"}], "+", 
         RowBox[{"CLt3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt2"}], "-", 
            RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt4"}], "-", 
            RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
        ")"}]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
           RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
           RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", "R3"}],
         "+", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CUxt4", " ", "CUyt5", " ", "R1"}], "-", 
           RowBox[{"CUxt1", " ", "CUyt5", " ", "R4"}], "-", 
           RowBox[{"CUxt4", " ", "CUyt1", " ", "R5"}]}], ")"}]}]}], ")"}]}], 
     "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
           RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
           RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
         "CXt3"}], "+", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
           RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
           RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt5"}]}], ")"}]}]}], ")"}], 
      " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
           RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
           RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", "R3"}],
         "+", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CUxt4", " ", "CUyt5", " ", "R2"}], "-", 
           RowBox[{"CUxt2", " ", "CUyt5", " ", "R4"}], "-", 
           RowBox[{"CUxt4", " ", "CUyt2", " ", "R5"}]}], ")"}]}]}], ")"}]}]}],
     ")"}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"CLt5", " ", "CTt3", " ", "CUxt4", " ", "CUyt2"}], "+", 
        RowBox[{"CTt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt4", " ", "CUxt2"}], "-", 
           RowBox[{"CLt2", " ", "CUxt4"}]}], ")"}], " ", "CUyt5"}], "-", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CTt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
           RowBox[{"CTt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
           RowBox[{"CTt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}]}], ")"}], 
      " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
           RowBox[{"CLt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
           RowBox[{"CLt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
         "CXt3"}], "+", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt1"}], "-", 
           RowBox[{"CUxt1", " ", "CUyt5", " ", "CXt4"}], "-", 
           RowBox[{"CUxt4", " ", "CUyt1", " ", "CXt5"}]}], ")"}]}]}], ")"}]}],
      "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"CLt5", " ", "CTt3", " ", "CUxt4", " ", "CUyt1"}], "+", 
        RowBox[{"CTt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt4", " ", "CUxt1"}], "-", 
           RowBox[{"CLt1", " ", "CUxt4"}]}], ")"}], " ", "CUyt5"}], "-", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CTt5", " ", "CUxt4", " ", "CUyt1"}], "+", 
           RowBox[{"CTt4", " ", "CUxt1", " ", "CUyt5"}], "-", 
           RowBox[{"CTt1", " ", "CUxt4", " ", "CUyt5"}]}], ")"}]}]}], ")"}], 
      " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CLt5", " ", "CUxt4", " ", "CUyt2"}], "+", 
           RowBox[{"CLt4", " ", "CUxt2", " ", "CUyt5"}], "-", 
           RowBox[{"CLt2", " ", "CUxt4", " ", "CUyt5"}]}], ")"}], " ", 
         "CXt3"}], "+", 
        RowBox[{"CLt3", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"CUxt4", " ", "CUyt5", " ", "CXt2"}], "-", 
           RowBox[{"CUxt2", " ", "CUyt5", " ", "CXt4"}], "-", 
           RowBox[{"CUxt4", " ", "CUyt2", " ", "CXt5"}]}], ")"}]}]}], 
       ")"}]}]}], ")"}]}]}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.},
 CellChangeTimes->{3.596370016537695*^9, 3.596371073527665*^9, 
  3.596372810326844*^9, 3.5963728761792316`*^9, 3.596373011971154*^9, 
  3.5969941139951887`*^9, 3.59699414987125*^9, 3.598049312698044*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
(* Define names of variables used by RK algorithm in C++ code *)\
\>", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10005.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, {3.5959407095943537`*^9, 3.595940717144854*^9}, 
   3.59594203222393*^9, {3.595942542180497*^9, 3.5959425683564987`*^9}, 
   3.596370253355792*^9, 3.5968388106256423`*^9, {3.596841879589353*^9, 
   3.596841897801182*^9}, {3.598049506100008*^9, 3.5980495199977417`*^9}},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "Y", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Defs", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      SqrtBox[
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"ux", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"], 
        "+", 
        SuperscriptBox[
         RowBox[{"uy", "[", 
          RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
         "2"]}]], "\[Rule]", " ", "U0"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      FractionBox["1", 
       SqrtBox[
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"ux", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"],
          "+", 
         SuperscriptBox[
          RowBox[{"uy", "[", 
           RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
          "2"]}]]], "\[Rule]", 
      FractionBox["1", "U0"]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"1", "+", 
       SuperscriptBox[
        RowBox[{"ux", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{"uy", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"]}],
       "\[Rule]", " ", 
      SuperscriptBox["U0", "2"]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        RowBox[{"ux", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"], 
       "+", 
       SuperscriptBox[
        RowBox[{"uy", "[", 
         RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "2"]}],
       "\[Rule]", " ", 
      RowBox[{
       SuperscriptBox["U0", "2"], "-", "1"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Eta]", "-", 
       RowBox[{"\[CapitalTheta]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      "\[Rule]", " ", "Y"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"1", "+", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}]}], 
      "\[Rule]", "X1"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"RT", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "RT"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"RL", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", " ", "RL"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"R", "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      " ", "\[Rule]", "R"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"P", "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "P"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[CurlyEpsilon]", "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "En"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["P", "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "Pp"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CurlyEpsilon]", "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"\[CapitalLambda]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "Enp"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["RT", "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], "->",
       "RTp"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["RL", "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "RLp"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       SuperscriptBox["R", "\[Prime]",
        MultilineFunction->None], "[", 
       RowBox[{"\[Xi]", "[", 
        RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], "]"}], 
      "\[Rule]", "Rp"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ux", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      "\[Rule]", "Ux"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"uy", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      "\[Rule]", "Uy"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[CapitalLambda]", "[", 
       RowBox[{"\[Tau]", ",", "\[Eta]", ",", "x", ",", "y"}], "]"}], 
      "\[Rule]", " ", "L"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DefsDerivs", "=", " ", 
   RowBox[{"{", "\n", 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["ux", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUxDe"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["ux", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUxDx"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["ux", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUxDy"}], ",", " ", "\[IndentingNewLine]", 
     "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["uy", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUyDe"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["uy", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUyDx"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["uy", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DUyDy"}], ",", " ", "\[IndentingNewLine]", 
     "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalLambda]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DLDe"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalLambda]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DLDx"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalLambda]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DLDy"}], ",", " ", "\[IndentingNewLine]", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DXDe"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DXDx"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Xi]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DXDy"}], ",", " ", "\[IndentingNewLine]", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalTheta]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DTDe"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalTheta]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "1", ",", "0"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DTDx"}], ",", " ", "\n", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[CapitalTheta]", 
        TagBox[
         RowBox[{"(", 
          RowBox[{"0", ",", "0", ",", "0", ",", "1"}], ")"}],
         Derivative],
        MultilineFunction->None], "[", 
       RowBox[{"\[Tau]", ",", " ", "\[Eta]", ",", " ", "x", ",", " ", "y"}], 
       "]"}], " ", "->", " ", "DTDy"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"PT", ",", "PL"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DefsFunc", "=", " ", 
   RowBox[{"{", "\n", 
    RowBox[{
     RowBox[{
      RowBox[{"Sinh", "[", "Y", "]"}], "\[Rule]", " ", "Sh"}], ",", "\n", 
     RowBox[{
      RowBox[{"Cosh", "[", "Y", "]"}], "\[Rule]", " ", "Ch"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"En", " ", "R"}], "\[Rule]", " ", "EE"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"P", " ", "RT"}], "\[Rule]", " ", "PT"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"P", " ", "RL"}], "\[Rule]", " ", "PL"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["U0", "2"], "\[Rule]", "U02"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Ux", "2"], "\[Rule]", " ", "Ux2"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Ux", "3"], "\[Rule]", " ", "Ux3"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Ux", "4"], "\[Rule]", " ", "Ux4"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Uy", "2"], "\[Rule]", " ", "Uy2"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Uy", "3"], "\[Rule]", " ", "Uy3"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      SuperscriptBox["Uy", "4"], "\[Rule]", " ", "Uy4"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"\[CapitalLambda]", "\[Rule]", " ", "L"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"\[Xi]", "\[Rule]", " ", "X"}], ",", "\[IndentingNewLine]", 
     RowBox[{"\[Tau]", "\[Rule]", " ", "t"}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10005.},
 CellChangeTimes->{{3.596366423803344*^9, 3.59636642929171*^9}, {
   3.596370297788629*^9, 3.596370317199441*^9}, {3.596370581019341*^9, 
   3.596370633674273*^9}, {3.596370717555862*^9, 3.596370779002513*^9}, {
   3.5963733700910263`*^9, 3.5963734133531723`*^9}, {3.5963736311892967`*^9, 
   3.596373637357957*^9}, 3.596373692101616*^9, {3.596373798924842*^9, 
   3.596373813681273*^9}, {3.59637401594415*^9, 3.596374033020299*^9}, {
   3.5969936210909395`*^9, 3.5969936579430366`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CoefTs", "=", 
   RowBox[{
    RowBox[{"CoefTau", "/.", "Defs"}], "/.", "DefsFunc"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoefSs", "=", 
   RowBox[{
    RowBox[{"CoefSpace", "/.", "Defs"}], "/.", "DefsFunc"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubsT", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CoefTs", "\[LeftDoubleBracket]", 
       RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}], "->", 
      RowBox[{"CoefTs", "\[LeftDoubleBracket]", 
       RowBox[{"i", ",", "2"}], "\[RightDoubleBracket]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "CoefTs", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubsS", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CoefSs", "\[LeftDoubleBracket]", 
       RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}], "->", 
      RowBox[{"CoefSs", "\[LeftDoubleBracket]", 
       RowBox[{"i", ",", "2"}], "\[RightDoubleBracket]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "CoefSs", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubsT", "//", "MatrixForm"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubsS", "//", "MatrixForm"}], ";"}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10005.},
 CellChangeTimes->{{3.596369160622403*^9, 3.5963692735790167`*^9}, {
  3.5963697192028637`*^9, 3.5963697418188143`*^9}, {3.5963734520822563`*^9, 
  3.596373503692782*^9}, {3.596993699158105*^9, 3.5969937615903*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "(* Derive the expressions for coefficients ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", 
    RowBox[{"ij", " "}]], TraditionalForm]],
  FormatType->"TraditionalForm"],
 "*)"
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, {3.5959407095943537`*^9, 3.595940717144854*^9}, 
   3.59594203222393*^9, {3.595942542180497*^9, 3.5959425683564987`*^9}, 
   3.596370253355792*^9, 3.5968388106256423`*^9, {3.596839248596693*^9, 
   3.5968392674397707`*^9}, {3.59684105577628*^9, 3.59684106822189*^9}, 
   3.596841327598268*^9, {3.598049455553763*^9, 3.598049462963853*^9}},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[
 RowBox[{"Do", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Print", "[", 
    RowBox[{"\"\<double \>\"", "<>", 
     RowBox[{"ToString", "[", 
      RowBox[{"CoefTs", "\[LeftDoubleBracket]", 
       RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], "<>", 
     "\"\<;\>\""}], "]"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "CoefTs", "]"}]}], "}"}]}], "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.596375264579802*^9, 3.596375285865349*^9}, {
   3.596375926773019*^9, 3.5963759282764053`*^9}, 3.596993796972367*^9}],

Cell[BoxData["\<\"double CLt1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938107793884`*^9}}],

Cell[BoxData["\<\"double CXt1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938108003883`*^9}}],

Cell[BoxData["\<\"double CUxt1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938108173895`*^9}}],

Cell[BoxData["\<\"double CUyt1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938108353906`*^9}}],

Cell[BoxData["\<\"double CTt1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993810851392*^9}}],

Cell[BoxData["\<\"double CLt2;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938108613915`*^9}}],

Cell[BoxData["\<\"double CXt2;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938108813915`*^9}}],

Cell[BoxData["\<\"double CUxt2;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938109013915`*^9}}],

Cell[BoxData["\<\"double CUyt2;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938109113913`*^9}}],

Cell[BoxData["\<\"double CTt2;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938109313917`*^9}}],

Cell[BoxData["\<\"double CLt3;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938109513917`*^9}}],

Cell[BoxData["\<\"double CXt3;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938109713917`*^9}}],

Cell[BoxData["\<\"double CUxt3;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993810981392*^9}}],

Cell[BoxData["\<\"double CUyt3;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938110013914`*^9}}],

Cell[BoxData["\<\"double CTt3;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811021392*^9}}],

Cell[BoxData["\<\"double CLt4;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938110313916`*^9}}],

Cell[BoxData["\<\"double CXt4;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938110513916`*^9}}],

Cell[BoxData["\<\"double CUxt4;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811071392*^9}}],

Cell[BoxData["\<\"double CUyt4;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811081392*^9}}],

Cell[BoxData["\<\"double CTt4;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811101392*^9}}],

Cell[BoxData["\<\"double CLt5;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811121392*^9}}],

Cell[BoxData["\<\"double CXt5;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811131392*^9}}],

Cell[BoxData["\<\"double CUxt5;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811151392*^9}}],

Cell[BoxData["\<\"double CUyt5;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.596993811171392*^9}}],

Cell[BoxData["\<\"double CTt5;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{
  3.596375289695278*^9, 3.596375737479435*^9, 3.596375810721518*^9, 
   3.596375929008095*^9, 3.5964566444953117`*^9, {3.596993796972367*^9, 
   3.5969938111813917`*^9}}],

Cell[BoxData[
 RowBox[{"Do", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{
      RowBox[{"ToString", "[", 
       RowBox[{"CoefTs", "\[LeftDoubleBracket]", 
        RowBox[{"i", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], "<>", 
      "\"\< = \>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"CForm", "[", 
        RowBox[{"CoefTs", "\[LeftDoubleBracket]", 
         RowBox[{"i", ",", "2"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
      "<>", "\"\<;\>\""}], "]"}], ";"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "CoefTs", "]"}]}], "}"}]}], "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.596374314677517*^9, 3.5963743476953707`*^9}, {
   3.5963745734848824`*^9, 3.596374580433751*^9}, {3.596374683276505*^9, 
   3.596374865938485*^9}, 3.5963749080556993`*^9, {3.596374971656151*^9, 
   3.596375017885085*^9}, {3.5963750789123163`*^9, 3.596375142236709*^9}, {
   3.5963752236382732`*^9, 3.596375226010309*^9}, {3.59637531997885*^9, 
   3.596375361549968*^9}, {3.596375487541543*^9, 3.5963755121057043`*^9}, {
   3.5963758936414537`*^9, 3.596375897975174*^9}, 3.5969937994023705`*^9}],

Cell[BoxData["\<\"CLt1 = (-6*Ch*U0)/L;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938112913923`*^9}}],

Cell[BoxData["\<\"CXt1 = (Ch*U0)/X1;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811311392*^9}}],

Cell[BoxData["\<\"CUxt1 = (-2*Ch*Ux)/U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938113313923`*^9}}],

Cell[BoxData["\<\"CUyt1 = (-2*Ch*Uy)/U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938113513923`*^9}}],

Cell[BoxData["\<\"CTt1 = 2*Sh*U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938113713923`*^9}}],

Cell[BoxData["\<\"CLt2 = Ch*Enp*R*U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811391392*^9}}],

Cell[BoxData["\<\"CXt2 = Ch*En*Rp*U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811411392*^9}}],

Cell[BoxData["\<\"CUxt2 = (Ch*(EE + PT)*Ux)/U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938114313927`*^9}}],

Cell[BoxData["\<\"CUyt2 = (Ch*(EE + PT)*Uy)/U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811451392*^9}}],

Cell[BoxData["\<\"CTt2 = -((EE + PL)*Sh*U0);\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938114713926`*^9}}],

Cell[BoxData["\<\"CLt3 = -(Pp*RL*Sh);\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811491392*^9}}],

Cell[BoxData["\<\"CXt3 = -(P*RLp*Sh);\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938115113926`*^9}}],

Cell[BoxData["\<\"CUxt3 = 0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811531392*^9}}],

Cell[BoxData["\<\"CUyt3 = 0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938115513926`*^9}}],

Cell[BoxData["\<\"CTt3 = Ch*(EE*Power(U0,2) + P*(RL + RT*(-1 + U02)));\"\>"], \
"Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811571392*^9}}],

Cell[BoxData["\<\"CLt4 = Ch*Pp*RT*U0*Ux;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938115913925`*^9}}],

Cell[BoxData["\<\"CXt4 = Ch*P*RTp*U0*Ux;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938116113925`*^9}}],

Cell[BoxData["\<\"CUxt4 = Ch*(EE + PT)*U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938116313925`*^9}}],

Cell[BoxData["\<\"CUyt4 = 0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811651393*^9}}],

Cell[BoxData["\<\"CTt4 = P*(RL - RT)*Sh*U0*Ux;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.5969938116713924`*^9}}],

Cell[BoxData["\<\"CLt5 = Ch*Pp*RT*U0*Uy;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811691393*^9}}],

Cell[BoxData["\<\"CXt5 = Ch*P*RTp*U0*Uy;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811711393*^9}}],

Cell[BoxData["\<\"CUxt5 = 0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811731393*^9}}],

Cell[BoxData["\<\"CUyt5 = Ch*(EE + PT)*U0;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811751393*^9}}],

Cell[BoxData["\<\"CTt5 = P*(RL - RT)*Sh*U0*Uy;\"\>"], "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.},
 CellChangeTimes->{{3.59637550064615*^9, 3.596375512578444*^9}, 
   3.596375707565362*^9, 3.596375756931829*^9, 3.596375898891622*^9, 
   3.596456647950508*^9, {3.5969937994023705`*^9, 3.596993811771393*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "(* Derive the expressions for reminders ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "j"], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " *)"
}], "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{{3.594743737188568*^9, 3.5947437491147223`*^9}, 
   3.594743782658474*^9, {3.595938257947772*^9, 3.5959382610095043`*^9}, {
   3.595940524370751*^9, 3.5959405476341543`*^9}, {3.595940598587166*^9, 
   3.5959406041455803`*^9}, {3.5959407095943537`*^9, 3.595940717144854*^9}, 
   3.59594203222393*^9, {3.595942542180497*^9, 3.5959425683564987`*^9}, 
   3.596370253355792*^9, 3.5968388106256423`*^9, {3.596839248596693*^9, 
   3.5968392674397707`*^9}, {3.59684105577628*^9, 3.59684106822189*^9}, {
   3.596841327598268*^9, 3.596841337563171*^9}, {3.596841665249427*^9, 
   3.59684168096058*^9}, {3.596984200455063*^9, 3.5969842006270723`*^9}, {
   3.598049351512931*^9, 3.598049362043014*^9}},
 FontWeight->"Bold",
 Background->RGBColor[0.88, 1, 0.88]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RemArrs", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], "/.", "SubsS"}], "/.",
     "DefsDerivs"}], "/.", "DefsFunc"}], " ", "//", "CForm"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{{3.596370140439816*^9, 3.596370142735903*^9}, {
   3.596370177561578*^9, 3.596370196947136*^9}, {3.596370333506147*^9, 
   3.596370339023266*^9}, {3.5963703709933033`*^9, 3.596370427387369*^9}, {
   3.596370546487111*^9, 3.596370555560196*^9}, {3.596370640396975*^9, 
   3.5963706764322853`*^9}, {3.596370812149016*^9, 3.5963708592582073`*^9}, {
   3.596373245101542*^9, 3.596373248193527*^9}, {3.5963735568681173`*^9, 
   3.596373618748591*^9}, {3.596373714252359*^9, 3.5963737442022142`*^9}, 
   3.59637384521185*^9, 3.596993865139493*^9}],

Cell["\<\
-2*DUxDx - 2*DUyDy - (2*Ch*DTDe*U0)/t + (6*DLDe*Sh*U0)/(L*t) - (6*DLDx*Ux)/L \
+ (2*DUxDe*Sh*Ux)/(t*U0) - (6*DLDy*Uy)/L + (2*DUyDe*Sh*Uy)/(t*U0) - \
(DXDe*Sh*U0)/(t*X1) + (DXDx*Ux)/X1 + (DXDy*Uy)/X1\
\>", "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{
  3.5963701432026*^9, {3.596370180980503*^9, 3.596370197959014*^9}, {
   3.5963703518396797`*^9, 3.59637039429102*^9}, 3.5963704318554363`*^9, {
   3.596370547004118*^9, 3.596370556040914*^9}, {3.596370652190195*^9, 
   3.5963706772381973`*^9}, 3.596370785861484*^9, {3.5963708180267553`*^9, 
   3.596370859933608*^9}, 3.596373044235469*^9, 3.5963732487346907`*^9, 
   3.596373518505602*^9, {3.596373559771109*^9, 3.596373660429759*^9}, {
   3.596373698324697*^9, 3.596373744913742*^9}, 3.596373837250874*^9, 
   3.5964566560427732`*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RemArrs", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "2"}], "\[RightDoubleBracket]"}], "/.", "SubsS"}], "/.",
     "DefsDerivs"}], "/.", "DefsFunc"}], " ", "//", "CForm"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596373883632389*^9, 3.596993865139493*^9}],

Cell["\<\
DUxDx*(EE + PT) + DUyDy*(EE + PT) + (Ch*DTDe*(EE + PL)*U0)/t - \
(DLDe*Enp*R*Sh*U0)/t - (DXDe*En*Rp*Sh*U0)/t + DLDx*Enp*R*Ux + DXDx*En*Rp*Ux - \
(DUxDe*(EE + PT)*Sh*Ux)/(t*U0) + DLDy*Enp*R*Uy + DXDy*En*Rp*Uy - 
   (DUyDe*(EE + PT)*Sh*Uy)/(t*U0)\
\>", "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.59637388455558*^9, 3.5964566573796997`*^9, 
  3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RemArrs", "\[LeftDoubleBracket]", 
      RowBox[{"3", ",", "2"}], "\[RightDoubleBracket]"}], "/.", "SubsS"}], "/.",
     "DefsDerivs"}], "/.", "DefsFunc"}], " ", "//", "CForm"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{{3.596373940976264*^9, 3.5963739423923264`*^9}, 
   3.596993865139493*^9}],

Cell["\<\
(Ch*DLDe*Pp*RL)/t + (Ch*DXDe*P*RLp)/t - (DTDe*Sh*(P*(RL + RT*(-1 + U02)) + \
EE*U02))/t + DTDx*(EE + PT)*U0*Ux + DTDy*(EE + PT)*U0*Uy\
\>", "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596373943250072*^9, 3.5963740403701067`*^9, 
  3.596456658954157*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RemArrs", "\[LeftDoubleBracket]", 
      RowBox[{"4", ",", "2"}], "\[RightDoubleBracket]"}], "/.", "SubsS"}], "/.",
     "DefsDerivs"}], "/.", "DefsFunc"}], " ", "//", "CForm"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.59637397211229*^9, 3.596993865139493*^9}],

Cell["\<\
-((DUxDe*(EE + PT)*Sh*U0)/t) + DUxDx*(EE + PT)*Ux + (Ch*DTDe*P*(-RL + \
RT)*U0*Ux)/t - (DLDe*Pp*RT*Sh*U0*Ux)/t - (DXDe*P*RTp*Sh*U0*Ux)/t + \
DLDx*Pp*RT*(1 + Ux2) + DXDx*P*RTp*(1 + Ux2) + DUxDy*(EE + PT)*Uy + 
   DLDy*Pp*RT*Ux*Uy + DXDy*P*RTp*Ux*Uy\
\>", "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596373972801937*^9, 3.596374043273987*^9, 
  3.596456660467124*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"RemArrs", "\[LeftDoubleBracket]", 
      RowBox[{"5", ",", "2"}], "\[RightDoubleBracket]"}], "/.", "SubsS"}], "/.",
     "DefsDerivs"}], "/.", "DefsFunc"}], " ", "//", "CForm"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.5963740685687237`*^9, 3.596993865139493*^9}],

Cell["\<\
-((DUyDe*(EE + PT)*Sh*U0)/t) + DUyDx*(EE + PT)*Ux + DUyDy*(EE + PT)*Uy + \
(Ch*DTDe*P*(-RL + RT)*U0*Uy)/t - (DLDe*Pp*RT*Sh*U0*Uy)/t - \
(DXDe*P*RTp*Sh*U0*Uy)/t + DLDx*Pp*RT*Ux*Uy + DXDx*P*RTp*Ux*Uy + 
   DLDy*Pp*RT*(1 + Uy2) + DXDy*P*RTp*(1 + Uy2)\
\>", "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.59637406923413*^9, 3.5964566615443087`*^9, 
  3.596993865139493*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
     "RemArrs", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "/.", 
     "SubsS"}], "/.", "DefsDerivs"}], "/.", "DefsFunc"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
     "RemArrs", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], "/.", 
     "SubsS"}], "/.", "DefsDerivs"}], "/.", "DefsFunc"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
     "RemArrs", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], "/.", 
     "SubsS"}], "/.", "DefsDerivs"}], "/.", "DefsFunc"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RemArrs", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}],
     "/.", "SubsS"}], "/.", "DefsDerivs"}], "/.", "DefsFunc", " "}]}], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"R2", ",", 
   RowBox[{
    RowBox[{"DUxDx", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}]}], "+", 
    RowBox[{"DUyDy", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}]}], "+", 
    FractionBox[
     RowBox[{"Ch", " ", "DTDe", " ", 
      RowBox[{"(", 
       RowBox[{"EE", "+", "PL"}], ")"}], " ", "U0"}], "t"], "-", 
    FractionBox[
     RowBox[{"DLDe", " ", "Enp", " ", "R", " ", "Sh", " ", "U0"}], "t"], "-", 
    FractionBox[
     RowBox[{"DXDe", " ", "En", " ", "Rp", " ", "Sh", " ", "U0"}], "t"], "+", 
    RowBox[{"DLDx", " ", "Enp", " ", "R", " ", "Ux"}], "+", 
    RowBox[{"DXDx", " ", "En", " ", "Rp", " ", "Ux"}], "-", 
    FractionBox[
     RowBox[{"DUxDe", " ", 
      RowBox[{"(", 
       RowBox[{"EE", "+", "PT"}], ")"}], " ", "Sh", " ", "Ux"}], 
     RowBox[{"t", " ", "U0"}]], "+", 
    RowBox[{"DLDy", " ", "Enp", " ", "R", " ", "Uy"}], "+", 
    RowBox[{"DXDy", " ", "En", " ", "Rp", " ", "Uy"}], "-", 
    FractionBox[
     RowBox[{"DUyDe", " ", 
      RowBox[{"(", 
       RowBox[{"EE", "+", "PT"}], ")"}], " ", "Sh", " ", "Uy"}], 
     RowBox[{"t", " ", "U0"}]]}]}], "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.5964566629118357`*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"R3", ",", 
   RowBox[{
    FractionBox[
     RowBox[{"Ch", " ", "DLDe", " ", "Pp", " ", "RL"}], "t"], "+", 
    FractionBox[
     RowBox[{"Ch", " ", "DXDe", " ", "P", " ", "RLp"}], "t"], "-", 
    FractionBox[
     RowBox[{"DTDe", " ", "Sh", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"P", " ", 
         RowBox[{"(", 
          RowBox[{"RL", "+", 
           RowBox[{"RT", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", "U02"}], ")"}]}]}], ")"}]}], "+", 
        RowBox[{"EE", " ", "U02"}]}], ")"}]}], "t"], "+", 
    RowBox[{"DTDx", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "U0", " ", "Ux"}], "+", 
    RowBox[{"DTDy", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "U0", " ", "Uy"}]}]}], 
  "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.5964566629155693`*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"R4", ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"DUxDe", " ", 
       RowBox[{"(", 
        RowBox[{"EE", "+", "PT"}], ")"}], " ", "Sh", " ", "U0"}], "t"]}], "+", 
    RowBox[{"DUxDx", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "Ux"}], "+", 
    FractionBox[
     RowBox[{"Ch", " ", "DTDe", " ", "P", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "RL"}], "+", "RT"}], ")"}], " ", "U0", " ", "Ux"}], "t"],
     "-", 
    FractionBox[
     RowBox[{"DLDe", " ", "Pp", " ", "RT", " ", "Sh", " ", "U0", " ", "Ux"}], 
     "t"], "-", 
    FractionBox[
     RowBox[{"DXDe", " ", "P", " ", "RTp", " ", "Sh", " ", "U0", " ", "Ux"}], 
     "t"], "+", 
    RowBox[{"DLDx", " ", "Pp", " ", "RT", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", "Ux2"}], ")"}]}], "+", 
    RowBox[{"DXDx", " ", "P", " ", "RTp", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", "Ux2"}], ")"}]}], "+", 
    RowBox[{"DUxDy", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "Uy"}], "+", 
    RowBox[{"DLDy", " ", "Pp", " ", "RT", " ", "Ux", " ", "Uy"}], "+", 
    RowBox[{"DXDy", " ", "P", " ", "RTp", " ", "Ux", " ", "Uy"}]}]}], 
  "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596456662920527*^9, 3.596993865139493*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"R5", ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"DUyDe", " ", 
       RowBox[{"(", 
        RowBox[{"EE", "+", "PT"}], ")"}], " ", "Sh", " ", "U0"}], "t"]}], "+", 
    RowBox[{"DUyDx", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "Ux"}], "+", 
    RowBox[{"DUyDy", " ", 
     RowBox[{"(", 
      RowBox[{"EE", "+", "PT"}], ")"}], " ", "Uy"}], "+", 
    FractionBox[
     RowBox[{"Ch", " ", "DTDe", " ", "P", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "RL"}], "+", "RT"}], ")"}], " ", "U0", " ", "Uy"}], "t"],
     "-", 
    FractionBox[
     RowBox[{"DLDe", " ", "Pp", " ", "RT", " ", "Sh", " ", "U0", " ", "Uy"}], 
     "t"], "-", 
    FractionBox[
     RowBox[{"DXDe", " ", "P", " ", "RTp", " ", "Sh", " ", "U0", " ", "Uy"}], 
     "t"], "+", 
    RowBox[{"DLDx", " ", "Pp", " ", "RT", " ", "Ux", " ", "Uy"}], "+", 
    RowBox[{"DXDx", " ", "P", " ", "RTp", " ", "Ux", " ", "Uy"}], "+", 
    RowBox[{"DLDy", " ", "Pp", " ", "RT", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", "Uy2"}], ")"}]}], "+", 
    RowBox[{"DXDy", " ", "P", " ", "RTp", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", "Uy2"}], ")"}]}]}]}], "}"}]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.},
 CellChangeTimes->{3.596456662925411*^9, 3.596993865139493*^9}]
}, Closed]]
},
WindowSize->{1301, 744},
WindowMargins->{{Automatic, 0}, {0, Automatic}},
FrontEndVersion->"7.0 for Linux x86 (64-bit) (February 25, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "iso8859-1"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[400, 13, 1783, 53, 238, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10000.}],
Cell[CellGroupData[{
Cell[2208, 70, 311, 6, 48, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.}],
Cell[2522, 78, 958, 23, 284, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.}],
Cell[3483, 103, 9117, 254, 508, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10002.}]
}, Closed]],
Cell[CellGroupData[{
Cell[12637, 362, 345, 6, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10001.}],
Cell[12985, 370, 2645, 60, 109, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10001.}]
}, Closed]],
Cell[CellGroupData[{
Cell[15667, 435, 423, 7, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10002.}],
Cell[16093, 444, 9979, 255, 379, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10002.}]
}, Closed]],
Cell[CellGroupData[{
Cell[26109, 704, 517, 8, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[26629, 714, 974, 29, 74, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[27606, 745, 14403, 380, 527, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[42012, 1127, 11067, 243, 1077, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[53082, 1372, 881, 23, 38, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[53966, 1397, 337, 6, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[54306, 1405, 337, 6, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[54646, 1413, 337, 6, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[54986, 1421, 337, 6, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[55326, 1429, 862, 14, 41, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[56191, 1445, 222, 3, 65, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[56416, 1450, 1492, 34, 62, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[57911, 1486, 448, 10, 41, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[58362, 1498, 1108, 28, 153, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[59473, 1528, 1759, 51, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[61235, 1581, 1759, 51, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[62997, 1634, 1189, 33, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[64189, 1669, 1474, 42, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[65666, 1713, 1474, 42, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[67143, 1757, 479, 9, 65, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}],
Cell[67625, 1768, 16804, 453, 132, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10003.}]
}, Closed]],
Cell[CellGroupData[{
Cell[84466, 2226, 828, 19, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[85297, 2247, 3242, 75, 417, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[88542, 2324, 407, 11, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[88952, 2337, 407, 11, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[89362, 2350, 421, 11, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[89786, 2363, 410, 11, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[90199, 2376, 420, 11, 43, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[90622, 2389, 813, 14, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[91438, 2405, 9713, 190, 422, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[101154, 2597, 403, 9, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[101560, 2608, 9713, 190, 422, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[111276, 2800, 372, 8, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[111651, 2810, 7612, 166, 322, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[119266, 2978, 568, 11, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[119837, 2991, 7556, 193, 70, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[127396, 3186, 370, 8, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}],
Cell[127769, 3196, 5286, 141, 70, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10004.}]
}, Closed]],
Cell[CellGroupData[{
Cell[133092, 3342, 715, 12, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10005.}],
Cell[133810, 3356, 13165, 348, 1426, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10005.}],
Cell[146978, 3706, 1670, 44, 153, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10005.}]
}, Closed]],
Cell[CellGroupData[{
Cell[148685, 3755, 905, 19, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[149593, 3776, 657, 14, 87, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[150253, 3792, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[150546, 3799, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[150839, 3806, 291, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[151133, 3813, 291, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[151427, 3820, 288, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[151718, 3827, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[152011, 3834, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[152304, 3841, 291, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[152598, 3848, 291, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[152892, 3855, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[153185, 3862, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[153478, 3869, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[153771, 3876, 289, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[154063, 3883, 291, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[154357, 3890, 288, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[154648, 3897, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[154941, 3904, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[155234, 3911, 289, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[155526, 3918, 289, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[155818, 3925, 288, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[156109, 3932, 288, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[156400, 3939, 288, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[156691, 3946, 289, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[156983, 3953, 289, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[157275, 3960, 290, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[157568, 3967, 1257, 25, 87, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[158828, 3994, 318, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[159149, 4000, 314, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[159466, 4006, 320, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[159789, 4012, 320, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[160112, 4018, 313, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[160428, 4024, 315, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[160746, 4030, 315, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[161064, 4036, 327, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[161394, 4042, 325, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[161722, 4048, 324, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[162049, 4054, 315, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[162367, 4060, 317, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[162687, 4066, 306, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[162996, 4072, 308, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[163307, 4078, 350, 5, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[163660, 4085, 320, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[163983, 4091, 320, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[164306, 4097, 322, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[164631, 4103, 306, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[164940, 4109, 326, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[165269, 4115, 318, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[165590, 4121, 318, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[165911, 4127, 306, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[166220, 4133, 320, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}],
Cell[166543, 4139, 324, 4, 24, "Print",
 CellGroupingRules->{GroupTogetherGrouping, 10006.}]
}, Closed]],
Cell[CellGroupData[{
Cell[166904, 4148, 1007, 20, 45, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[167914, 4170, 876, 16, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[168793, 4188, 857, 14, 41, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[169653, 4204, 379, 9, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[170035, 4215, 413, 8, 58, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[170451, 4225, 409, 10, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[170863, 4237, 325, 6, 41, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[171191, 4245, 378, 9, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[171572, 4256, 437, 8, 58, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[172012, 4266, 381, 9, 42, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[172396, 4277, 416, 8, 58, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[172815, 4287, 994, 31, 109, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[173812, 4320, 1300, 33, 95, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[175115, 4355, 983, 28, 58, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[176101, 4385, 1365, 37, 95, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}],
Cell[177469, 4424, 1362, 36, 95, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10007.}]
}, Closed]]
}
]
*)

